Package: methimpute
Type: Package
Title: Imputation-guided re-construction of complete methylomes from
        WGBS data
Version: 1.34.0
Author: Aaron Taudt
Maintainer: Aaron Taudt <aaron.taudt@gmail.com>
Description: This package implements functions for calling methylation for all cytosines in the genome.
Depends: R (>= 3.5.0), GenomicRanges, ggplot2
Imports: Rcpp (>= 0.12.4.5), methods, utils, grDevices, stats,
        GenomeInfoDb, IRanges, Biostrings, reshape2, minpack.lm,
        data.table
Suggests: knitr, BiocStyle
LinkingTo: Rcpp
License: Artistic-2.0
LazyLoad: yes
VignetteBuilder: knitr
RoxygenNote: 6.1.1
biocViews: ImmunoOncology, Software, DNAMethylation, Epigenetics,
        HiddenMarkovModel, Sequencing, Coverage
git_url: https://git.bioconductor.org/packages/methimpute
git_branch: RELEASE_3_23
git_last_commit: 5c8ef59
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 23:10:59 UTC; biocbuild

# Generated by roxygen2: do not edit by hand

S3method(print,methimputeBinomialHMM)
export(binCounts)
export(binMethylome)
export(binPositions)
export(binomialTestMethylation)
export(callMethylation)
export(callMethylationSeparate)
export(collapseBins)
export(distanceCorrelation)
export(estimateTransDist)
export(exportMethylome)
export(extractCytosinesFromFASTA)
export(getDistinctColors)
export(getStateColors)
export(importBSMAP)
export(importBSSeeker)
export(importBismark)
export(importMethylpy)
export(inflateMethylome)
export(loadFromFiles)
export(plotConvergence)
export(plotEnrichment)
export(plotHistogram)
export(plotPosteriorDistance)
export(plotScatter)
export(plotTransitionProbs)
import(GenomeInfoDb)
import(GenomicRanges)
import(IRanges)
import(Rcpp)
import(ggplot2)
importFrom(Biostrings,readDNAStringSet)
importFrom(Biostrings,reverseComplement)
importFrom(Biostrings,vcountPattern)
importFrom(Biostrings,vmatchPattern)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(grDevices,col2rgb)
importFrom(grDevices,rgb2hsv)
importFrom(methods,as)
importFrom(methods,is)
importFrom(minpack.lm,nlsLM)
importFrom(reshape2,melt)
importFrom(stats,aggregate)
importFrom(stats,coefficients)
importFrom(stats,dbinom)
importFrom(stats,dist)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,pbinom)
importFrom(utils,read.table)
useDynLib(methimpute)

METHimpute
==========

Methimpute implements a powerful HMM-based binomial test for methylation status calling. Besides improved accuracy over the classical binomial test, the HMM allows imputation of the methylation status of **all cytosines** in the genome. It achieves this by borrowing information from neighboring covered cytosines. The confidence in the methylation status call is reported as well. Methimpute also outputs context-specific conversion rates, which might be used to optimize the experimental procedure.

For the exact workings of methimpute we refer the interested reader to our publication TODO. A tutorial is available [here](https://github.com/ataudt/methimpute/blob/master/vignettes/methimpute.pdf).

Installation
------------

### Development version from Github
To install the *development* version from Github, follow the steps given below. The installation has only been tested on Ubuntu so far, if you need to install on Windows or Mac additional steps might be necessary (e.g. installation of Rtools from https://cran.r-project.org/bin/windows/Rtools/)

1. Install a recent version of R (>=3.3.0) from https://www.r-project.org/
2. Optional: For ease of use, install Rstudio from https://www.rstudio.com/
3. Open R and install all dependencies. Please ensure that you have writing permissions to install packages. Execute the following lines one by one:

   install.packages("devtools")  
	 if (!requireNamespace("BiocManager", quietly=TRUE))
    	 install.packages("BiocManager")
	 BiocManager::install(c("GenomicRanges"))  
	 library(devtools)  
	 install_github("ataudt/methimpute")  

How to use METHimpute
---------------------

Please refer to the [vignette](https://github.com/ataudt/methimpute/blob/master/vignettes/methimpute.pdf) for a tutorial.

Report Errors
-------------

If you encounter errors of any kind, please file an [issue here](https://github.com/ataudt/methimpute/issues/new). I will try to react within one day.

#' methIMPUTE: Imputation-guided methylation status calling for WGBS-seq
#'
#' \pkg{methimpute} is an R-package for methylation status calling in Whole-Genome Bisulfite-sequencing (WGBS-seq) data. Its powerful Hidden Markov model implementation enables imputation of methylation status calls for cytosines without any coverage.
#'
#' Please read the vignette for a tutorial on how to use this package. You can do this by typing \code{browseVignettes("methimpute")}. Here is an overview of all \code{\link{plotting}} functions.
#'
#' @author Aaron Taudt
#' @docType package
#' @name methimpute-package
#' @aliases methimpute
NULL

check.positive.integer = function(testvar) {
    if (!is(testvar,"numeric") & !is(testvar,"integer")) return(1)
    if (length(testvar)>1) return(2)
    if (testvar <= 0) return(3)
    if (testvar != as.integer(testvar)) return(4)
    return(0)
}
check.positive.integer.vector = function(testvec) {
    if (!is(testvec,"numeric") & !is(testvec,"integer")) return(1)
    for (elem in testvec) {
        if (elem <= 0) return(2)
        if (elem != as.integer(elem)) return(3)
    }
    return(0)
}
check.nonnegative.integer.vector = function(testvec) {
    if (!is(testvec,"numeric") & !is(testvec,"integer")) return(1)
    for (elem in testvec) {
        if (elem < 0) return(2)
        if (elem != as.integer(elem)) return(3)
    }
    return(0)
}
check.positive = function(testvar) {
    if (!is(testvar,"numeric") & !is(testvar,"integer")) return(1)
    if (length(testvar)>1) return(2)
    if (testvar <= 0) return(3)
    return(0)
}
check.positive.vector = function(testvec) {
    if (!is(testvec,"numeric") & !is(testvec,"integer")) return(1)
    for (elem in testvec) {
        if (elem <= 0) return(2)
    }
    return(0)
}
check.nonnegative.vector = function(testvec) {
    if (!is(testvec,"numeric") & !is(testvec,"integer")) return(1)
    for (elem in testvec) {
        if (elem < 0) return(2)
    }
    return(0)
}
check.integer = function(testvar) {
    if (!is(testvar,"numeric") & !is(testvar,"integer")) return(1)
    if (length(testvar)>1) return(2)
    if (testvar != as.integer(testvar)) return(3)
    return(0)
}
check.logical = function(testbool) {
    if (!is(testbool,"logical")) return(1)
    if (length(testbool)>1) return(2)
    return(0)
}
check.strand = function(teststrand) {
    if (teststrand!='+' & teststrand!='-' & teststrand!='*') return(1)
    return(0)
}

#' Perform a parameter scan
#' 
#' Perform a parameter scan for an arbitrary parameter.
#' 
#' @param f A function for which to perform the scan.
#' @param param A character with the parameter for which to perform the scan.
#' @param values A vector with parameter values for which to perform the scan.
#' @param ... Other parameters passed through to \code{f}.
#' @return A data.frame with loglikelihood values.
parameterScan <- function(f, param, values, ...) {

    arglist <- list(values[1], ...)
    names(arglist)[1] <- param
    result <- list()
    for (val in values) {
        arglist[1] <- val
        model <- do.call(f, arglist)
        result[[as.character(val)]] <- data.frame(val, loglik=model$convergenceInfo$logliks[length(model$convergenceInfo$logliks)])
    }
    result <- do.call(rbind, result)
    names(result)[1] <- param
    return(result)
    
}

# ============================================================================
# Functions for a Negative Binomial to transform (mean,variance)<->(size,prob)
# ============================================================================
dnbinom.size <- function(mean, var) {
    return(mean^2 / (var - mean))
}

dnbinom.prob <- function(mean, var) {
    return(mean/var)
}

dnbinom.mean <- function(size, prob) {
    return(size/prob - size)
}

dnbinom.var <- function(size, prob) {
    return( (size - prob*size) / prob^2 )
}

#' Export a methylome
#' 
#' Export a methylome as a TSV file.
#' 
#' @param model A \code{\link{methimputeBinomialHMM}} object.
#' @param filename The name of the file to be exported.
#' @return \code{NULL}
#' @importFrom data.table fwrite
#' @export
#' @examples
#'\dontrun{
#'## Get some toy data
#'file <- system.file("data","arabidopsis_toydata.RData", package="methimpute")
#'data <- get(load(file))
#'print(data)
#'model <- callMethylation(data, max.iter=10)
#'exportMethylome(model, filename = tempfile())
#'}
#'
exportMethylome <- function(model, filename) {

    ptm <- startTimedMessage("Writing to file ", filename, " ...")
    data <- model$data
    df <- methods::as(data, 'data.frame')
    df <- df[,c('seqnames', 'start', 'strand', 'context', 'counts.methylated', 'counts.total', 'posteriorMax', 'posteriorMeth', 'posteriorUnmeth', 'status','rc.meth.lvl')]
    data.table::fwrite(df,file = filename, quote = FALSE, sep = '\t', row.names = FALSE, col.names = TRUE)
    stopTimedMessage(ptm)
    
}

#' Print model object
#' 
#' @param x A \code{\link{methimputeBinomialHMM}} object.
#' @param ... Ignored.
#' @return An invisible \code{NULL}.
#' @export
print.methimputeBinomialHMM <- function(x, ...) {
    
    print(x$data)
    message("Use the list operator $ to access all elements of this object.")

}

#' Methimpute data import
#' 
#' This page provides an overview of all \pkg{\link{methimpute}} data import functions.
#' @param file The file to import.
#' @param chrom.lengths A data.frame with chromosome names in the first, and chromosome lengths in the second column. Only chromosomes named in here will be returned. Alternatively a tab-separated file with such a data.frame (with headers).
#' @param skip The number of lines to skip. Usually 1 if the file contains a header and 0 otherwise.
#' @param contexts A character vector of the contexts that are to be assigned. Since some programs report 5-letter contexts, this parameter can be used to obtain a reduced number of contexts. Will yield contexts CG, CHG, CHH by default. Set \code{contexts=NULL} to obtain all available contexts.
#' @return A \code{\link{methimputeData}} object.
#' @name import
#' @examples 
#'## Get an example file in BSSeeker format
#'file <- system.file("extdata","arabidopsis_bsseeker.txt.gz", package="methimpute")
#'data(arabidopsis_chromosomes)
#'bsseeker.data <- importBSSeeker(file, chrom.lengths=arabidopsis_chromosomes)
#'
#'## Get an example file in Bismark format
#'file <- system.file("extdata","arabidopsis_bismark.txt", package="methimpute")
#'data(arabidopsis_chromosomes)
#'arabidopsis_chromosomes$chromosome <- sub('chr', '', arabidopsis_chromosomes$chromosome)
#'bismark.data <- importBismark(file, chrom.lengths=arabidopsis_chromosomes)
#'
#'## Get an example file in BSMAP format
#'file <- system.file("extdata","arabidopsis_BSMAP.txt", package="methimpute")
#'data(arabidopsis_chromosomes)
#'bsmap.data <- importBSMAP(file, chrom.lengths=arabidopsis_chromosomes)
#'
#'## Get an example file in Methylpy format
#'file <- system.file("extdata","arabidopsis_methylpy.txt", package="methimpute")
#'data(arabidopsis_chromosomes)
#'arabidopsis_chromosomes$chromosome <- sub('chr', '', arabidopsis_chromosomes$chromosome)
#'methylpy.data <- importMethylpy(file, chrom.lengths=arabidopsis_chromosomes)
NULL


#' @describeIn import Import a BSMAP methylation extractor file.
#' @importFrom Biostrings readDNAStringSet vcountPattern reverseComplement
#' @importFrom data.table fread
#' @importFrom utils read.table
#' @export
#' 
importBSMAP <- function(file, chrom.lengths=NULL, skip=1, contexts=c(CG='NNCGN', CHG='NNCHG', CHH='NNCHH')) {
  
    ## Import data
    classes <- c('character', 'numeric', 'character', 'character', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric')
    ptm <- startTimedMessage("Reading file ", file, " ...")
    data.raw <- tryCatch({
        data.raw <- data.table::fread(file, skip=skip, sep='\t', colClasses=classes)
    }, error = function(err) {
        data.raw <- utils::read.table(file, skip=skip, sep='\t', colClasses=classes)
    })
    data <- GRanges(seqnames=data.raw$V1, ranges=IRanges(start=data.raw$V2, end=data.raw$V2), strand=data.raw$V3, context=factor(NA, levels=names(contexts)), context.full=data.raw$V4)
    counts <- array(NA, dim=c(length(data), 2), dimnames=list(NULL, c("methylated", "total")))
    counts[,"methylated"] <- data.raw$V7
    counts[,"total"] <- data.raw$V8
    data$counts <- counts
    rm(data.raw)
    stopTimedMessage(ptm)
    
    ## Rework contexts
    ptm <- startTimedMessage("Reworking contexts ...")
    if (is.null(contexts)) {
        data$context <- factor(data$context.full)
    } else {
        context.full <- Biostrings::DNAStringSet(data$context.full)
        # Reverse complement minus
        mask <- strand(data) == '-'
        context.full[mask] <- Biostrings::reverseComplement(x = context.full[mask])
        for (i1 in 1:length(contexts)) {
            ind <- which(Biostrings::vcountPattern(contexts[i1], subject = context.full, fixed = FALSE) == 1)
            data$context[ind] <- names(contexts)[i1]
        }
    }
    data$context.full <- NULL
    stopTimedMessage(ptm)
    
    
    ## Assign seqlengths
    if (!is.null(chrom.lengths)) {
        if (is.character(chrom.lengths)) {
            df <- tryCatch({
                df <- data.table::fread(chrom.lengths, header=TRUE)
            }, error = function(err) {
                df <- utils::read.table(chrom.lengths, header=TRUE)
            })
        } else if (is.data.frame(chrom.lengths)) {
            df <- chrom.lengths
        }
        chrom.lengths <- df[,2]
        names(chrom.lengths) <- df[,1]
        # Filter by chromosomes supplied in chrom.lengths
        data <- keepSeqlevels(data, seqlevels(data)[seqlevels(data) %in% names(chrom.lengths)])
        seqlengths(data) <- chrom.lengths[names(seqlengths(data))]
    }   
    
    return(data)
}


#' @describeIn import Import a Methylpy methylation extractor file.
#' @importFrom data.table fread
#' @importFrom utils read.table
#' @export
#'
importMethylpy <- function(file, chrom.lengths=NULL, skip=1, contexts=c(CG='CGN', CHG='CHG', CHH='CHH')) {
    
    ## Import data
    classes <- c('character', 'numeric', 'character', 'character', 'numeric', 'numeric', 'numeric')
    ptm <- startTimedMessage("Reading file ", file, " ...")
    data.raw <- tryCatch({
        data.raw <- data.table::fread(file, skip=skip, sep='\t', colClasses=classes)
    }, error = function(err) {
        data.raw <- utils::read.table(file, skip=skip, sep='\t', colClasses=classes)
    })
    data <- GRanges(seqnames=data.raw$V1, ranges=IRanges(start=data.raw$V2, end=data.raw$V2), strand=data.raw$V3, context=factor(NA, levels=names(contexts)), context.full=data.raw$V4)
    counts <- array(NA, dim=c(length(data), 2), dimnames=list(NULL, c("methylated", "total")))
    counts[,"methylated"] <- data.raw$V5
    counts[,"total"] <- data.raw$V6
    data$counts <- counts
    data$binomial.test <- data.raw$V7
    rm(data.raw)
    stopTimedMessage(ptm)
    
    ## Rework contexts
    ptm <- startTimedMessage("Reworking contexts ...")
    if (is.null(contexts)) {
        data$context <- factor(data$context.full)
    } else {
        context.full <- Biostrings::DNAStringSet(data$context.full)
        for (i1 in 1:length(contexts)) {
            ind <- which(Biostrings::vcountPattern(contexts[i1], subject = context.full, fixed = FALSE) == 1)
            data$context[ind] <- names(contexts)[i1]
        }
    }
    data$context.full <- NULL
    stopTimedMessage(ptm)
    
    
    ## Assign seqlengths
    if (!is.null(chrom.lengths)) {
        if (is.character(chrom.lengths)) {
            df <- tryCatch({
                df <- data.table::fread(chrom.lengths, header=TRUE)
            }, error = function(err) {
                df <- utils::read.table(chrom.lengths, header=TRUE)
            })
        } else if (is.data.frame(chrom.lengths)) {
            df <- chrom.lengths
        }
        chrom.lengths <- df[,2]
        names(chrom.lengths) <- df[,1]
        # Filter by chromosomes supplied in chrom.lengths
        data <- keepSeqlevels(data, seqlevels(data)[seqlevels(data) %in% names(chrom.lengths)])
        seqlengths(data) <- chrom.lengths[names(seqlengths(data))]
    }
    
    return(data)
}


#' @describeIn import Import a BSSeeker methylation extractor file.
#' @importFrom data.table fread
#' @importFrom utils read.table
#' @export
#' 
importBSSeeker <- function(file, chrom.lengths=NULL, skip=0) {
    
    # classes <- c(seqnames='character', nucleotide='character', position='numeric', context='character', context.dinucleotide='character', methylation.level='numeric', counts.methylated='numeric', counts.total='numeric')
    classes <- c('character', 'character', 'numeric', 'character', 'character', 'numeric', 'numeric', 'numeric')
    ptm <- startTimedMessage("Reading file ", file, " ...")
    data.raw <- tryCatch({
        data.raw <- data.table::fread(file, skip=skip, sep='\t', colClasses=classes)
    }, error = function(err) {
        data.raw <- utils::read.table(file, skip=skip, sep='\t', colClasses=classes)
    })
    data <- GRanges(seqnames=data.raw$V1, ranges=IRanges(start=data.raw$V3, end=data.raw$V3), strand=c('C'='+', 'G'='-')[data.raw$V2], context=data.raw$V4)
    counts <- array(NA, dim=c(length(data), 2), dimnames=list(NULL, c("methylated", "total")))
    counts[,"methylated"] <- data.raw$V7
    counts[,"total"] <- data.raw$V8
    data$counts <- counts
    rm(data.raw)
    stopTimedMessage(ptm)
    
    
    ## Assign seqlengths
    if (!is.null(chrom.lengths)) {
        if (is.character(chrom.lengths)) {
            df <- tryCatch({
                df <- data.table::fread(chrom.lengths, header=TRUE)
            }, error = function(err) {
                df <- utils::read.table(chrom.lengths, header=TRUE)
            })
        } else if (is.data.frame(chrom.lengths)) {
            df <- chrom.lengths
        }
        chrom.lengths <- df[,2]
        names(chrom.lengths) <- df[,1]
        # Filter by chromosomes supplied in chrom.lengths
        data <- keepSeqlevels(data, seqlevels(data)[seqlevels(data) %in% names(chrom.lengths)])
        seqlengths(data) <- chrom.lengths[names(seqlengths(data))]
    }
    
    ## Make factors
    data$context <- factor(data$context)
    
    return(data)
}


#' @describeIn import Import a Bismark methylation extractor file.
#' @importFrom data.table fread
#' @importFrom utils read.table
#' @export
#'
importBismark <- function(file, chrom.lengths=NULL, skip=0) {
    
    # classes <- c(seqnames='character', position='numeric', strand='character', counts.methylated='numeric', counts.total='numeric', context='character', context.trinucleotide='character')
    classes <- c('character', 'numeric', 'character', 'numeric', 'numeric', 'character', 'character')
    ptm <- startTimedMessage("Reading file ", file, " ...")
    data.raw <- tryCatch({
        data.raw <- data.table::fread(file, skip=skip, sep='\t', colClasses=classes)
    }, error = function(err) {
        data.raw <- utils::read.table(file, skip=skip, sep='\t', colClasses=classes)
    })
    data <- GRanges(seqnames=data.raw$V1, ranges=IRanges(start=data.raw$V2, end=data.raw$V2), strand=data.raw$V3, context=data.raw$V6)
    counts <- array(NA, dim=c(length(data), 2), dimnames=list(NULL, c("methylated", "total")))
    counts[,"methylated"] <- data.raw$V4
    counts[,"total"] <- data.raw$V4 + data.raw$V5
    data$counts <- counts
    rm(data.raw)
    stopTimedMessage(ptm)
    
    
    ## Assign seqlengths
    if (!is.null(chrom.lengths)) {
        if (is.character(chrom.lengths)) {
            df <- tryCatch({
                df <- data.table::fread(chrom.lengths, header=TRUE)
            }, error = function(err) {
                df <- utils::read.table(chrom.lengths, header=TRUE)
            })
        } else if (is.data.frame(chrom.lengths)) {
            df <- chrom.lengths
        }
        chrom.lengths <- df[,2]
        names(chrom.lengths) <- df[,1]
        # Filter by chromosomes supplied in chrom.lengths
        data <- keepSeqlevels(data, seqlevels(data)[seqlevels(data) %in% names(chrom.lengths)])
        seqlengths(data) <- chrom.lengths[names(seqlengths(data))]
    }
    
    ## Make factors
    data$context <- factor(data$context)
    
    return(data)
}


#' Import a Rene methylation extractor file
#' 
#' Import a Rene methylation extractor file into a \code{\link[GenomicRanges]{GRanges-class}} object.
#' 
#' @param file The file to import.
#' @param chrom.lengths A data.frame with chromosome names in the first, and chromosome lengths in the second column. Only chromosomes named in here will be returned. Alternatively a tab-separated file with such a data.frame (with headers).
#' @param skip The number of lines to skip. Usually 1 if the file contains a header and 0 otherwise.
#' @return A \code{\link{methimputeData}} object.
#' 
#' @importFrom data.table fread
#' @importFrom utils read.table
#' @examples
#'## Get an example file in Rene format
#'file <- system.file("extdata","arabidopsis_rene.txt", package="methimpute")
#'data(arabidopsis_chromosomes)
#'rene.data <- methimpute:::importRene(file, chrom.lengths=arabidopsis_chromosomes)
#'
importRene <- function(file, chrom.lengths=NULL, skip=1) {

    classes <- c('character', 'numeric', 'character', 'NULL', 'character', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric')
    ptm <- startTimedMessage("Reading file ", file, " ...")
    data.raw <- tryCatch({
        data.raw <- data.table::fread(file, skip=skip, sep='\t', colClasses=classes)
    }, error = function(err) {
        data.raw <- utils::read.table(file, skip=skip, sep='\t', colClasses=classes)
    })
    data <- GRanges(seqnames=data.raw$V1, ranges=IRanges(start=data.raw$V2, end=data.raw$V2), strand=c('F'='+', 'R'='-')[data.raw$V3], methylated=data.raw$V10, context=data.raw$V5)
    counts <- array(NA, dim=c(length(data), 2), dimnames=list(NULL, c("methylated", "total")))
    counts[,"methylated"] <- data.raw$V6
    counts[,"total"] <- data.raw$V7
    data$counts <- counts
    rm(data.raw)
    stopTimedMessage(ptm)
    
    seqlevels(data) <- paste0('chr', seqlevels(data))
    ## Assign seqlengths
    if (!is.null(chrom.lengths)) {
        if (is.character(chrom.lengths)) {
            df <- tryCatch({
                df <- data.table::fread(chrom.lengths, header=TRUE)
            }, error = function(err) {
                df <- utils::read.table(chrom.lengths, header=TRUE)
            })
        } else if (is.data.frame(chrom.lengths)) {
            df <- chrom.lengths
        }
        chrom.lengths <- df[,2]
        names(chrom.lengths) <- df[,1]
        # Filter by chromosomes supplied in chrom.lengths
        data <- keepSeqlevels(data, seqlevels(data)[seqlevels(data) %in% names(chrom.lengths)])
        seqlengths(data) <- chrom.lengths[names(seqlengths(data))]
    }
    
    ## Make factors
    data$context <- factor(data$context)
    
    return(data)
}

#' #' Import a Bismark methylation extractor file
#' #' 
#' #' Import a Bismark methylation extractor file into a \code{\link[GenomicRanges]{GRanges-class}} object.
#' #' 
#' #' @param files The files to import.
#' #' @param chrom.lengths A data.frame with chromosome names in the first, and chromosome lengths in the second column. Only chromosomes named in here will be returned. Alternatively a tab-separated file with such a data.frame (with headers).
#' #' @param temp.store A folder where to save temporary files on disk. Set to \code{NULL} if no temporary files should be created.
#' #' @return A \code{\link{methimputeData}} object.
#' #' 
#' #' @importFrom data.table fread
#' #' @importFrom utils read.table
#' #' @export
#' importBismark <- function(files, chrom.lengths=NULL, temp.store=tempfile("importBismark")) {
#'   
#'     classes <- c('NULL', 'character', 'character', 'numeric', 'character')
#'     datas <- GRangesList()
#'     
#'     if (!is.null(temp.store)) {
#'         ## Create folder for temporary files
#'         if (!file.exists(temp.store)) {
#'             dir.create(temp.store)
#'         }
#'     }
#'     
#'     ### Do each file at a time to avoid memory overflow ###
#'     data.dedups <- GRangesList()
#'     for (i1 in 1:length(files)) {
#'         file <- files[i1]
#'         savename <- file.path(temp.store, paste0(basename(file), '.RData'))
#'         if (!file.exists(savename)) {
#'             ptm <- startTimedMessage("Reading file ", file, " ...")
#'             data.raw <- tryCatch({
#'                 data.raw <- data.table::fread(file, skip=1, sep='\t', colClasses=classes)
#'             }, error = function(err) {
#'                 data.raw <- utils::read.table(file, skip=1, sep='\t', colClasses=classes)
#'             })
#'             data <- GRanges(seqnames=data.raw$V3, ranges=IRanges(start=data.raw$V4, end=data.raw$V4), strand="*", methylated=factor(data.raw$V2, levels=c("-", "+")), context=data.raw$V5)
#'             rm(data.raw)
#'             stopTimedMessage(ptm)
#'             if (!is.null(temp.store)) {
#'                 ptm <- startTimedMessage("Saving to file ", savename, " ...")
#'                 save(data, file=savename)
#'                 stopTimedMessage(ptm)
#'             }
#'         } else {
#'             ptm <- startTimedMessage("Loading file ", savename, " ...")
#'             temp.env <- new.env()
#'             data <- get(load(savename, envir=temp.env), envir=temp.env)
#'             stopTimedMessage(ptm)
#'         }
#'         
#'         ## Sorting
#'         ptm <- startTimedMessage("Sorting ...")
#'         data <- sort(data)
#'         stopTimedMessage(ptm)
#'         
#'         ## Get counts at each position
#'         ptm <- startTimedMessage("Getting counts ...")
#'         data$methylated <- as.logical(as.integer(data$methylated)-1)
#'         data$mask.dup <- c(FALSE, as.logical(data@seqnames[-1] == data@seqnames[-length(data)])) & c(FALSE, as.logical(data@ranges@start[-1] == data@ranges@start[-length(data)]))
#'         data$unmeth.sum <- rev(cumsum(rev(!data$methylated)))
#'         data$meth.sum <- rev(cumsum(rev(data$methylated)))
#'         data.dedup <- data[!data$mask.dup]
#'         rm(data)
#'         data.dedup$counts.unmethylated <- -diff(c(data.dedup$unmeth.sum, 0))
#'         data.dedup$counts.methylated <- -diff(c(data.dedup$meth.sum, 0))
#'         data.dedup$methylated <- NULL
#'         data.dedup$mask.dup <- NULL
#'         data.dedup$unmeth.sum <- NULL
#'         data.dedup$meth.sum <- NULL
#'         data.dedups[[length(data.dedups)+1]] <- data.dedup
#'         stopTimedMessage(ptm)
#'         
#'         ## Merging counts from data objects
#'         if (length(data.dedups) == 2) {
#'             ptm <- startTimedMessage("Merging counts ...")
#'             data <- unlist(data.dedups, use.names=FALSE)
#'             data <- sort(data)
#'             rm(data.dedups)
#'             data$mask.dup <- c(FALSE, as.logical(data@seqnames[-1] == data@seqnames[-length(data)])) & c(FALSE, as.logical(data@ranges@start[-1] == data@ranges@start[-length(data)]))
#'             data$unmeth.sum <- rev(cumsum(rev(data$counts.unmethylated)))
#'             data$meth.sum <- rev(cumsum(rev(data$counts.methylated)))
#'             data.dedup <- data[!data$mask.dup]
#'             rm(data)
#'             data.dedup$counts.unmethylated <- -diff(c(data.dedup$unmeth.sum, 0))
#'             data.dedup$counts.methylated <- -diff(c(data.dedup$meth.sum, 0))
#'             data.dedup$mask.dup <- NULL
#'             data.dedup$unmeth.sum <- NULL
#'             data.dedup$meth.sum <- NULL
#'             data.dedups <- GRangesList(data.dedup)
#'             stopTimedMessage(ptm)
#'         }
#'         
#'     }
#'     data <- data.dedups[[1]]
#'     rm(data.dedups, data.dedup)
#'   
#'     ## Assign seqlengths
#'     if (!is.null(chrom.lengths)) {
#'         if (is.character(chrom.lengths)) {
#'             df <- tryCatch({
#'                 df <- data.table::fread(chrom.lengths, header=TRUE)
#'             }, error = function(err) {
#'                 df <- utils::read.table(chrom.lengths, header=TRUE)
#'             })
#'         } else if (is.data.frame(chrom.lengths)) {
#'             df <- chrom.lengths
#'         }
#'         chrom.lengths <- df[,2]
#'         names(chrom.lengths) <- df[,1]
#'         # Filter by chromosomes supplied in chrom.lengths
#'         data <- keepSeqlevels(data, seqlevels(data)[seqlevels(data) %in% names(chrom.lengths)])
#'         seqlengths(data) <- chrom.lengths[names(seqlengths(data))]
#'     }
#'     
#'     ## Make factors
#'     data$context <- factor(data$context)
#'     
#'     return(data)
#' }
#' 

#' Call methylation status
#' 
#' Call methylation status of cytosines (or bins) with a separate Hidden Markov Model for each context.
#' 
#' The Hidden Markov model uses a binomial test for the emission densities. Transition probabilities are modeled with a distance dependent decay, specified by the parameter \code{transDist}.
#' 
#' @inheritParams callMethylation
#' @return A \code{\link{methimputeBinomialHMM}} object.
#' 
#' @export
#' @examples
#'## Get some toy data
#'file <- system.file("data","arabidopsis_toydata.RData", package="methimpute")
#'data <- get(load(file))
#'print(data)
#'model <- callMethylationSeparate(data)
#'print(model)
#'
callMethylationSeparate <- function(data, fit.on.chrom=NULL, transDist=Inf, eps=1, max.time=Inf, max.iter=Inf, count.cutoff=500, verbosity=1, num.threads=2+include.intermediate, initial.params=NULL, include.intermediate=FALSE, update='independent', min.reads=0) {

    ## Variables
    contexts <- intersect(levels(data$context), unique(data$context))
    transitionContexts <- paste(contexts, contexts, sep='-')
    if (!include.intermediate) {
        states <- c('Unmethylated', 'Methylated')
    } else {
        states <- c('Unmethylated', 'Intermediate', 'Methylated')
    }
    transDistvec <- rep(Inf, length(transitionContexts))
    names(transDistvec) <- transitionContexts
    if (is.null(names(transDist))) {
        transDistvec[1:length(transDistvec)] <- transDist
    } else {
        if (any(! names(transDist) %in% names(transDistvec))) {
            stop("Names of 'transDist' must be ", paste0(names(transDistvec), collapse=', '), ".")
        }
        transDistvec[names(transDist)] <- transDist
        rev.names <- sapply(strsplit(names(transDist), '-'), function(x) { paste0(rev(x), collapse = '-')})
        transDistvec[rev.names] <- transDist
    }

    ## Prepare adding of meta-data columns
    data$distance <- numeric(length(data))
    data$transitionContext <- factor(NA, levels=transitionContexts)
    data$posteriorMax <- numeric(length(data))
    data$posteriorMeth <- numeric(length(data))
    data$posteriorUnmeth <- numeric(length(data))
    data$status <- factor(NA, levels=states)
    data$rc.meth.lvl <- numeric(length(data))
    # data$rc.counts <- data$counts
    
    cmodels <- list()
    for (context in contexts) {
        messageU("Running context ", context, underline = '-', overline = '-')
        context.mask <- data$context == context
        cdata <- data[context.mask]
        cmodel <- callMethylation(cdata, fit.on.chrom=fit.on.chrom, transDist=transDistvec[paste0(context, '-', context)], eps=eps, max.time=max.time, max.iter=max.iter, count.cutoff=count.cutoff, verbosity=verbosity, num.threads=num.threads, initial.params=initial.params, include.intermediate=include.intermediate, update=update, min.reads=min.reads)
        data$distance[context.mask] <- cmodel$data$distance
        data$transitionContext[context.mask] <- cmodel$data$transitionContext
        data$posteriorMax[context.mask] <- cmodel$data$posteriorMax
        data$posteriorMeth[context.mask] <- cmodel$data$posteriorMeth
        data$posteriorUnmeth[context.mask] <- cmodel$data$posteriorUnmeth
        data$status[context.mask] <- cmodel$data$status
        data$rc.meth.lvl[context.mask] <- cmodel$data$rc.meth.lvl
        # data$rc.counts[context.mask,] <- cmodel$data$rc.counts
        
        cmodel$data <- NULL
        cmodel$segments <- NULL
        cmodels[[context]] <- cmodel
    }

    ### Construct result object ###
    messageU("Combining contexts ", paste0(contexts, collapse=', '), underline = '-', overline = '-')
    ptm <- startTimedMessage("Compiling results ...")
    r <- list()
    class(r) <- "methimputeBinomialHMM"
    ## Convergence info
    convergenceInfo <- list()
    convergenceInfo$logliks <- lapply(cmodels, function(x) { x$convergenceInfo$logliks })
    convergenceInfo$dlogliks <- lapply(cmodels, function(x) { x$convergenceInfo$dlogliks })
    convergenceInfo$parameterInfo <- lapply(cmodels, function(x) { x$convergenceInfo$parameterInfo })
    convergenceInfo$time <- lapply(cmodels, function(x) { x$convergenceInfo$time })
    r$convergenceInfo <- convergenceInfo
    
    ## Parameters
    params <- list()
    params$startProbs <- lapply(cmodels, function(x) { x$params$startProbs })
    params$transProbs <- lapply(cmodels, function(x) { x$params$transProbs[[1]] })
    names(params$transProbs) <- unlist(lapply(cmodels, function(x) { names(x$params$transProbs) }))
    params$transDist <- unlist(lapply(cmodels, function(x) { x$params$transDist }))
    names(params$transDist) <- unlist(lapply(cmodels, function(x) { names(x$params$transDist) }))
    params$emissionParams <- list()
    for (state in states) {
        params$emissionParams[[state]] <- array(unlist(lapply(cmodels, function(x) { x$params$emissionParams[[state]] })), dim=c(length(contexts), 1), dimnames=list(contexts, 'prob'))
    }
    params$count.cutoff <- count.cutoff
    params$weights <- lapply(cmodels, function(x) { x$params$weights[[1]] })
    r$params <- params
    
    ## Parameters initial
    params.initial <- cmodels[[1]]$params.initial
    params.initial$startProbs_initial <- lapply(cmodels, function(x) { x$params.initial$startProbs_initial })
    params.initial$transProbs_initial <- lapply(cmodels, function(x) { x$params.initial$transProbs_initial[[1]] })
    names(params.initial$transProbs_initial) <- unlist(lapply(cmodels, function(x) { names(x$params.initial$transProbs_initial) }))
    params.initial$emissionParams_initial <- list()
    for (state in states) {
        params.initial$emissionParams_initial[[state]] <- array(unlist(lapply(cmodels, function(x) { x$params.initial$emissionParams_initial[[state]] })), dim=c(length(contexts), 1), dimnames=list(contexts, 'prob'))
    }
    params.initial$transDist <- unlist(lapply(cmodels, function(x) { x$params.initial$transDist }))
    names(params.initial$transDist) <- unlist(lapply(cmodels, function(x) { names(x$params.initial$transDist) }))
    r$params.initial <- params.initial
    
    ## Segmentation
    data.collapse <- data
    mcols(data.collapse) <- NULL
    data.collapse$status <- data$status
    df <- suppressMessages( collapseBins(as.data.frame(data.collapse), column2collapseBy = 'status') )
    segments <- methods::as(df, 'GRanges')
    seqlengths(segments) <- seqlengths(data)[seqlevels(segments)]
    
    ## Data and segments
    r$data <- data
    r$segments <- segments
    stopTimedMessage(ptm)
    
    return(r)
}



#' Call methylation status
#' 
#' Call methylation status of cytosines (or bins) with a Hidden Markov Model.
#' 
#' The Hidden Markov model uses a binomial test for the emission densities. Transition probabilities are modeled with a distance dependent decay, specified by the parameter \code{transDist}.
#' 
#' @param data A \code{\link{methimputeData}} object.
#' @param fit.on.chrom A character vector specifying the chromosomes on which the HMM will be fitted.
#' @param transDist The decaying constant for the distance-dependent transition matrix. Either a single numeric or a named numeric vector, where the vector names correspond to the transition contexts. Such a vector can be obtained from \code{\link{estimateTransDist}}.
#' @param eps Convergence threshold for the Baum-Welch algorithm.
#' @param max.time Maximum running time in seconds for the Baum-Welch algorithm.
#' @param max.iter Maximum number of iterations for the Baum-Welch algorithm.
#' @param count.cutoff A cutoff for the counts to remove artificially high counts from mapping artifacts. Set to \code{Inf} to disable this filtering (not recommended).
#' @param verbosity An integer from 1 to 5 specifying the verbosity of the fitting procedure. Values > 1 are only for debugging.
#' @param num.threads Number of CPU to use for the computation. Parallelization is implemented on the number of states, which is 2 or 3, so setting \code{num.threads > 3} will not give additional performance increase.
#' @param initial.params A \code{\link{methimputeBinomialHMM}} object. This parameter is useful to continue the fitting procedure for a \code{\link{methimputeBinomialHMM}} object.
#' @param include.intermediate A logical specifying wheter or not the intermediate component should be included in the HMM.
#' @param update One of \code{c("independent", "constrained")}. If \code{update="independent"} probability parameters for the binomial test will be updated independently. If \code{update="constrained"} the probability parameter of the intermediate component will be constrained to the mean of the unmethylated and the methylated component.
#' @param min.reads The minimum number of reads that a position must have to contribute in the Baum-Welch fitting procedure.
#' @return A \code{\link{methimputeBinomialHMM}} object.
#' 
#' @export
#' @examples
#'## Get some toy data
#'file <- system.file("data","arabidopsis_toydata.RData", package="methimpute")
#'data <- get(load(file))
#'print(data)
#'model <- callMethylation(data)
#'print(model)
callMethylation <- function(data, fit.on.chrom=NULL, transDist=Inf, eps=1, max.time=Inf, max.iter=Inf, count.cutoff=500, verbosity=1, num.threads=2+include.intermediate, initial.params=NULL, include.intermediate=FALSE, update='independent', min.reads=0) {

    ### Input checks ###
    if (!is.null(fit.on.chrom)) {
        if (!fit.on.chrom %in% seqlevels(data)) {
            stop("Cannot find 'fit.on.chrom' = ", fit.on.chrom, " in the data.")
        }
    }

    ### Assign variables ###
    update <- factor(update, levels=c('independent', 'constrained'))
    if (is.na(update)) { stop("Argument 'update' must be one of c('independent', 'constrained').") }
    contexts <- intersect(levels(data$context), unique(data$context))
    ncontexts <- length(contexts)
    if (!include.intermediate) {
        states <- c('Unmethylated', 'Methylated')
    } else {
        states <- c('Unmethylated', 'Intermediate', 'Methylated')
    }
    numstates <- length(states)
    counts <- data$counts
    
    ### Add distance and transition context to bins ###
    data$distance <- addDistance(data)
    data$transitionContext <- addTransitionContext(data)
    # Assign variables
    transitionContexts <- levels(data$transitionContext)
    transDistvec <- rep(Inf, length(transitionContexts))
    names(transDistvec) <- transitionContexts
    if (is.null(names(transDist))) {
        transDistvec[1:length(transDistvec)] <- transDist
    } else {
        if (any(! names(transDist) %in% names(transDistvec))) {
            stop("Names of 'transDist' must be ", paste0(names(transDistvec), collapse=', '), ".")
        }
        transDistvec[names(transDist)] <- transDist
        rev.names <- sapply(strsplit(names(transDist), '-'), function(x) { paste0(rev(x), collapse = '-')})
        transDistvec[rev.names] <- transDist
    }
    distances <- data$distance
    context <- factor(data$context, levels=contexts)
    transitionContext <- factor(data$transitionContext, levels=transitionContexts)
    
    ## Filter counts by cutoff
    mask <- counts[,'total'] > count.cutoff
    counts[mask,] <- round(sweep(x = counts[mask,, drop=FALSE], MARGIN = 1, STATS = counts[mask,'total', drop=FALSE]/count.cutoff, FUN = '/'))
    data$counts <- counts # assign it now to have filtered values in there
    
    ## Subset by chromosomes
    if (!is.null(fit.on.chrom)) {
        mask <- as.logical(data@seqnames %in% fit.on.chrom)
        counts <- counts[mask,]
        context <- context[mask]
        transitionContext <- transitionContext[mask]
        distances <- distances[mask]
    }

    ### Initial probabilities ###
    if (is.null(initial.params)) {
        transProbs_initial <- list()
        for (context.transition in transitionContexts) {
            s <- 0.9
            transProbs_initial[[context.transition]] <- matrix((1-s)/(numstates-1), ncol=numstates, nrow=numstates, dimnames=list(from=states, to=states))
            diag(transProbs_initial[[context.transition]]) <- s
        }
        startProbs_initial <- rep(1/numstates, numstates)
        names(startProbs_initial) <- states
        
        ### Initialization of emission distributions ###
        ep <- list()
        probUN.start <- 0.01
        probM.start <- 0.9
        if (!include.intermediate) {
            probs <- rep(probUN.start, ncontexts)
            names(probs) <- contexts
            ep[[states[1]]] <- data.frame(prob=probs)
            probs <- rep(probM.start, ncontexts)
            names(probs) <- contexts
            ep[[states[2]]] <- data.frame(prob=probs)
        } else {
            probs <- rep(probUN.start, ncontexts)
            names(probs) <- contexts
            ep[[states[1]]] <- data.frame(prob=probs)
            probs <- rep(0.5*(probUN.start+probM.start), ncontexts)
            names(probs) <- contexts
            ep[[states[2]]] <- data.frame(prob=probs)
            probs <- rep(probM.start, ncontexts)
            names(probs) <- contexts
            ep[[states[3]]] <- data.frame(prob=probs)
        }
        names(ep) <- states
        emissionParams_initial <- ep
    } else {
        model.init <- loadFromFiles(initial.params, check.class='methimputeBinomialHMM')[[1]]
        transProbs_initial <- model.init$params$transProbs
        startProbs_initial <- model.init$params$startProbs
        emissionParams_initial <- model.init$params$emissionParams
    }

    ### Define parameters for C function ###
    params <- list()
    params$startProbs_initial <- startProbs_initial
    params$transProbs_initial <- transProbs_initial
    params$emissionParams_initial <- emissionParams_initial
    params$transDist <- transDistvec
    params$eps <- eps
    params$maxtime <- max.time
    params$maxiter <- max.iter
    params$minreads <- min.reads
    params$verbosity <- verbosity
    params$numThreads <- num.threads
    
    ### Call the C function ###
    on.exit(cleanup())
    if (is.null(fit.on.chrom)) {
        ptm <- startTimedMessage("Baum-Welch: Fitting HMM parameters\n")
        hmm <- fitBinomialTestHMMcontextTransition(counts_total=counts[,'total'], counts_meth=counts[,'methylated'], context=as.integer(context)-1, transitionContext=as.integer(transitionContext)-1, distances=distances, params=params, algorithm=1, update_procedure=update)
        message("Time spent in Baum-Welch:", appendLF=FALSE)
        stopTimedMessage(ptm)
        if (hmm$error == "") {
            ## Cast convergence info
            parray <- array(NA, dim=c(length(contexts), length(hmm$convergenceInfo$logliks), length(states)), dimnames=list(context=contexts, iteration=0:(length(hmm$convergenceInfo$logliks)-1), status=states))
            parray[,,'Unmethylated'] <- hmm$convergenceInfo$parameterInfo$probsUN
            parray[,,'Methylated'] <- hmm$convergenceInfo$parameterInfo$probsM
            if ('Intermediate' %in% states) {
                parray[,,'Intermediate'] <- (parray[,,'Unmethylated'] + parray[,,'Methylated']) / 2
            }
            convergenceInfo <- hmm$convergenceInfo
            convergenceInfo$parameterInfo <- parray[,-1,, drop=FALSE]
            convergenceInfo$logliks <- convergenceInfo$logliks[-1]
            convergenceInfo$dlogliks <- convergenceInfo$dlogliks[-1]
        }
    } else {
        ptm <- startTimedMessage("Baum-Welch: Fitting HMM parameters\n")
        message(" ... on chromosomes ", paste0(fit.on.chrom, collapse=', '))
        hmm <- fitBinomialTestHMMcontextTransition(counts_total=counts[,'total'], counts_meth=counts[,'methylated'], context=as.integer(context)-1, transitionContext=as.integer(transitionContext)-1, distances=distances, params=params, algorithm=1, update_procedure=update)
        message("Time spent in Baum-Welch:", appendLF=FALSE)
        stopTimedMessage(ptm)
        if (hmm$error == "") {
            ## Cast convergence info
            convergenceInfo <- list(prefit=list(), fit=list())
            convergenceInfo$prefit$fit.on.chrom <- fit.on.chrom
            parray <- array(NA, dim=c(length(contexts), length(hmm$convergenceInfo$logliks), length(states)), dimnames=list(context=contexts, iteration=0:(length(hmm$convergenceInfo$logliks)-1), status=states))
            parray[,,'Unmethylated'] <- hmm$convergenceInfo$parameterInfo$probsUN
            parray[,,'Methylated'] <- hmm$convergenceInfo$parameterInfo$probsM
            if ('Intermediate' %in% states) {
                parray[,,'Intermediate'] <- (parray[,,'Unmethylated'] + parray[,,'Methylated']) / 2
            }
            convergenceInfo$prefit <- hmm$convergenceInfo
            convergenceInfo$prefit$parameterInfo <- parray[,-1,, drop=FALSE]
            convergenceInfo$prefit$logliks <- convergenceInfo$prefit$logliks[-1]
            convergenceInfo$prefit$dlogliks <- convergenceInfo$prefit$dlogliks[-1]
            ## Redo for all chromosomes
            counts <- data$counts
            context <- data$context
            transitionContext <- data$transitionContext
            distances <- data$distance
            params2 <- list()
            params2$startProbs_initial <- hmm$startProbs
            params2$transProbs_initial <- hmm$transProbs
            params2$emissionParams_initial <- hmm$emissionParams
            params2$transDist <- transDistvec
            params2$eps <- eps
            params2$maxtime <- max.time
            params2$maxiter <- max.iter
            params2$minreads <- min.reads
            params2$verbosity <- verbosity
            params2$numThreads <- num.threads
            ptm <- startTimedMessage("Forward-Backward: Obtaining state sequence - no updates\n")
            message(" ... on all chromosomes")
            hmm <- fitBinomialTestHMMcontextTransition(counts_total=counts[,'total'], counts_meth=counts[,'methylated'], context=as.integer(context)-1, transitionContext=as.integer(transitionContext)-1, distances=distances, params=params2, algorithm=2, update_procedure=update)
            message("Time spent in Forward-Backward:", appendLF=FALSE)
            stopTimedMessage(ptm)
            ## Cast convergence info
            convergenceInfo$fit <- hmm$convergenceInfo
        }
    }
    
    ### Construct result object ###
    ptm <- startTimedMessage("Compiling results ...")
    r <- list()
    class(r) <- "methimputeBinomialHMM"
    if (hmm$error == "") {
        r$convergenceInfo <- convergenceInfo
        names(hmm$weights) <- states
        r$params <- list(startProbs=hmm$startProbs, transProbs=hmm$transProbs, transDist=hmm$transDist, emissionParams=hmm$emissionParams, count.cutoff=count.cutoff)
        r$params.initial <- params
        # States and posteriors
        rownames(hmm$posteriors) <- states
        data$posteriorMax <- NA
        for (i1 in 0:(nrow(hmm$posteriors)-1)) {
            mask <- hmm$states == i1
            data$posteriorMax[mask] <- hmm$posteriors[i1+1,mask]
        }
        data$posteriorMeth <- hmm$posteriors["Methylated",]
        data$posteriorUnmeth <- hmm$posteriors["Unmethylated",]
        data$status <- factor(states, levels=states)[hmm$states+1]
        # Recalibrated methylation levels
        data.context <- as.character(data$context)
        if (include.intermediate) {
            data$rc.meth.lvl <- r$params$emissionParams$Unmethylated[data.context,] * data$posteriorUnmeth + r$params$emissionParams$Intermediate[data.context,] * (1 - data$posteriorMeth - data$posteriorUnmeth) + r$params$emissionParams$Methylated[data.context,] * data$posteriorMeth
        } else {
            data$rc.meth.lvl <- r$params$emissionParams$Unmethylated[data.context,] * data$posteriorUnmeth + r$params$emissionParams$Methylated[data.context,] * data$posteriorMeth
        }
        # ## Recalibrated counts
        # data$rc.counts <- data$counts
        # data$rc.counts[,2] <- sort(data$counts[,2])[rank(data$posteriorMax, ties.method = 'first')]
        # data$rc.counts[,1] <- round(data$rc.counts[,2] * data$rc.meth.lvl)
        ## Segmentation
        data.collapse <- data
        mcols(data.collapse) <- NULL
        data.collapse$status <- data$status
        df <- suppressMessages( collapseBins(as.data.frame(data.collapse), column2collapseBy = 'status') )
        segments <- methods::as(df, 'GRanges')
        seqlengths(segments) <- seqlengths(data)[seqlevels(segments)]
        ## Context-dependent weights
        r$params$weights <- list()
        for (context in contexts) {
            r$params$weights[[context]] <- rowMeans(hmm$posteriors[,data$context==context])
        }
    } else {
        warning("Baum-Welch aborted: ", hmm$error)
    }
    r$data <- data
    r$segments <- segments
    stopTimedMessage(ptm)
    
    return(r)
}


#' Call methylation status
#' 
#' Call methylation status of cytosines (or bins) with a binomial test.
#' 
#' The function uses a binomial test with the specified \code{conversion.rate}. P-values are then multiple testing corrected with the Benjamini & Yekutieli procedure. Methylated positions are selected with the \code{p.threshold}.
#' 
#' @param data A \code{\link{methimputeData}} object.
#' @param conversion.rate A conversion rate between 0 and 1.
#' @param min.coverage Minimum coverage to consider for the binomial test.
#' @param p.threshold Significance threshold between 0 and 1.
#' @return A vector with methylation statuses.
#' @importFrom stats p.adjust pbinom
#' @export
#' 
#' @examples
#'## Get some toy data
#'file <- system.file("data","arabidopsis_toydata.RData", package="methimpute")
#'data <- get(load(file))
#'data$binomial <- binomialTestMethylation(data, conversion.rate=0.998)
#'
binomialTestMethylation <- function(data, conversion.rate, min.coverage=3, p.threshold=0.05) {

    p <- stats::pbinom(q = data$counts[,'methylated']-1, size = data$counts[,'total'], prob = conversion.rate, lower.tail = FALSE)
    p[data$counts[,'total'] < min.coverage] <- NA
    p <- stats::p.adjust(p, method = 'BY')
    levels <- c("Unmethylated", "Methylated")
    methylated <- factor(levels[c(1,2)][(p <= p.threshold)+1], levels=levels)
    return(methylated)

}

#' #' Make a multivariate segmentation
#' #' 
#' #' Make a multivariate segmentation by fitting the transition probabilities of a multivariate Hidden Markov Model.
#' #' 
#' #' The function will use the provided univariate Hidden Markov Models (HMMs) to build the multivariate emission density. Number of states are also taken from the combination of univaraite HMMs.
#' #' 
#' #' @return A list with fitted parameters, posteriors.
#' multivariateSegmentation <- function(models, ID, fit.on.chrom=NULL, transDist=700, eps=0.01, max.time=Inf, max.iter=Inf, max.states=Inf, verbosity=1, num.threads=1) {
#'   
#'     ### Input checks ###
#'     if (is.null(max.time)) {
#'         max.time <- Inf
#'     }
#'     if (is.null(max.iter)) {
#'         max.iter <- Inf
#'     }
#'     ID.check <- ID
#'   
#'     ### Assign variables ###
#'     models <- loadFromFiles(models, check.class = 'NcomponentHMM')
#'     components <- lapply(models, function(model) { levels(model$data$state) })
#'     statedef <- permute(components)
#'     # Data object
#'     data <- models[[1]]$data
#'     mcols(data) <- NULL
#'     data$distance <- models[[1]]$data$distance
#'     
#'     ### Prepare the multivariate ###
#'     messageU("Multivariate HMM: preparing fitting procedure", underline="=", overline="=")
#'     cormat <- prepareMultivariate(data=models[[1]]$data, hmms=models, statedef=statedef, max.states=max.states)
#'     data$observable <- cormat$observable
#'     statenames <- cormat$mapping
#'     statenames <- lapply(strsplit(statenames, ' '), function(x) { paste(paste0(names(models), "=", x), collapse=" ") })
#'     numstates <- length(statenames)
#'     statedef <- cormat$statedef
#'     
#'     ### Initial probabilities ###
#'     s <- 0.9
#'     transProbs_initial <- matrix((1-s)/(numstates-1), ncol=numstates, nrow=numstates, dimnames=list(from=statenames, to=statenames))
#'     diag(transProbs_initial) <- s
#'     startProbs_initial <- rep(1/numstates, numstates)
#'     names(startProbs_initial) <- statenames
#'     
#'     ### Define parameters for C function ###
#'     params <- list()
#'     params$startProbs_initial <- startProbs_initial
#'     params$transProbs_initial <- transProbs_initial
#'     params$emissionParamsList <- cormat$emissionParams
#'     params$transDist <- transDist
#'     params$eps <- eps
#'     params$maxtime <- max.time
#'     params$maxiter <- max.iter
#'     params$verbosity <- verbosity
#'     params$numThreads <- num.threads
#'     params$correlationMatrixInverse <- cormat$correlationMatrixInverse
#'     params$determinant <- cormat$determinant
#'     params$statedef <- cormat$statedef
#'     
#'     ### Fit the HMM ###
#'     message("Baum-Welch: Fitting parameters for multivariate HMM")
#'     on.exit(cleanup())
#'     hmm <- fitMultiHMM(data$observable, data$distance, params)
#'     
#'     ### Construct result object ###
#'     ptm <- startTimedMessage("Compiling results ...")
#'     r <- list()
#'     class(r) <- 'NcomponentMultiHMM'
#'     r$ID <- ID
#'     if (hmm$error == "") {
#'         r$convergenceInfo <- hmm$convergenceInfo
#'         names(hmm$weights) <- statenames
#'         r$params <- list(startProbs=hmm$startProbs, transProbs=hmm$transProbs, transDist=hmm$transDist, emissionParamsList=params$emissionParamsList, weights=hmm$weights)
#'         r$params.initial <- params
#'         # States and posteriors
#'         data$posteriorMax <- NA
#'         for (i1 in 0:(nrow(hmm$posteriors)-1)) {
#'             mask <- hmm$states == i1
#'             data$posteriorMax[mask] <- hmm$posteriors[i1+1,mask]
#'         }
#'         rownames(hmm$posteriors) <- statenames
#'         data$posteriors <- t(hmm$posteriors)
#'         data$state <- factor(statenames, levels=statenames)[hmm$states+1]
#'         ## Make segmentation
#'         df <- as.data.frame(data)
#'         df <- df[, c(names(df)[1:5], 'state')]
#'         segments <- suppressMessages( collapseBins(df, column2collapseBy = 'state') )
#'         segments <- methods::as(segments, 'GRanges')
#'         seqlevels(segments) <- seqlevels(data)
#'         seqlengths(segments) <- seqlengths(data)[seqlevels(segments)]
#'     }
#'     r$segments <- segments
#'     r$data <- data
#'     stopTimedMessage(ptm)
#'     
#'     return(r)
#' }
#' 

#' #' Fit a two-component HMM
#' #' 
#' #' Fit a two-component Hidden Markov Model to the supplied counts. The transition matrix is distance-dependent with exponential decaying constant \code{transDist}. Components are modeled as negative binomial distributions.
#' #' 
#' #' @param data A \code{\link[GenomicRanges]{GRanges-class}} object with metadata column 'counts' (or any other column specified as \code{observable}).
#' #' @param observable A character naming the metadata column of \code{data} that will serve as observable for the HMM.
#' #' @param fit.on.chrom A character vector giving the chromosomes on which the HMM will be fitted.
#' #' @param transDist The exponential decaying constant for the distance-dependent transition matrix. Should be given in the same units as \code{distances}.
#' #' @param eps Convergence threshold for the Baum-Welch algorithm.
#' #' @param max.time Maximum running time in seconds for the Baum-Welch algorithm.
#' #' @param max.iter Maximum number of iterations for the Baum-Welch algorithm.
#' #' @param count.cutoff A cutoff for the \code{observable}. Lower cutoffs will speed up the fitting procedure and improve convergence in some cases. Set to \code{Inf} to disable this filtering.
#' #' @param verbosity Integer from c(0,1) specifying the verbosity of the fitting procedure.
#' #' @param num.threads Number of CPU to use for the computation.
#' #' @return A list with fitted parameters, posteriors, and the input parameters.
#' #' 
#' fitSignalBackground <- function(data, observable='counts', fit.on.chrom=NULL, transDist=700, eps=0.01, max.time=Inf, max.iter=Inf, cutoff=1000, verbosity=1, num.threads=1) {
#'   
#'     ### Input checks ###
#'     if (is.null(max.time)) {
#'         max.time <- Inf
#'     }
#'     if (is.null(max.iter)) {
#'         max.iter <- Inf
#'     }
#'   
#'     ### Add distance to bins ###
#'     data$distance <- addDistance(data)
#'     
#'     ### Assign variables ###
#'     states <- c("background", "signal")
#'     numstates <- length(states)
#'     counts <- mcols(data)[,observable]
#'     counts <- as.matrix(counts)
#'     distances <- data$distance
#'     
#'     ## Filter counts by cutoff
#'     mask <- counts[,'total'] > count.cutoff
#'     counts[mask,] <- round(sweep(x = counts[mask,], MARGIN = 1, STATS = counts[mask,'total']/count.cutoff, FUN = '/'))
#'     data$observable <- counts # assign it now to have filtered values in there
#'     colnames(data$observable) <- observable
#'     
#'     ## Subset by chromosomes
#'     if (!is.null(fit.on.chrom)) {
#'         counts <- counts[as.logical(data@seqnames %in% fit.on.chrom)]
#'     }
#'   
#'     ### Initial probabilities ###
#'     s <- 0.9
#'     transProbs_initial <- matrix((1-s)/(numstates-1), ncol=numstates, nrow=numstates, dimnames=list(from=states, to=states))
#'     diag(transProbs_initial) <- s
#'     startProbs_initial <- rep(1/numstates, numstates)
#'     names(startProbs_initial) <- states
#'     
#'     ### Initialization of emission distributions ###
#'     counts.greater.0 <- counts[counts>0]
#'     mean.counts <- mean(counts.greater.0)
#'     var.counts <- var(counts.greater.0)
#'     ep <- list()
#'     ep[["background"]] <- data.frame(type='dnbinom', mu=mean.counts, var=var.counts)
#'     ep[["signal"]] <- data.frame(type='dnbinom', mu=mean.counts*1.5, var=var.counts*2)
#'     ep <- do.call(rbind, ep)
#'     # Make sure variance is bigger than mean for negative binomial
#'     ep$var[ep$mu >= ep$var] <- ep$mu[ep$mu >= ep$var] + 1
#'     ep$size <- dnbinom.size(ep$mu, ep$var)
#'     ep$prob <- dnbinom.prob(ep$mu, ep$var)
#'     emissionParams_initial <- ep
#'   
#'     ### Define parameters for C function ###
#'     params <- list()
#'     params$startProbs_initial <- startProbs_initial
#'     params$transProbs_initial <- transProbs_initial
#'     params$emissionParams_initial <- emissionParams_initial
#'     params$transDist <- transDist
#'     params$eps <- eps
#'     params$maxtime <- max.time
#'     params$maxiter <- max.iter
#'     params$verbosity <- verbosity
#'     params$numThreads <- num.threads
#'     
#'     ### Call the C function ###
#'     on.exit(cleanup())
#'     if (is.null(fit.on.chrom)) {
#'         ptm <- startTimedMessage("Baum-Welch: Fitting HMM parameters\n")
#'         hmm <- fitHMM(counts=counts, distances=distances, params=params, algorithm=1)
#'         message("Time spent in Baum-Welch:", appendLF=FALSE)
#'         stopTimedMessage(ptm)
#'     } else {
#'         ptm <- startTimedMessage("Baum-Welch: Fitting HMM parameters\n")
#'         message(" ... on chromosomes ", paste0(fit.on.chrom, collapse=', '))
#'         hmm <- fitHMM(counts=counts, distances=distances, params=params, algorithm=1)
#'         message("Time spent in Baum-Welch:", appendLF=FALSE)
#'         stopTimedMessage(ptm)
#'         counts <- as.integer(data$observable)
#'         params2 <- list()
#'         params2$startProbs_initial <- hmm$startProbs
#'         params2$transProbs_initial <- hmm$transProbs
#'         params2$emissionParams_initial <- hmm$emissionParams
#'         params2$transDist <- transDist
#'         params2$eps <- eps
#'         params2$maxtime <- max.time
#'         params2$maxiter <- max.iter
#'         params2$verbosity <- verbosity
#'         params2$numThreads <- num.threads
#'         ptm <- startTimedMessage("Viterbi: Obtaining state sequence\n")
#'         message(" ... on all chromosomes")
#'         hmm <- fitHMM(counts=counts, distances=distances, params=params2, algorithm=2)
#'         message("Time spent in Viterbi:", appendLF=FALSE)
#'         stopTimedMessage(ptm)
#'     }
#'     
#'     ### Construct result object ###
#'     ptm <- startTimedMessage("Compiling results ...")
#'     r <- list()
#'     if (hmm$error == "") {
#'         r$convergenceInfo <- hmm$convergenceInfo
#'         names(hmm$weights) <- states
#'         r$params <- list(startProbs=hmm$startProbs, transProbs=hmm$transProbs, transDist=hmm$transDist, emissionParams=hmm$emissionParams, weights=hmm$weights)
#'         r$params.initial <- params
#'         # States and posteriors
#'         data$posteriorMax <- NA
#'         for (i1 in 0:(nrow(hmm$posteriors)-1)) {
#'             mask <- hmm$states == i1
#'             data$posteriorMax[mask] <- hmm$posteriors[i1+1,mask]
#'         }
#'         rownames(hmm$posteriors) <- states
#'         data$posteriors <- t(hmm$posteriors)
#'         data$state <- factor(states, levels=states)[hmm$states+1]
#'     }
#'     r$data <- data
#'     stopTimedMessage(ptm)
#'     
#'     return(r)
#' }




#' #' Fit an n-component HMM
#' #'
#' #' Fit an n-component Hidden Markov Model to the supplied counts. The transition matrix is distance-dependent with exponential decaying constant \code{transDist} (only relevant in non-consecutive bins). The zero-th component is a delta distribution to account for empty bins, and all other n-components are modeled as negative binomial distributions.
#' #'
#' #' @param data A \code{\link[GenomicRanges]{GRanges-class}} object with metadata columns 'distance' and 'counts' (or any other column specified as \code{observable}).
#' #' @param states An integer vector giving the states for the Hidden Markov Model. State '0' will be modeled by a delta distribution, all other states ('1','2','3',...) with negative binomial distributions.
#' #' @param observable A character naming the column of \code{data} that will serve as observable for the HMM.
#' #' @param fit.on.chrom A character vector giving the chromosomes on which the HMM will be fitted.
#' #' @param transDist The exponential decaying constant for the distance-dependent transition matrix. Should be given in the same units as \code{distances}.
#' #' @param eps Convergence threshold for the Baum-Welch algorithm.
#' #' @param max.time Maximum running time in seconds for the Baum-Welch algorithm.
#' #' @param max.iter Maximum number of iterations for the Baum-Welch algorithm.
#' #' @param count.cutoff A cutoff for the \code{observable}. Lower cutoffs will speed up the fitting procedure and improve convergence in some cases. Set to \code{Inf} to disable this filtering.
#' #' @param verbosity Integer from c(0,1) specifying the verbosity of the fitting procedure.
#' #' @param num.threads Number of CPU to use for the computation.
#' #' @param initial.params An \code{\link{NcomponentHMM}} or a file that contains such an object. Parameters from this model will be used for initialization of the fitting procedure.
#' #' @return A list with fitted parameters, posteriors, and the input parameters.
#' #'
#' fitNComponentHMM <- function(data, states=0:5, observable='counts', fit.on.chrom=NULL, transDist=700, eps=0.01, max.time=Inf, max.iter=Inf, count.cutoff=500, verbosity=1, num.threads=1, initial.params=NULL) {
#' 
#'     ### Input checks ###
#'     if (is.null(max.time)) {
#'         max.time <- Inf
#'     }
#'     if (is.null(max.iter)) {
#'         max.iter <- Inf
#'     }
#' 
#'     ### Add distance to bins ###
#'     data$distance <- addDistance(data)
#' 
#'     ### Assign variables ###
#'     numstates <- length(states)
#'     counts <- mcols(data)[,observable]
#'     distances <- data$distance
#'     data$observable <- counts
#' 
#'     ## Subset by chromosomes
#'     if (!is.null(fit.on.chrom)) {
#'         counts <- counts[as.logical(data@seqnames %in% fit.on.chrom)]
#'     }
#' 
#'     ### Initial probabilities ###
#'     if (is.null(initial.params)) {
#'         s <- 0.9
#'         transProbs_initial <- matrix((1-s)/(numstates-1), ncol=numstates, nrow=numstates, dimnames=list(from=states, to=states))
#'         diag(transProbs_initial) <- s
#'         startProbs_initial <- rep(1/numstates, numstates)
#'         names(startProbs_initial) <- states
#' 
#'         ### Initialization of emission distributions ###
#'         counts.greater.0 <- counts[counts>0]
#'         mean.counts <- mean(counts.greater.0)
#'         var.counts <- var(counts.greater.0)
#'         ep <- list()
#'         for (state in setdiff(states,0)) {
#'             ep[[as.character(state)]] <- data.frame(type='dnbinom', mu=mean.counts*state/2, var=var.counts*state/2)
#'         }
#'         ep <- do.call(rbind, ep)
#'         # Make sure variance is bigger than mean for negative binomial
#'         ep$var[ep$mu >= ep$var] <- ep$mu[ep$mu >= ep$var] + 1
#'         ep$size <- dnbinom.size(ep$mu, ep$var)
#'         ep$prob <- dnbinom.prob(ep$mu, ep$var)
#'         ## Add zero-inflation
#'         if (0 %in% states) {
#'             ep <- rbind(data.frame(type='delta', mu=0, var=0, size=NA, prob=NA), ep)
#'         }
#'         rownames(ep) <- states
#'         emissionParams_initial <- ep
#'     } else {
#'         model.init <- loadFromFiles(initial.params, check.class='NcomponentHMM')[[1]]
#'         transProbs_initial <- model.init$params$transProbs
#'         startProbs_initial <- model.init$params$startProbs
#'         emissionParams_initial <- model.init$params$emissionParams
#'     }
#' 
#'     ### Define parameters for C function ###
#'     params <- list()
#'     params$startProbs_initial <- startProbs_initial
#'     params$transProbs_initial <- transProbs_initial
#'     params$emissionParams_initial <- emissionParams_initial
#'     params$transDist <- transDist
#'     params$eps <- eps
#'     params$maxtime <- max.time
#'     params$maxiter <- max.iter
#'     params$verbosity <- verbosity
#'     params$numThreads <- num.threads
#' 
#'     ### Call the C function ###
#'     on.exit(cleanup())
#'     if (is.null(fit.on.chrom)) {
#'         ptm <- startTimedMessage("Baum-Welch: Fitting HMM parameters\n")
#'         hmm <- fitHMM(counts=counts, distances=distances, params=params, algorithm=1)
#'         message("Time spent in Baum-Welch:", appendLF=FALSE)
#'         stopTimedMessage(ptm)
#'     } else {
#'         ptm <- startTimedMessage("Baum-Welch: Fitting HMM parameters\n")
#'         message(" ... on chromosomes ", paste0(fit.on.chrom, collapse=', '))
#'         hmm <- fitHMM(counts=counts, distances=distances, params=params, algorithm=1)
#'         message("Time spent in Baum-Welch:", appendLF=FALSE)
#'         stopTimedMessage(ptm)
#'         counts <- mcols(data)[,observable]
#'         params2 <- list()
#'         params2$startProbs_initial <- hmm$startProbs
#'         params2$transProbs_initial <- hmm$transProbs
#'         params2$emissionParams_initial <- hmm$emissionParams
#'         params2$transDist <- transDist
#'         params2$eps <- eps
#'         params2$maxtime <- max.time
#'         params2$maxiter <- max.iter
#'         params2$verbosity <- verbosity
#'         params2$numThreads <- num.threads
#'         ptm <- startTimedMessage("Forward-Backward: Obtaining state sequence - no updates\n")
#'         message(" ... on all chromosomes")
#'         hmm <- fitHMM(counts=counts, distances=distances, params=params2, algorithm=2)
#'         message("Time spent in Forward-Backward:", appendLF=FALSE)
#'         stopTimedMessage(ptm)
#'     }
#' 
#'     ### Construct result object ###
#'     ptm <- startTimedMessage("Compiling results ...")
#'     r <- list()
#'     class(r) <- "NcomponentHMM"
#'     if (hmm$error == "") {
#' 
#'         r$convergenceInfo <- hmm$convergenceInfo
#'         names(hmm$weights) <- states
#'         r$params <- list(startProbs=hmm$startProbs, transProbs=hmm$transProbs, transDist=hmm$transDist, emissionParams=hmm$emissionParams, weights=hmm$weights)
#'         r$params.initial <- params
#'         # States and posteriors
#'         data$posteriorMax <- NA
#'         for (i1 in 0:(nrow(hmm$posteriors)-1)) {
#'             mask <- hmm$states == i1
#'             data$posteriorMax[mask] <- hmm$posteriors[i1+1,mask]
#'         }
#'         rownames(hmm$posteriors) <- states
#'         data$posteriors <- t(hmm$posteriors)
#'         data$state <- factor(states, levels=states)[hmm$states+1]
#'         ## Segmentation
#'         data.collapse <- data
#'         mcols(data.collapse) <- NULL
#'         data.collapse$state <- data$state
#'         df <- suppressMessages( collapseBins(as.data.frame(data.collapse), column2collapseBy = 'state') )
#'         segments <- methods::as(df, 'GRanges')
#'         seqlengths(segments) <- seqlengths(data)[seqlevels(segments)]
#' 
#'         ## Reorder the states by mean of the emission distrbution
#'         stateorder <- as.character(states[order(hmm$emissionParams$mu)])
#'         # Initial params
#'         r$params.initial$startProbs_initial <- r$params.initial$startProbs_initial[stateorder]
#'         names(r$params.initial$startProbs_initial) <- states
#'         r$params.initial$transProbs_initial <- r$params.initial$transProbs_initial[stateorder, stateorder]
#'         rownames(r$params.initial$transProbs_initial) <- states
#'         colnames(r$params.initial$transProbs_initial) <- states
#'         r$params.initial$emissionParams_initial <- r$params.initial$emissionParams_initial[stateorder, ]
#'         rownames(r$params.initial$emissionParams_initial) <- states
#'         # Params
#'         r$params$startProbs <- r$params$startProbs[stateorder]
#'         names(r$params$startProbs) <- states
#'         r$params$transProbs <- r$params$transProbs[stateorder, stateorder]
#'         rownames(r$params$transProbs) <- states
#'         colnames(r$params$transProbs) <- states
#'         r$params$emissionParams <- r$params$emissionParams[stateorder, ]
#'         rownames(r$params$emissionParams) <- states
#'         r$params$weights <- r$params$weights[stateorder]
#'         names(r$params$weights) <- states
#'         # Posteriors
#'         data$posteriors <- data$posteriors[,stateorder]
#'         colnames(data$posteriors) <- states
#'         # Actual states
#'         stateorder.mapping <- states
#'         names(stateorder.mapping) <- stateorder
#'         stateorder.mapping <- factor(stateorder.mapping, levels=states)
#'         data$state <- stateorder.mapping[as.character(data$state)]
#'         segments$state <- stateorder.mapping[as.character(segments$state)]
#'     }
#'     r$data <- data
#'     r$segments <- segments
#'     stopTimedMessage(ptm)
#' 
#'     return(r)
#' }


#' #' @importFrom stats pnbinom qnorm dnbinom
#' prepareMultivariate = function(data, hmms, statedef, max.states=Inf) {
#'   
#'     ## Assign variables
#'     bins <- data
#'     mcols(bins) <- NULL
#'     bins$distance <- data$distance
#'     modelnames <- names(hmms)
#'     l <- list()
#'     for (i1 in 1:ncol(statedef)) {
#'         l[[i1]] <- statedef[,i1]
#'     }
#'     states <- rownames(statedef)
#'     names(states) <- do.call(paste, l)
#'     mapping <- names(states)
#'     names(mapping) <- states
#'     components <- lapply(statedef, levels)
#'     if (any(names(components) != modelnames)) {
#'         stop("names(hmms) must be equal to names(statedef)")
#'     }
#'     numstates <- length(states)
#'     
#'     ## Go through HMMs and extract stuff
#'     ptm <- startTimedMessage("Extracting states ...")
#'     counts <- matrix(NA, nrow=length(bins), ncol=length(modelnames), dimnames=list(NULL, modelnames))
#'     emissionParams <- list()
#'     statevec <- list()
#'     for (i1 in 1:length(modelnames)) {
#'         modelname <- modelnames[i1]
#'         counts[,i1] <- hmms[[modelname]]$data$observable
#'         emissionParams[[modelname]] <- hmms[[modelname]]$params$emissionParams
#'         statevec[[modelname]] <- hmms[[modelname]]$data$state
#'     }
#'     maxcounts <- max(apply(counts, 2, max))
#'     bins$counts <- counts
#'     bins$state <- factor(states[do.call(paste, statevec)], levels=states)
#'     stopTimedMessage(ptm)
#'     
#'     # Clean up to reduce memory usage
#'     # remove(hmm)
#'     
#'     ## We pre-compute the z-values for each number of counts
#'     ptm <- startTimedMessage("Computing pre z-matrix ...")
#'     z.per.read <- list()
#'     for (modelname in modelnames) {
#'         z.per.read[[modelname]] <- array(NA, dim=c(maxcounts+1, length(components[[modelname]])), dimnames=list(counts=NULL, component=components[[modelname]]))
#'         xcounts = 0:maxcounts
#'         for (component in components[[modelname]]) {
#'             
#'             size = emissionParams[[modelname]][component,'size']
#'             prob = emissionParams[[modelname]][component,'prob']
#'             u = stats::pnbinom(xcounts, size, prob)
#'             
#'             # Check for infinities in u and set them to max value which is not infinity
#'             qnorm_u = stats::qnorm(u)
#'             mask <- qnorm_u==Inf
#'             qnorm_u[mask] <- stats::qnorm(1-1e-16)
#'             z.per.read[[modelname]][ , component] <- qnorm_u
#'           
#'         }
#'     }
#'     stopTimedMessage(ptm)
#'     
#'     ## Compute the z matrix
#'     ptm <- startTimedMessage("Transfering values into z-matrix ...")
#'     z.per.bin <- list()
#'     for (modelname in modelnames) {
#'         z.per.bin[[modelname]] = array(NA, dim=c(length(bins), length(components[[modelname]])), dimnames=list(bin=NULL, component=components[[modelname]]))
#'         for (component in components[[modelname]]) {
#'             z.per.bin[[modelname]][ , component] <- z.per.read[[modelname]][bins$counts[, modelname]+1, i1]
#'         }
#'     }
#'     
#'     # Clean up to reduce memory usage
#'     remove(z.per.read)
#'     stopTimedMessage(ptm)
#'     
#'     ### Calculate correlation matrix
#'     ptm <- startTimedMessage("Computing inverse of correlation matrix ...")
#'     correlationMatrix = array(NA, dim=c(length(modelnames),length(modelnames),length(states)), dimnames=list(model=modelnames, model=modelnames, state=states))
#'     for (i1 in 1:dim(correlationMatrix)[3]) {
#'         correlationMatrix[,,i1] <- diag(length(modelnames))
#'     }
#'     correlationMatrixInverse = correlationMatrix
#'     determinant = rep(1, length(states))
#'     names(determinant) <- states
#'     
#'     for (state in states) {
#'         istate = which(state==states)
#'         mask = which(bins$state==state)
#'         if (length(mask) > 100) {
#'             # Subselect z
#'             z.temp <- matrix(NA, nrow=length(mask), ncol=length(modelnames), dimnames=list(NULL, model=modelnames))
#'             for (modelname in modelnames) {
#'                 z.temp[,modelname] <- z.per.bin[[modelname]][mask, as.character(statedef[state, modelname])]
#'             }
#'             temp <- tryCatch({
#'                 correlationMatrix[,,state] <- cor(z.temp)
#'                 determinant[state] <- det( correlationMatrix[,,state] )
#'                 correlationMatrixInverse[,,state] <- chol2inv(chol(correlationMatrix[,,state]))
#'                 0
#'             }, warning = function(war) {
#'                 1
#'             }, error = function(err) {
#'                 1
#'             })
#'         }
#'         if (any(is.na(correlationMatrixInverse[,,state]))) {
#'             correlationMatrixInverse[,,state] <- diag(length(modelnames))
#'         }
#'     }
#'     remove(z.per.bin)
#'     stopTimedMessage(ptm)
#'     
#'     ### Put correlation matrix into list
#'     corrList <- list()
#'     corrInvList <- list()
#'     for (state in states) {
#'         corrList[[state]] <- correlationMatrix[,, state]
#'         corrInvList[[state]] <- correlationMatrixInverse[,, state]
#'     }
#'     correlationMatrix <- corrList
#'     correlationMatrixInverse <- corrInvList
#'     
#'     ### Select states to use ###
#'     weights <- sort(table(bins$state), decreasing = TRUE)
#'     states2use <- names(weights)[1:min(max.states, length(weights))]
#'     correlationMatrix <- correlationMatrix[states2use]
#'     correlationMatrixInverse <- correlationMatrixInverse[states2use]
#'     determinant <- determinant[states2use]
#'     statedef <- statedef[states2use,]
#'     mapping <- mapping[states2use]
#'     
#'     # Return parameters
#'     out <- list(
#'                 correlationMatrix = correlationMatrix,
#'                 correlationMatrixInverse = correlationMatrixInverse,
#'                 determinant = determinant,
#'                 observable = counts,
#'                 emissionParams = emissionParams,
#'                 statedef = statedef,
#'                 mapping = mapping,
#'                 weights = weights
#'     )
#'     return(out)
#' }

#' Methimpute plotting functions
#' 
#' This page provides an overview of all \pkg{\link{methimpute}} plotting functions.
#'
#' @param model A \code{\link{methimputeBinomialHMM}} object.
#' @param datapoints The number of randomly selected datapoints for the plot.
#' @param binwidth The bin width for the histogram/boxplot.
#' @return A \code{\link[ggplot2]{ggplot}} object.
#' @name plotting
#' @import ggplot2
#' @examples 
#'## Get some toy data
#'file <- system.file("data","arabidopsis_toydata.RData",
#'                     package="methimpute")
#'data <- get(load(file))
#'print(data)
#'model <- callMethylation(data)
#'## Make nice plots
#'plotHistogram(model, total.counts=5)
#'plotScatter(model)
#'plotTransitionProbs(model)
#'plotConvergence(model)
#'plotPosteriorDistance(model$data)
#'
#'## Get annotation data and make an enrichment profile
#'# Note that this looks a bit ugly because our toy data
#'# has only 200000 datapoints.
#'data(arabidopsis_genes)
#'plotEnrichment(model, annotation=arabidopsis_genes)
#'
NULL

#' Get state colors
#'
#' Get the colors that are used for plotting.
#'
#' @param states A character vector.
#' @return A character vector with colors.
#' @seealso \code{\link{plotting}}
#' @export
#'@examples
#'cols <- getStateColors()
#'pie(1:length(cols), col=cols, labels=names(cols))
getStateColors <- function(states=NULL) {
    state.colors <- c("Background" = "gray", "Unmethylated" = "red","Methylated" = "blue","Intermediate" = "green", "Total" = "black", "Signal" = "red")
    if (is.null(states)) {
        return(state.colors)
    } else {
        states <- as.character(states)
        states.other <- setdiff(states, names(state.colors))
        if (length(states.other) > 0) {
            state.colors.other <- getDistinctColors(states.other)
            state.colors <- c(state.colors, state.colors.other)
        }
        return(state.colors[states])
    }
}


#' Transform genomic coordinates
#'
#' Add two columns with transformed genomic coordinates to the \code{\link{GRanges-class}} object. This is useful for making genomewide plots.
#'
#' @param gr A \code{\link{GRanges-class}} object.
#' @return The input \code{\link{GRanges-class}} with two additional metadata columns 'start.genome' and 'end.genome'.
transCoord <- function(gr) {
    cum.seqlengths <- cumsum(as.numeric(seqlengths(gr)))
    cum.seqlengths.0 <- c(0,cum.seqlengths[-length(cum.seqlengths)])
    names(cum.seqlengths.0) <- seqlevels(gr)
    gr$start.genome <- start(gr) + cum.seqlengths.0[as.character(seqnames(gr))]
    gr$end.genome <- end(gr) + cum.seqlengths.0[as.character(seqnames(gr))]
    return(gr)
}


#' @describeIn plotting Plot a histogram of count values and fitted distributions.
#' @param total.counts The number of total counts for which the histogram is to be plotted.
#' @importFrom stats dbinom
#' @export
plotHistogram <- function(model, total.counts, binwidth=1) {

    if (class(model) == 'GRanges') {
        model <- list(data=model)
    }
    ## Get cross section at total counts ##
    counts <- model$data$counts
    mask.crosssec <- counts[,'total'] == total.counts
    counts.methylated <- counts[mask.crosssec,'methylated']
    
    contexts <- intersect(levels(model$data$context), unique(model$data$context))
    ## Histogram ##
    df <- data.frame(counts=counts.methylated, context=model$data$context[mask.crosssec])
    ggplt <- ggplot(df) + geom_histogram(aes_string(x='counts', y='..density..'), binwidth=binwidth, color='black', fill='white')
    ggplt <- ggplt + theme_bw()
    ggplt <- ggplt + xlab(paste0("methylated counts\nat positions with total counts = ", total.counts))
    ggplt <- ggplt + coord_cartesian(xlim=c(0,total.counts))
    ggplt <- ggplt + facet_wrap(~ context)
    
    ## Fits ##
    if (!is.null(model$params$emissionParams)) {
        x <- seq(0, total.counts, by = binwidth)
        distr.list <- list()
        for (context in contexts) {
            distr <- list(x=x)
            for (i1 in 1:length(model$params$emissionParams)) {
                e <- model$params$emissionParams[[i1]]
                distr[[names(model$params$emissionParams)[i1]]] <- model$params$weights[[context]][i1] * stats::dbinom(x, size=total.counts, prob=e[context,'prob'])
            }
            distr <- as.data.frame(distr)
            distr$Total <- rowSums(distr[,-1])
            distr$context <- context
            distr.list[[context]] <- distr
        }
        distr <- do.call(rbind, distr.list)
        distr <- reshape2::melt(distr, id.vars=c('x', 'context'), variable.name='components')
        distr$components <- sub('^X', '', distr$components)
        distr$components <- factor(distr$components, levels=c('Total', levels(model$data$status)))
        ggplt <- ggplt + geom_line(data=distr, mapping=aes_string(x='x', y='value', col='components'))
        
        # ## Make legend
        # lprobs <- round(sapply(model$params$emissionParams, function(x) { x[,'prob'] }), 4)
        # lweights <- round(model$params$weights[[context]], 2)
        # legend <- paste0(names(model$params$emissionParams), ", prob=", lprobs, ", weight=", lweights)
        # legend <- c(paste0("Total, weight=", round(sum(lweights))), legend)
        # ggplt <- ggplt + scale_color_manual(name="components", values=getStateColors(levels(distr$components)), labels=legend) + theme(legend.position=c(0.5,0.99), legend.justification=c(0.5,1))
        
        ## Make legend
        # lprobs <- round(sapply(model$params$emissionParams, function(x) { x[,'prob'] }), 4)
        # rownames(lprobs) <- rownames(model$params$emissionParams[[1]])
        # lprobs <- lprobs[contexts,]
        # lweights <- round(t(as.data.frame(model$params$weights)), 2)
        # lweights <- lweights[contexts,]
        ggplt <- ggplt + scale_color_manual(name="components", values=getStateColors(levels(distr$components))) + theme(legend.position=c(0.5,0.99), legend.justification=c(0.5,1))
    }
    ggplt <- ggplt + ggtitle('Fitted distributions')
        
    return(ggplt)

}


#' @describeIn plotting Plot a scatter plot of read counts colored by methylation status.
#' @export
plotScatter <- function(model, datapoints=1000) {

    contexts <- intersect(levels(model$data$context), unique(model$data$context))
    ggplts <- list()
    limits <- list()
    data <- model$data
    ## Find sensible limits
    xmax <- quantile(data$counts[,'total']-data$counts[,'methylated'], 0.99)
    ymax <- quantile(data$counts[,'methylated'], 0.99)
    limits <- c(xmax, ymax)
    df <- data.frame(status=data$status, unmethylated=data$counts[,'total']-data$counts[,'methylated'], methylated=data$counts[,'methylated'], context=data$context)
    if (datapoints < nrow(df)) {
        df <- df[sample(1:nrow(df), datapoints, replace = FALSE), ]
    }
    
    ggplt <- ggplot(df, aes_string(x='methylated', y='unmethylated', col='status'))
    ggplt <- ggplt + geom_point(alpha=0.3)
    ggplt <- ggplt + coord_cartesian(xlim=c(0,xmax), ylim=c(0,ymax))
    ggplt <- ggplt + theme_bw()
    ggplt <- ggplt + xlab('methylated counts') + ylab('unmethylated counts')
    ggplt <- ggplt + facet_wrap(~ context)
    ggplt <- ggplt + scale_color_manual(values=getStateColors(levels(df$status)))
    
    # ## Legend
    # lweights <- round(model$params$weights[[context]], 2)
    # legend <- paste0(names(model$params$weights[[context]]), ", weight=", lweights)
    # ggplt <- ggplt + scale_color_manual(values=getStateColors(names(model$params$weights[[context]])), labels=legend)
    # ggplt <- ggplt + theme(legend.position=c(1,1), legend.justification=c(1,1))
    
    ggplt <- ggplt + ggtitle('Classification')
    
    return(ggplt)
    
}


#' @describeIn plotting Plot a heatmap of transition probabilities.
#' @importFrom reshape2 melt
#' @export
plotTransitionProbs <- function(model) {

    if (is.list(model$params$transProbs)) {
        As <- model$params$transProbs
    } else if (is.matrix(model$params$transProbs)) {
        As <- list(transitions=model$params$transProbs)
    }

    A <- reshape2::melt(As, varnames=c('from','to'), value.name='prob')
    names(A)[4] <- 'transitionContext'
    A$from <- factor(A$from, levels=rev(unique(A$from)))
    A$to <- factor(A$to, levels=rev(unique(A$from)))
    ggplt <- ggplot(data=A) + geom_tile(aes_string(x='to', y='from', fill='prob')) + scale_fill_gradient(low="white", high="blue", limits=c(0,1)) + theme_bw() + theme(axis.text.x = element_text(angle = 90, hjust = 1, vjust=0.5))
    ggplt <- ggplt + facet_wrap(~ transitionContext)
    
    ggplt <- ggplt + ggtitle('Transition probabilities')
    
    return(ggplt)

}

insertNULL <- function(plotlist) {
    n <- 0.5 * ( sqrt(8*length(plotlist)+1) - 1 )
    plotlist2 <- list()
    i3 <- 1
    for (i1 in 1:n) {
        if (i1 > 1) {
            for (i2 in 1:(n-i1)) {
                plotlist2[length(plotlist2)+1] <- list(NULL)
            }
        }
        for (i2 in i1:n) {
            plotlist2[[length(plotlist2)+1]] <- plotlist[[i3]]
            i3 <- i3 + 1
        }
    }
    return(plotlist2)
}

#' @describeIn plotting Plot the convergence of the probability parameters.
#' @export
plotConvergence <- function(model) {
    
    info <- model$convergenceInfo$prefit
    if (is.null(info)) {
        info <- model$convergenceInfo
    }
    # ## Plot loglikelihood
    # df <- data.frame(iteration=0:(length(info$logliks)-1), loglik=info$logliks)
    # ggplt <- ggplot(df) + geom_line(aes_string(x='iteration', y='loglik')) + theme_bw()
    # ggplt <- ggplt + scale_x_continuous(breaks=df$iteration, limits=c(2, length(info$logliks)-1))
    
    ## Plot parameters
    df <- reshape2::melt(info$parameterInfo, value.name='prob')
    ggplt <- ggplot(df) + geom_line(aes_string(x='iteration', y='prob', col='status')) + theme_bw()
    ggplt <- ggplt + theme(panel.grid.minor.x = element_blank())
    ggplt <- ggplt + facet_wrap(~ context)
    ggplt <- ggplt + scale_y_continuous(limits=c(0,1))
    ggplt <- ggplt + scale_color_manual(values=getStateColors(unique(df$status)))
    
    ggplt <- ggplt + ggtitle('Baum-Welch convergence')
    
    return(ggplt)
}


#' @describeIn plotting Plot an enrichment profile around an annotation.
#' @param annotation A \code{\link[GenomicRanges]{GRanges-class}} object with coordinates for the annotation.
#' @param windowsize Resolution in base-pairs for the curve upstream and downstream of the annotation.
#' @param insidewindows Number of data points for the curve inside the annotation.
#' @param range Distance upstream and downstream for which the enrichment profile is calculated.
#' @param category.column The name of a column in \code{data} that will be used for facetting of the plot.
#' @param plot Logical indicating whether a plot or the underlying data.frame is to be returned.
#' @param df.list A list() of data.frames, output from \code{plotEnrichment(..., plot=FALSE)}. If specified, option \code{data} will be ignored.
#' @export
plotEnrichment <- function(model, annotation, windowsize=100, insidewindows=20, range=1000, category.column=NULL, plot=TRUE, df.list=NULL) {
    ## For debugging
    # windowsize=100; insidewindows=20; range=1000; category.column=NULL
    # annotation <- arabidopsis_genes
    # model$data$imputed <- FALSE
    # model$data$imputed[model$data$counts[,'total'] == 0] <- TRUE
    # model$data$imputed <- factor(model$data$imputed)
    # category.column <- 'imputed'
    # data <- model$data

    if (class(model) == 'methimputeBinomialHMM') {
        data <- model$data
        distr <- model$params$emissionParams
    } else if (class(model) == 'GRanges') {
        data <- model
        distr <- NULL
    } else {
        stop("Argument 'model' needs to be of class 'GRanges' or 'methimputeBinomialHMM'.")
    }
    if (is.null(df.list)) {
        ## Variables
        categories <- 'none'
        if (!is.null(category.column)) {
            categories <- levels(mcols(data)[,category.column])
        }
        if (is.null(data$meth.lvl)) {
            data$meth.lvl <- data$counts[,'methylated'] / data$counts[,'total']
        }
        if (is.null(data$rc.meth.lvl) & !is.null(distr)) {
            if (!is.null(distr$Intermediate)) {
                data$rc.meth.lvl <- distr$Unmethylated[data$context,] * data$posteriorUnmeth + distr$Intermediate[data$context,] * (1 - data$posteriorUnmeth - data$posteriorMeth) + distr$Methylated[data$context,] * data$posteriorMeth
            } else {
                data$rc.meth.lvl <- distr$Unmethylated[data$context,] * data$posteriorUnmeth + distr$Methylated[data$context,] * data$posteriorMeth
            }
        }
        
        ## Subset annotation to data range
        annotation <- subsetByOverlaps(annotation, data)

        overlaps.rc.meth.lvl <- array(NA, dim=c(length(levels(data$context)), length(categories), range%/%windowsize, 2, 2), dimnames=list(context=levels(data$context), category=categories, distance=1:(range%/%windowsize), what=c('mean', 'weight'), where=c('upstream', 'downstream')))
        overlaps.meth.lvl <- array(NA, dim=c(length(levels(data$context)), length(categories), range%/%windowsize, 2, 2), dimnames=list(context=levels(data$context), category=categories, distance=1:(range%/%windowsize), what=c('mean', 'weight'), where=c('upstream', 'downstream')))
        ## Upstream and downstream annotation
        annotation.up <- resize(x = annotation, width = 1, fix = 'start')
        annotation.down <- resize(x = annotation, width = 1, fix = 'end')
        for (context in levels(data$context)) {
            message("Upstream and downstream counts for context ", context)
            data.context <- data[data$context == context]
            for (category in categories) {
                message("  category ", category)
                if (!is.null(category.column)) {
                    data.category <- data.context[mcols(data.context)[,category.column] == category]
                } else {
                    data.category <- data.context
                }
                for (i1 in 1:(range %/% windowsize)) {
                    anno.up <- suppressWarnings( promoters(x = annotation.up, upstream = i1*windowsize, downstream = 0) )
                    anno.up <- suppressWarnings( resize(x = anno.up, width = windowsize, fix = 'start') )
                    ind <- findOverlaps(anno.up, data.category)
                    overlaps.rc.meth.lvl[context, category, as.character(i1), 'mean', 'upstream'] <- mean(data.category$rc.meth.lvl[ind@to], na.rm=TRUE)
                    overlaps.rc.meth.lvl[context, category, as.character(i1), 'weight', 'upstream'] <- length(ind@to) / sum(as.numeric(width(anno.up)))
                    overlaps.meth.lvl[context, category, as.character(i1), 'mean', 'upstream'] <- mean(data.category$meth.lvl[ind@to], na.rm=TRUE)
                    overlaps.meth.lvl[context, category, as.character(i1), 'weight', 'upstream'] <- length(ind@to) / sum(as.numeric(width(anno.up)))
                }
                for (i1 in 1:(range %/% windowsize)) {
                    anno.down <- suppressWarnings( promoters(x = annotation.down, upstream = 0, downstream = i1*windowsize) )
                    anno.down <- suppressWarnings( resize(x = anno.down, width = windowsize, fix = 'end') )
                    ind <- findOverlaps(anno.down, data.category)
                    overlaps.rc.meth.lvl[context, category, as.character(i1), 'mean', 'downstream'] <- mean(data.category$rc.meth.lvl[ind@to], na.rm=TRUE)
                    overlaps.rc.meth.lvl[context, category, as.character(i1), 'weight', 'downstream'] <- length(ind@to) / sum(as.numeric(width(anno.down)))
                    overlaps.meth.lvl[context, category, as.character(i1), 'mean', 'downstream'] <- mean(data.category$meth.lvl[ind@to], na.rm=TRUE)
                    overlaps.meth.lvl[context, category, as.character(i1), 'weight', 'downstream'] <- length(ind@to) / sum(as.numeric(width(anno.down)))
                }
            }
        }
    
        ## Inside annotation
        ioverlaps.rc.meth.lvl <- array(NA, dim=c(length(levels(data$context)), length(categories), insidewindows, 2, 1), dimnames=list(context=levels(data$context), category=categories, distance=1:insidewindows, what=c('mean', 'weight'), where=c('inside')))
        ioverlaps.meth.lvl <- array(NA, dim=c(length(levels(data$context)), length(categories), insidewindows, 2, 1), dimnames=list(context=levels(data$context), category=categories, distance=1:insidewindows, what=c('mean', 'weight'), where=c('inside')))
        ioverlaps.status.Methylated <- array(NA, dim=c(length(levels(data$context)), length(categories), insidewindows, 2, 1), dimnames=list(context=levels(data$context), category=categories, distance=1:insidewindows, what=c('mean', 'weight'), where=c('inside')))
        ioverlaps.status.Intermediate <- array(NA, dim=c(length(levels(data$context)), length(categories), insidewindows, 2, 1), dimnames=list(context=levels(data$context), category=categories, distance=1:insidewindows, what=c('mean', 'weight'), where=c('inside')))
        mask.plus <- as.logical(strand(annotation) == '+' | strand(annotation) == '*')
        mask.minus <- !mask.plus
        widths <- width(annotation)
        for (context in levels(data$context)) {
            message("Inside counts for context ", context)
            data.context <- data[data$context == context]
            for (category in categories) {
                message("  category ", category)
                if (!is.null(category.column)) {
                    data.category <- data.context[mcols(data.context)[,category.column] == category]
                } else {
                    data.category <- data.context
                }
                for (i1 in 1:insidewindows) {
                    anno.in <- annotation
                    end(anno.in[mask.plus]) <- start(annotation[mask.plus]) + round(widths[mask.plus] * i1 / insidewindows)
                    start(anno.in[mask.plus]) <- start(annotation[mask.plus]) + round(widths[mask.plus] * (i1-1) / insidewindows)
                    start(anno.in[mask.minus]) <- end(annotation[mask.minus]) - round(widths[mask.minus] * i1 / insidewindows)
                    end(anno.in[mask.minus]) <- end(annotation[mask.minus]) - round(widths[mask.minus] * (i1-1) / insidewindows)
                    ind <- findOverlaps(anno.in, data.category)
                    ioverlaps.rc.meth.lvl[context, category, as.character(i1), 'mean', 'inside'] <- mean(data.category$rc.meth.lvl[ind@to], na.rm=TRUE)
                    ioverlaps.rc.meth.lvl[context, category, as.character(i1), 'weight', 'inside'] <- length(ind@to) / sum(as.numeric(width(anno.in)))
                    ioverlaps.meth.lvl[context, category, as.character(i1), 'mean', 'inside'] <- mean(data.category$meth.lvl[ind@to], na.rm=TRUE)
                    ioverlaps.meth.lvl[context, category, as.character(i1), 'weight', 'inside'] <- length(ind@to) / sum(as.numeric(width(anno.in)))
                    ioverlaps.status.Methylated[context, category, as.character(i1), 'mean', 'inside'] <- mean(data.category$status[ind@to] == "Methylated", na.rm=TRUE)
                    ioverlaps.status.Methylated[context, category, as.character(i1), 'weight', 'inside'] <- length(ind@to) / sum(as.numeric(width(anno.in)))
                    ioverlaps.status.Intermediate[context, category, as.character(i1), 'mean', 'inside'] <- mean(data.category$status[ind@to] == "Intermediate", na.rm=TRUE)
                    ioverlaps.status.Intermediate[context, category, as.character(i1), 'weight', 'inside'] <- length(ind@to) / sum(as.numeric(width(anno.in)))
                }
            }
        }
        
        overlaps.list <- list(meth.lvl = overlaps.meth.lvl, rc.meth.lvl = overlaps.rc.meth.lvl)
        ioverlaps.list <- list(meth.lvl = ioverlaps.meth.lvl, rc.meth.lvl = ioverlaps.rc.meth.lvl)
        
        df.list <- list()
        for (i1 in 1:length(overlaps.list)) {
            name <- names(overlaps.list)[i1]
            overlaps <- overlaps.list[[name]]
            ioverlaps <- ioverlaps.list[[name]]
            
            ## Normalize weight over categories
            overlaps[,,,what='weight',] <- sweep(x = overlaps[,,,what='weight',, drop=FALSE], MARGIN = c(1,3,4,5), STATS = apply(overlaps[,,,what='weight',, drop=FALSE], c(1,3,4,5), sum), FUN = '/')
            ioverlaps[,,,what='weight',] <- sweep(x = ioverlaps[,,,what='weight',, drop=FALSE], MARGIN = c(1,3,4,5), STATS = apply(ioverlaps[,,,what='weight',, drop=FALSE], c(1,3,4,5), sum), FUN = '/')
            
            ## Prepare data.frames for plotting
            dfo <- reshape2::melt(overlaps)
            df <- dfo[dfo$what == 'mean',]
            names(df)[6] <- 'mean'
            df$weight <- dfo$value[dfo$what == 'weight']
            df$distance <- as.numeric(df$distance)
            # Adjust distances for plot
            df$distance[df$where == 'upstream'] <- -df$distance[df$where == 'upstream'] * windowsize
            df$distance[df$where == 'downstream'] <- (df$distance[df$where == 'downstream']-1) * windowsize + range
            
            dfo <- reshape2::melt(ioverlaps)
            idf <- dfo[dfo$what == 'mean',]
            names(idf)[6] <- 'mean'
            idf$weight <- dfo$value[dfo$what == 'weight']
            idf$distance <- as.numeric(idf$distance)
            # Adjust distances for plot
            idf$distance <- (idf$distance-1) * range / insidewindows
            
            df.list[[name]] <- rbind(df, idf)
        }
    }
    
    if (!plot) {
        return(df.list)
    }
    
    plotlist <- list()
    for (i1 in 1:length(df.list)) {
        name <- names(df.list)[i1]
        df <- df.list[[name]]
        
        breaks <- c(c(-1, -0.5, 0, 0.5, 1, 1.5, 2) * range)
        labels <- c(-range, -range/2, '0%', '50%', '100%', range/2, range)
        ylabs <- c('Mean methylation\nlevel', 'Mean recalibrated\nmethylation level')
            
        ggplts <- list()
        # Enrichment profile
        ggplt <- ggplot(df) + geom_line(aes_string(x='distance', y='mean', col='context'))
        ggplt <- ggplt + scale_x_continuous(breaks=breaks, labels=labels)
        ggplt <- ggplt + scale_y_continuous(limits=c(0,1))
        ggplt <- ggplt + xlab('Distance from annotation')
        ggplt <- ggplt + ylab(ylabs[i1])
        if (!is.null(category.column)) {
            ggplt <- ggplt + facet_grid(.~category)
        }
        ggplt <- ggplt + theme_bw()
        ggplts[['profile']] <- ggplt
        plotlist[[name]] <- ggplt
        
        # # Cytosine density
        # ggplt <- ggplot(df) + geom_line(aes_string(x='distance', y='weight', col='context'))
        # ggplt <- ggplt + scale_x_continuous(breaks=breaks, labels=labels)
        # ggplt <- ggplt + xlab('Distance from annotation') + ylab('Normalized C frequency\nper bp of annotation')
        # if (!is.null(category.column)) {
        #     ggplt <- ggplt + facet_grid(.~category)
        # }
        # ggplts[['weight']] <- ggplt
        
        # cowplt <- cowplot::plot_grid(plotlist = ggplts, align = 'v', nrow=2, rel_heights=c(2,1))
        # plotlist[[name]] <- cowplt
    }

    if (plot) {
        return(plotlist)
    }
}


plotTransitionDistances <- function(model) {

    df <- as.data.frame(mcols(model$data)[c('transitionContext', 'distance')])
    df <- df[!is.na(df$transitionContext),]
    ggplt <- ggplot(df) + geom_histogram(aes_string(x='distance', y='..density..'), binwidth=1, fill='white', col='black') + theme_bw()
    ggplt <- ggplt + facet_wrap(~transitionContext, ncol = length(levels(model$data$context)))
    return(ggplt)

}


#' @describeIn plotting Maximum posterior vs. distance to nearest covered cytosine.
#' @param max.coverage.y Maximum coverage for positions on the y-axis.
#' @param min.coverage.x Minimum coverage for positions on the x-axis.
#' @param xmax Upper limit for the x-axis.
#' @param xbreaks.interval Interval for breaks on the x-axis.
#' @param cutoffs A vector with values that are plotted as horizontal lines. The names of the vector must match the context levels in \code{data$context}.
#' @export
plotPosteriorDistance <- function(model, datapoints=1e6, binwidth=5, max.coverage.y=0, min.coverage.x=3, xmax=200, xbreaks.interval=xmax/10, cutoffs=NULL) {

    if (class(model) == 'methimputeBinomialHMM') {
        data <- model$data
    } else if (class(model) == 'GRanges') {
        data <- model
    } else {
        stop("Argument 'model' needs to be of class 'GRanges' or 'methimputeBinomialHMM'.")
    }
    if (!is.null(cutoffs)) {
        if (length(setdiff(levels(data$context), names(cutoffs))) > 0 ) {
            stop("names(cutoffs) must be equal to levels(data$context)")
        }
    }
    ptm <- startTimedMessage("Plotting maximum posterior vs. distance to nearest covered ...")
    if (length(data) > datapoints) {
        data.small <- data[sort(sample(length(data), datapoints))]
    } else {
        data.small <- data
    }
    datac <- data.small[data.small$counts[,'total'] <= max.coverage.y]
    # Distance of nearest covered cytosine
    cov <- data[data$counts[,'total'] >= min.coverage.x]
    near <- distanceToNearest(datac, cov)
    datac$distance.nearest <- NA
    datac$distance.nearest[near@from] <- mcols(near)$distance
    # Plot
    df <- data.frame(posteriorMax=datac$posteriorMax, distance.nearest=datac$distance.nearest, context=datac$context)
    df$distance.nearest <- df$distance.nearest %/% binwidth * binwidth
    df$distance.nearest.factor <- factor(df$distance.nearest)
    ggplt <- ggplot(df[df$distance.nearest <= xmax,]) + geom_boxplot(aes_string(x='distance.nearest.factor', y='posteriorMax', fill='context'), outlier.shape = NA) + theme_bw()
    ggplt <- ggplt + facet_grid(.~context)
    ggplt <- ggplt + xlab(paste0('Distance in [bp] to nearest position with coverage >= ', min.coverage.x)) + ylab(paste0('Maximum posterior\nat positions with coverage <= ', max.coverage.y))
    ggplt <- ggplt + scale_x_discrete(breaks=seq(0,max(df$distance.nearest, na.rm=TRUE), by=xbreaks.interval))
    if (!is.null(cutoffs)) {
        df <- data.frame(context=names(cutoffs), cutoff=cutoffs)
        ggplt <- ggplt + geom_hline(data=df, mapping=aes_string(yintercept='cutoff'), linetype=1)
    }
    ggplt <- ggplt + ggtitle('Confidence vs. distance')
    stopTimedMessage(ptm)
    return(ggplt)
}


#' #' Plot a count histogram
#' #'
#' #' Plot a histogram of count values and fitted distributions.
#' #'
#' #' @importFrom stats dnbinom
#' plotHistogram2 <- function(model, binwidth=10) {
#' 
#'     ## Assign variables
#'     counts <- model$data$observable
#'     maxcounts <- max(counts)
#' 
#'     ## Plot histogram
#'     ggplt <- ggplot(data.frame(counts)) + geom_histogram(aes_string(x='counts', y='..density..'), binwidth=binwidth, color='black', fill='white') + xlab("counts")
#'     ggplt <- ggplt + coord_cartesian(xlim=c(0,quantile(counts, 0.995)))
#'     ggplt <- ggplt + theme_bw()
#' 
#'     ## Add distributions
#'     if (!is.null(model$params$emissionParams)) {
#'         x <- seq(0, maxcounts, by = binwidth)
#'         distr <- list(x=x)
#'         for (irow in 1:nrow(model$params$emissionParams)) {
#'             e <- model$params$emissionParams
#'             if (e$type[irow] == 'delta') {
#'                 distr[[rownames(model$params$emissionParams)[irow]]] <- rep(0, length(x))
#'                 distr[[rownames(model$params$emissionParams)[irow]]][1] <- model$params$weights[irow]
#'             } else if (e$type[irow] == 'dnbinom') {
#'                 distr[[rownames(model$params$emissionParams)[irow]]] <- model$params$weights[irow] * stats::dnbinom(x, size=e[irow,'size'], prob=e[irow,'prob'])
#'             }
#'         }
#'         distr <- as.data.frame(distr)
#'         distr$Total <- rowSums(distr[,2:(1+nrow(model$params$emissionParams))])
#'         distr <- reshape2::melt(distr, id.vars='x', variable.name='components')
#'         distr$components <- sub('^X', '', distr$components)
#'         distr$components <- factor(distr$components, levels=c(levels(model$data$state), 'Total'))
#'         ggplt <- ggplt + geom_line(data=distr, mapping=aes_string(x='x', y='value', col='components'))
#' 
#'         ## Make legend
#'         lmeans <- round(model$params$emissionParams[,'mu'], 2)
#'         lvars <- round(model$params$emissionParams[,'var'], 2)
#'         lweights <- round(model$params$weights, 2)
#'         legend <- paste0(rownames(model$params$emissionParams), ", mean=", lmeans, ", var=", lvars, ", weight=", lweights)
#'         legend <- c(legend, paste0('Total, mean=', round(mean(counts),2), ', var=', round(var(counts),2)))
#'         ggplt <- ggplt + scale_color_manual(name="components", values=getStateColors(c(rownames(model$params$emissionParams),'Total')), labels=legend) + theme(legend.position=c(1,1), legend.justification=c(1,1))
#'     }
#' 
#'     return(ggplt)
#' }
#' 
#' 
#' plotBoxplot <- function(model, datapoints=1000) {
#'   
#'     df <- data.frame(state=model$data$state, observable=model$data$observable)
#'     if (datapoints < nrow(df)) {
#'         df <- df[sample(1:nrow(df), datapoints, replace = FALSE), ]
#'     }
#'     df <- suppressMessages( reshape2::melt(df) )
#'     names(df) <- c('state', 'status', 'observable')
#'     ggplt <- ggplot(df) + geom_boxplot(aes_string(x='state', y='observable', fill='status'))
#'     return(ggplt)
#'     
#' }

#' #' Histograms faceted by state
#' #' 
#' #' Make histograms of variables in \code{bins} faceted by state in \code{segmentation}.
#' #' 
#' #' @param segmentation A segmentation with metadata column 'state'.
#' #' @param bins A \code{\link{binnedMethylome}}.
#' #' @param plotcol A character vector specifying the meta-data columns for which to produce the faceted plot.
#' #' @param binwidth Bin width for the histogram.
#' #' @return A \code{\link[ggplot2]{ggplot}}.
#' #' @importFrom reshape2 melt
#' plotStateHistograms <- function(segmentation, bins, plotcol, binwidth) {
#'   
#'     states <- levels(segmentation$state)
#'     segmentation.split <- split(segmentation, segmentation$state)
#'     mind <- findOverlaps(bins, segmentation.split, select='first')
#'     bins$state <- factor(names(segmentation.split)[mind], levels=states)
#'     
#'     df <- data.frame(state=bins$state, mcols(bins)[,plotcol])
#'     names(df)[2:ncol(df)] <- plotcol
#'     dfm <- reshape2::melt(df, id.vars = 'state', measure.vars = plotcol, variable.name = 'plotcol')
#'     ggplt <- ggplot(dfm) + geom_freqpoly(aes_string(x='value', y='..density..', col='plotcol'), binwidth = binwidth)
#'     ggplt <- ggplt + theme_bw()
#'     ggplt <- ggplt + facet_wrap(~ state)
#'     
#'     return(ggplt)
#' }


#' #' Scatter plots faceted by state
#' #' 
#' #' Make scatter plots of two variables in \code{bins} faceted by state in \code{segmentation}.
#' #' 
#' #' @param segmentation A segmentation with metadata column 'state'.
#' #' @param bins A \code{\link{binnedMethylome}}.
#' #' @param x A character specifying the meta-data column that should go on the x-axis.
#' #' @param y A character specifying the meta-data column that should go on the y-axis.
#' #' @param col A character specifying the meta-data column that should be used for coloring.
#' #' @return A \code{\link[ggplot2]{ggplot}}.
#' plotStateScatter <- function(segmentation, bins, x, y, col=NULL, datapoints=1000) {
#'   
#'     states <- levels(segmentation$state)
#'     segmentation.split <- split(segmentation, segmentation$state)
#'     mind <- findOverlaps(bins, segmentation.split, select='first')
#'     bins$seg.state <- factor(names(segmentation.split)[mind], levels=states)
#'     
#'     df <- data.frame(state = bins$seg.state, x = mcols(bins)[,x], y = mcols(bins)[,y], color=mcols(bins)[,col])
#'     df <- df[sample(1:nrow(df), size=datapoints, replace=FALSE),]
#'     if (!is.null(col)) {
#'         ggplt <- ggplot(df) + geom_jitter(aes_string(x='x', y='y', col='color'), alpha=0.1)
#'     } else {
#'         ggplt <- ggplot(df) + geom_jitter(aes_string(x='x', y='y'), alpha=0.1)
#'     }
#'     ggplt <- ggplt + theme_bw()
#'     ggplt <- ggplt + xlab(x) + ylab(y)
#'     ggplt <- ggplt + facet_wrap(~ state)
#'     ggplt <- ggplt + scale_color_manual(values=getDistinctColors(levels(df$color)))
#'     
#'     return(ggplt)
#' }









#' #' @describeIn enrichment_analysis Compute the fold enrichment of combinatorial states for multiple annotations.
#' #' @param model A \code{\link{combinedMultimodel}} or \code{\link{multimodel}} object or a file that contains such an object.
#' #' @param annotations A \code{list()} with \code{\link{GRanges-class}} objects containing coordinates of multiple annotations The names of the list entries will be used to name the return values.
#' #' @param plot A logical indicating whether the plot or an array with the fold enrichment values is returned.
#' #' @param logscale Whether (\code{TRUE}) or not (\code{FALSE}) to plot on log scale.
#' #' @param cluster Whether (\code{TRUE}) or not (\code{FALSE}) to cluster the annotations.
#' #' @importFrom S4Vectors subjectHits queryHits
#' #' @importFrom IRanges subsetByOverlaps
#' #' @importFrom reshape2 melt
#' #' @importFrom stats hclust as.dendrogram
#' #' @export
#' heatmapEnrichment <- function(model, annotations, plot=TRUE, logscale=TRUE, cluster=TRUE) {
#'   
#'   ## Variables
#'   bins <- model$data
#'   genome <- sum(as.numeric(width(bins)))
#'   annotationsAtBins <- lapply(annotations, function(x) { IRanges::subsetByOverlaps(x, bins) })
#'   feature.lengths <- lapply(annotationsAtBins, function(x) { sum(as.numeric(width(x))) })
#'   state.levels <- levels(bins$state)
#'   # Only state levels that actually appear
#'   state.levels <- state.levels[state.levels %in% unique(bins$state)]
#'   
#'   ggplts <- list()
#'   folds <- list()
#'   maxfolds <- list()
#'   
#'   ptm <- startTimedMessage("Calculating fold enrichment ...")
#'   fold <- array(NA, dim=c(length(annotationsAtBins), length(state.levels)), dimnames=list(annotation=names(annotationsAtBins), state=state.levels))
#'   for (istate in 1:length(state.levels)) {
#'     mask <- bins$state == state.levels[istate]
#'     bins.mask <- bins[mask]
#'     state.length <- sum(as.numeric(width(bins.mask)))
#'     for (ifeat in 1:length(annotationsAtBins)) {
#'       feature <- annotationsAtBins[[ifeat]]
#'       ind <- findOverlaps(bins.mask, feature)
#'       
#'       binsinfeature <- bins.mask[unique(S4Vectors::queryHits(ind))]
#'       sum.binsinfeature <- sum(as.numeric(width(binsinfeature)))
#'       
#'       featuresinbins <- feature[unique(S4Vectors::subjectHits(ind))]
#'       sum.featuresinbins <- sum(as.numeric(width(featuresinbins)))
#'       
#'       fold[ifeat,istate] <- sum.binsinfeature / state.length / feature.lengths[[ifeat]] * genome
#'     }
#'   }
#'   stopTimedMessage(ptm)
#'   
#'   ## Clustering
#'   if (cluster) {
#'     hc.anno <- stats::hclust(dist(fold))
#'     fold <- fold[hc.anno$order, ]
#'   }
#'   
#'   if (plot) {
#'     df <- reshape2::melt(fold, value.name='fold')
#'     if (logscale) {
#'       df$fold <- log(df$fold)
#'     }
#'     df$state <- factor(df$state, levels=colnames(fold))
#'     # Transform annotation to numeric for compatibility with the dendrogram
#'     df$ylabel <- df$annotation
#'     df$annotation <- as.numeric(df$annotation)
#'     maxfold <- max(df$fold, na.rm=TRUE)
#'     minfold <- max(df$fold, na.rm=TRUE)
#'     limits <- max(abs(maxfold), abs(minfold))
#'     ggplt <- ggplot(df) + geom_tile(aes_string(x='state', y='annotation', fill='fold'))
#'     ggplt <- ggplt + scale_y_continuous(breaks=1:length(unique(df$annotation)), labels=unique(df$ylabel), position='right')
#'     ggplt <- ggplt + theme_bw()
#'     ggplt <- ggplt + theme(axis.text.x = element_text(angle=90, hjust=1, vjust=0.5))
#'     if (logscale) {
#'       ggplt <- ggplt + scale_fill_gradientn(name='log(enrichment)', colors=grDevices::colorRampPalette(c("blue","white","red"))(20), values=c(seq(-limits,0,length.out=10), seq(0,limits,length.out=10)), rescaler=function(x,...) {x}, oob=identity, limits=c(-limits,limits), na.value="blue")
#'     } else {
#'       ggplt <- ggplt + scale_fill_gradientn(name='enrichment', colors=grDevices::colorRampPalette(c("blue","white","red"))(20), values=c(seq(0,1,length.out=10), seq(1,maxfold,length.out=10)), rescaler=function(x,...) {x}, oob=identity, limits=c(0,maxfold))
#'     }
#'     cowplt <- ggplt
#'     ## Dendrograms
#'     if (cluster) {
#'       theme_none <- theme(axis.text = element_blank(),
#'                           axis.ticks = element_blank(),
#'                           panel.background = element_blank(),
#'                           axis.title = element_blank())
#'       # Row dendrogram
#'       df.dendro <- ggdendro::segment(ggdendro::dendro_data(stats::as.dendrogram(hc.anno)))
#'       row.dendro <- ggplot(df.dendro) +  geom_segment(aes_string(x='y', y='x', xend='yend', yend='xend')) +  theme_none + scale_x_reverse() + coord_cartesian(ylim=c(0.5,max(df$annotation)+0.5))
#'       # Column dendrogram
#'       # col.dendro <- ggplot(ggdendro::segment(ggdendro::dendro_data(stats::as.dendrogram(hc.state)))) +  geom_segment(aes_string(x='x', y='y', xend='xend', yend='yend')) + theme_none
#'       cowplt <- cowplot::plot_grid(row.dendro, ggplt, align = 'h', ncol = 2, rel_widths = c(1,3))
#'     }
#'     return(cowplt)
#'   } else {
#'     return(fold)
#'   }
#' }

addDistance <- function(data, separate.contexts=FALSE) {
    ptm <- startTimedMessage("Adding distance ...")
    contexts <- intersect(levels(data$context), unique(data$context))
    ## Define distance as "from previous to current"
    if (separate.contexts) {
        distance <- numeric(length(data))
        for (context in contexts) {
            context.mask <- data$context == context
            distance[context.mask] <- c(-1, start(data)[context.mask][-1] - end(data)[context.mask][-length(which(context.mask))] - 1)
            distance[context.mask][distance[context.mask] < 0] <- Inf 
        }
    } else {
        distance <- c(-1, start(data)[-1] - end(data)[-length(data)] - 1)
        distance[distance < 0] <- Inf 
    }
    stopTimedMessage(ptm)
    return(distance)
}

addTransitionContext <- function(data, separate.contexts=FALSE) {
    ptm <- startTimedMessage("Adding transition context ...")
    
    ## Generating possible transition context levels
    contexts <- intersect(levels(data$context), unique(data$context))
    ncontexts <- length(contexts)
    transitionContexts <- character()
    for (c1 in 1:length(contexts)) {
        # for (c2 in c1:length(contexts)) {
        for (c2 in 1:length(contexts)) {
            if (separate.contexts & c1!=c2) {
                next
            }
            transitionContexts[length(transitionContexts)+1] <- paste0(contexts[c1], '-', contexts[c2])
        }
    }
    ## Define transition context as "from previous to current"
    if (separate.contexts) {
        transitionContext <- factor(NA, levels=transitionContexts)
        for (context in contexts) {
            context.mask <- data$context == context
            transitionContext[context.mask] <- factor(c(NA, paste0(data$context[context.mask][-length(which(context.mask))], '-', data$context[context.mask][-1])), levels=transitionContexts)
        }
    } else {
        transitionContext <- factor(c(NA, paste0(data$context[-length(data)], '-', data$context[-1])), levels=transitionContexts)
    }
    stopTimedMessage(ptm)
    return(transitionContext)
}

permute <- function(components) {
    ncol <- length(components)
    ncomponents <- c(sapply(components, length), 1)
    total.rows <- prod(ncomponents)
    perms <- list()
    for (i1 in ncol:1) {
        p <- prod(ncomponents[(i1+1):length(ncomponents)])
        pp1 <- prod(ncomponents[i1:length(components)])
        perms[[i1]] <- rep(rep(components[[i1]], each = p), total.rows / pp1)
    }
    perms <- as.data.frame(perms)
    names(perms) <- names(components)
    return(perms)
}

## ----style-knitr, eval=TRUE, echo=FALSE, results="asis"--------------------
BiocStyle::latex()

## ----options, results='hide', message=FALSE, eval=TRUE, echo=FALSE------------
library(methimpute)
options(width=80)

## ----methylation_status_calling_separate, results='markup', message=FALSE, eval=TRUE, fig.width=10, fig.height=7, out.width='\\textwidth', fig.align='center'----
library(methimpute)

# ===== Step 1: Importing the data ===== #

# We load an example file in BSSeeker format that comes with the package
file <- system.file("extdata","arabidopsis_bsseeker.txt.gz", package="methimpute")
bsseeker.data <- importBSSeeker(file)
print(bsseeker.data)

# Because most methylation extractor programs report only covered cytosines,
# we need to inflate the data to inlcude all cytosines (including non-covered sites)
fasta.file <- system.file("extdata","arabidopsis_sequence.fa.gz", package="methimpute")
cytosine.positions <- extractCytosinesFromFASTA(fasta.file,
                                                contexts = c('CG','CHG','CHH'))
methylome <- inflateMethylome(bsseeker.data, cytosine.positions)
print(methylome)


# ===== Step 2: Obtain correlation parameters ===== #

# The correlation of methylation levels between neighboring cytosines is an important
# parameter for the methylation status calling, so we need to get it first. Keep in mind
# that we only use the first 200.000 bp here, that's why the plot looks a bit meagre.
distcor <- distanceCorrelation(methylome, separate.contexts = TRUE)
fit <- estimateTransDist(distcor)
print(fit)


# ===== Step 3: Methylation status calling (and imputation) ===== #

model <- callMethylationSeparate(data = methylome, transDist = fit$transDist,
                                 verbosity = 0)
# The confidence in the methylation status call is given in the column "posteriorMax".
# For further analysis one could split the results into high-confidence
# (posteriorMax >= 0.98) and low-confidence calls (posteriorMax < 0.98) for instance.
print(model)

# Bisulfite conversion rates can be obtained with
1 - model$params$emissionParams$Unmethylated

## ----methylation_status_calling_separate_checkresults, results='markup', message=FALSE, eval=TRUE, fig.width=10, fig.height=2.5, out.width='\\textwidth', fig.align='center'----
plotConvergence(model)
plotTransitionProbs(model)
plotHistogram(model, total.counts = 10)

## ----methylation_status_calling, results='markup', message=FALSE, eval=TRUE, fig.width=10, fig.height=7, out.width='\\textwidth', fig.align='center'----
library(methimpute)

# ===== Step 1: Importing the data ===== #

# We load an example file in BSSeeker format that comes with the package
file <- system.file("extdata","arabidopsis_bsseeker.txt.gz", package="methimpute")
bsseeker.data <- importBSSeeker(file)
print(bsseeker.data)

# Because most methylation extractor programs report only covered cytosines,
# we need to inflate the data to inlcude all cytosines (including non-covered sites)
fasta.file <- system.file("extdata","arabidopsis_sequence.fa.gz", package="methimpute")
cytosine.positions <- extractCytosinesFromFASTA(fasta.file,
                                                contexts = c('CG','CHG','CHH'))
methylome <- inflateMethylome(bsseeker.data, cytosine.positions)
print(methylome)


# ===== Step 2: Obtain correlation parameters ===== #

# The correlation of methylation levels between neighboring cytosines is an important
# parameter for the methylation status calling, so we need to get it first. Keep in mind
# that we only use the first 200.000 bp here, that's why the plot looks a bit meagre.
distcor <- distanceCorrelation(methylome)
fit <- estimateTransDist(distcor)
print(fit)


# ===== Step 3: Methylation status calling (and imputation) ===== #

model <- callMethylation(data = methylome, transDist = fit$transDist)
# The confidence in the methylation status call is given in the column "posteriorMax".
# For further analysis one could split the results into high-confidence
# (posteriorMax >= 0.98) and low-confidence calls (posteriorMax < 0.98) for instance.
print(model)

# Bisulfite conversion rates can be obtained with
1 - model$params$emissionParams$Unmethylated

## ----methylation_status_calling_binned, results='markup', message=FALSE, eval=TRUE, fig.width=10, fig.height=7, out.width='\\textwidth', fig.align='center'----
library(methimpute)

# ===== Step 1: Importing the data ===== #

# We load an example file in BSSeeker format that comes with the package
file <- system.file("extdata","arabidopsis_bsseeker.txt.gz", package="methimpute")
bsseeker.data <- importBSSeeker(file)
print(bsseeker.data)

# Because most methylation extractor programs report only covered cytosines,
# we need to inflate the data to inlcude all cytosines (including non-covered sites)
fasta.file <- system.file("extdata","arabidopsis_sequence.fa.gz", package="methimpute")
cytosine.positions <- extractCytosinesFromFASTA(fasta.file,
                                                contexts = c('CG','CHG','CHH'))
methylome <- inflateMethylome(bsseeker.data, cytosine.positions)
print(methylome)


# ===== Step 2: Binning into 100bp bins ===== #
binnedMethylome <- binMethylome(methylome, binsize = 100, contexts = c('total','CG'))
print(binnedMethylome$CG)


# ===== Step 3: Methylation status calling (and imputation) ===== #

binnedModel <- callMethylation(data = binnedMethylome$CG)
print(binnedModel)

## ----methylation_status_calling_binned_checkresults, results='markup', message=FALSE, eval=TRUE, fig.width=5, fig.height=2.5, out.width='0.5\\textwidth', fig.align='center'----
plotConvergence(binnedModel)
plotTransitionProbs(binnedModel)
plotHistogram(binnedModel, total.counts = 150)

## ----enrichment_analysis, results='markup', message=FALSE, eval=TRUE, fig.width=10, fig.height=4, out.width='0.8\\textwidth', fig.align='center'----
# Define categories to distinguish missing from covered cytosines
model$data$category <- factor('covered', levels=c('missing', 'covered'))
model$data$category[model$data$counts[,'total']>=1] <- 'covered'
model$data$category[model$data$counts[,'total']==0] <- 'missing'

# Note that the plots look a bit ugly because our toy data has only 200.000 datapoints.
data(arabidopsis_genes)
seqlengths(model$data) <- seqlengths(arabidopsis_genes)[seqlevels(model$data)] # this
                          # line should only be necessary for our toy example
plotEnrichment(model$data, annotation=arabidopsis_genes, range = 2000,
               category.column = 'category')
data(arabidopsis_TEs)
plotEnrichment(model$data, annotation=arabidopsis_TEs, range = 2000,
               category.column = 'category')

## ----export_results, results='markup', message=FALSE, eval=TRUE---------------
exportMethylome(model, filename = tempfile())

## ----sessionInfo, eval=TRUE, results="asis"-----------------------------------
toLatex(sessionInfo())

#' Extract cytosine coordinates
#' 
#' Extract cytosine coordinates and context information from a FASTA file. Cytosines in ambiguous reference contexts are not reported.
#' 
#' @param file A character with the file name.
#' @param contexts The contexts that should be extracted. If the contexts are named, the returned object will use those names for the contexts.
#' @param anchor.C A named vector with positions of the anchoring C in the \code{contexts}. This is necessary to distinguish contexts such as C*C*CG (anchor.C = 2) and *C*CCG (anchor.C = 1). Names must match the contexts. If unspecified, the first C within each context will be taken as anchor.
#' @return A \code{\link{GRanges-class}} object with coordinates of extracted cytosines and meta-data column 'context'.
#' 
#' @importFrom Biostrings readDNAStringSet vmatchPattern reverseComplement
#' @export
#' @examples
#' ## Read a non-compressed FASTA files:
#' filepath <- system.file("extdata", "arabidopsis_sequence.fa.gz", package="methimpute")
#' 
#' ## Only CG context
#' cytosines <- extractCytosinesFromFASTA(filepath, contexts = 'CG')
#' table(cytosines$context)
#' 
#' ## Split CG context into subcontexts
#' cytosines <- extractCytosinesFromFASTA(filepath,
#'                contexts = c('DCG', 'CCG'),
#'                anchor.C = c(DCG=2, CCG=2))
#' table(cytosines$context)
#'                
#' ## With contexts that differ only by anchor
#' cytosines <- extractCytosinesFromFASTA(filepath,
#'                contexts = c('DCG', 'CCG', 'CCG', 'CWG', 'CHH'),
#'                anchor.C = c(DCG=2, CCG=2, CCG=1, CWG=1, CHH=1))
#' table(cytosines$context)
#'                
#' ## With named contexts
#' contexts <- c(CG='DCG', CG='CCG', CHG='CCG', CHG='CWG', CHH='CHH')
#' cytosines <- extractCytosinesFromFASTA(filepath,
#'                contexts = contexts,
#'                anchor.C = c(DCG=2, CCG=2, CCG=1, CWG=1, CHH=1))
#' table(cytosines$context)
#' 
extractCytosinesFromFASTA <- function(file, contexts = c('CG','CHG','CHH'), anchor.C = NULL) {

    ### C anchors ###
    if (is.null(anchor.C)) {
        anchor.C <- regexpr('C', contexts)[1:length(contexts)]
        names(anchor.C) <- contexts
    }
    if (!all(names(anchor.C)==contexts)) {
        stop("names(anchor.C) must be equal to contexts")
    }
    ### Contexts ###
    if (is.null(names(contexts))) {
        context.names <- contexts
        dup.contexts <- duplicated(contexts) | duplicated(contexts, fromLast=TRUE)
        context.names[dup.contexts] <- paste0(contexts, '_', anchor.C)[dup.contexts]
    } else {
        context.names <- names(contexts)
    }

    ### Read file
    fasta <- Biostrings::readDNAStringSet(file)
    
    ### Chromosome lengths
    chromlengths <- width(fasta)
    names(chromlengths) <- names(fasta)
    
    ### Positions with ambiguous nucleotide codes
    ptm <- startTimedMessage("Scanning for ambiguous nucleotides ...")
    notuse <- GRangesList()
    ambicodes <- c('R','Y','S','W','K','M','B','D','H','V','N')
    for (ambicode in ambicodes) {
        notuse.string <- Biostrings::vmatchPattern(ambicode, subject = fasta, fixed = TRUE)
        notuse[[ambicode]] <- as(notuse.string, 'GRanges')
    }
    notuse <- unlist(notuse, use.names = FALSE)
    stopTimedMessage(ptm)
    
    ### Extract cytosine positions with context
    ptm <- startTimedMessage("Extracting cytosines from forward strand ...")
    cytosines.forward <- GRangesList()
    for (icontext in 1:length(contexts)) {
        context <- contexts[icontext]
        context.name <- context.names[icontext]
        positions <- Biostrings::vmatchPattern(context, subject = fasta, fixed = FALSE)
        positions <- as(positions, 'GRanges')
        strand(positions) <- '+'
        positions$context <- factor(context.name, levels=unique(context.names))
        cytosines.forward[[icontext]] <- positions
    }
    cytosines.forward <- unlist(cytosines.forward, use.names = FALSE)
    seqlengths(cytosines.forward) <- chromlengths
    # Remove ambiguous positions
    ind <- findOverlaps(cytosines.forward, notuse)
    if (length(ind) > 0) {
        cytosines.forward <- cytosines.forward[-ind@from]
    }
    # Set width to 1
    end(cytosines.forward) <- start(cytosines.forward)
    stopTimedMessage(ptm)
    
    ### Extract reverse context
    ptm <- startTimedMessage("Extracting cytosines from reverse strand ...")
    fasta <- Biostrings::reverseComplement(fasta)
    cytosines.reverse <- GRangesList()
    for (icontext in 1:length(contexts)) {
        context <- contexts[icontext]
        context.name <- context.names[icontext]
        positions <- Biostrings::vmatchPattern(context, subject = fasta, fixed = FALSE)
        positions <- as(positions, 'GRanges')
        strand(positions) <- '-'
        positions$context <- factor(context.name, levels=unique(context.names))
        cytosines.reverse[[icontext]] <- positions
    }
    cytosines.reverse <- unlist(cytosines.reverse, use.names = FALSE)
    seqlengths(cytosines.reverse) <- chromlengths
    # Correct coordinates from reverseComplement
    starts <- chromlengths[as.character(seqnames(cytosines.reverse))] - start(cytosines.reverse) + 1
    ends <- chromlengths[as.character(seqnames(cytosines.reverse))] - end(cytosines.reverse) + 1
    cytosines.reverse <- GRanges(seqnames = seqnames(cytosines.reverse), ranges = IRanges(start = ends, end = starts), strand = '-', context = cytosines.reverse$context)
    # Remove ambiguous positions
    ind <- findOverlaps(cytosines.reverse, notuse)
    if (length(ind) > 0) {
        cytosines.reverse <- cytosines.reverse[-ind@from]
    }
    # Set width to 1
    start(cytosines.reverse) <- end(cytosines.reverse)
    stopTimedMessage(ptm)
    
    ## Merge
    ptm <- startTimedMessage("Merging ...")
    cytosines <- c(cytosines.forward, cytosines.reverse)
    stopTimedMessage(ptm)
    
    # Shift positions by position of anchor C
    ptm <- startTimedMessage("Shifting by anchor ...")
    cind <- anchor.C[cytosines$context]
    strandint <- c('-'=-1,'+'=1,'*'=1)[as.character(strand(cytosines))]
    starts <- start(cytosines) + strandint * cind - strandint * 1
    ends <- end(cytosines) + strandint * cind - strandint * 1
    cytosines <- GRanges(seqnames=seqnames(cytosines), ranges=IRanges(start=starts, end=ends), strand=strand(cytosines), context=cytosines$context)
    seqlengths(cytosines) <- chromlengths
    stopTimedMessage(ptm)
    
    ## Sort
    ptm <- startTimedMessage("Sorting ...")
    cytosines <- sort(cytosines, ignore.strand=TRUE)
    stopTimedMessage(ptm)
    
    return(cytosines)
}

#' Inflate an imported methylation extractor file
#' 
#' Inflate an imported methylation extractor file to contain all cytosine positions. This is useful to obtain a full methylome, including non-covered cytosines, because most methylation extractor programs only report covered cytosines.
#' 
#' @param methylome A \code{\link[GenomicRanges]{GRanges-class}} with methylation counts. 
#' @param methylome.full A \code{\link[GenomicRanges]{GRanges-class}} with positions for all cytosines or a file with such an object.
#' @return The \code{methylome.full} object with added metadata column 'counts'.
#' 
#' @export
#' @examples
#'## Get an example file in BSSeeker format
#'file <- system.file("extdata","arabidopsis_bsseeker.txt.gz", package="methimpute")
#'bsseeker.data <- importBSSeeker(file)
#'bsseeker.data
#'
#'## Inflate to full methylome (including non-covered sites)
#'data(arabidopsis_toydata)
#'full.methylome <- inflateMethylome(bsseeker.data, arabidopsis_toydata)
#'full.methylome
#'
inflateMethylome <- function(methylome, methylome.full) {
    
    if (is.character(methylome.full)) {
        ptm <- startTimedMessage("Loading full methylome from file ", methylome.full, " ...")
        temp.env <- new.env()
        methylome.full <- get(load(methylome.full, envir=temp.env), envir=temp.env) 
        stopTimedMessage(ptm)
    }
    if (is.character(methylome)) {
        ptm <- startTimedMessage("Loading methylome from file ", methylome, " ...")
        temp.env <- new.env()
        methylome <- get(load(methylome, envir=temp.env), envir=temp.env) 
        stopTimedMessage(ptm)
    }
    ptm <- startTimedMessage("Inflating methylome ...")
    counts <- array(0, dim=c(length(methylome.full), 2), dimnames=list(NULL, c("methylated", "total")))
    ind <- findOverlaps(methylome.full, methylome)
    counts[ind@from,] <- methylome$counts[ind@to,]
    methylome.full$counts <- counts
    stopTimedMessage(ptm)
    return(methylome.full)
    
}

#' Methimpute binning functions
#' 
#' This page provides an overview of all \pkg{\link{methimpute}} binning functions.
#'
#' @param data A \code{\link[GenomicRanges]{GRanges-class}} object with metadata columns 'context' and 'counts' (which is a matrix with columns 'methylated' and 'total').
#' @param binsize The window size used for binning.
#' @return A \code{\link[GenomicRanges]{GRanges-class}} object for \code{binCounts} and \code{binPostions}. A \code{list()} of \code{\link[GenomicRanges]{GRanges-class}} objects for \code{binMethylome}.
#' @name binning
#' @examples 
#'## Get some toy data
#'file <- system.file("data","arabidopsis_toydata.RData",
#'                     package="methimpute")
#'data <- get(load(file))
#'print(data)
#'## Bin the data in various ways
#'binCounts(data, binsize=1000)
#'binPositions(data, binsize=1000)
#'binMethylome(data, binsize=1000, contexts=c("total", "CG"),
#'             columns.average=NULL)
#'
NULL


#' @describeIn binning Get the aggregated number of counts in each bin (no context).
#' 
#' @importFrom stats aggregate
#' @export
binCounts <- function(data, binsize) {

    ptm <- startTimedMessage("Making fixed-width bins with ", binsize, "bp ...")
    chrom.lengths.floor <- floor(seqlengths(data) / binsize) * binsize
    tiles <- GenomicRanges::tileGenome(chrom.lengths.floor, tilewidth=binsize)
    bins <- unlist(GenomicRanges::tileGenome(chrom.lengths.floor, tilewidth=binsize), use.names=FALSE)
    seqlengths(bins) <- seqlengths(data)
    if (any(width(bins)!=binsize)) {
        stop("tileGenome failed")
    }
    stopTimedMessage(ptm)
    
    ptm <- startTimedMessage("Aggregating counts ...")
    ind <- findOverlaps(data, bins, select='first')
    df <- stats::aggregate(as.data.frame(data$counts), by=list(ind), FUN=sum)
    bins$counts <- matrix(0, ncol=2, nrow=length(bins))
    colnames(bins$counts) <- colnames(data$counts)
    bins$counts[df$Group.1,'methylated'] <- df$methylated
    bins$counts[df$Group.1,'total'] <- df$total
    stopTimedMessage(ptm)
    
    return(bins)
}


#' @describeIn binning Get the number of cytosines in each bin (total and per context).
#' @export
binPositions <- function(data, binsize) {

    ptm <- startTimedMessage("Making fixed-width bins with ", binsize, "bp ...")
    chrom.lengths.floor <- floor(seqlengths(data) / binsize) * binsize
    tiles <- GenomicRanges::tileGenome(chrom.lengths.floor, tilewidth=binsize)
    bins <- unlist(GenomicRanges::tileGenome(chrom.lengths.floor, tilewidth=binsize), use.names=FALSE)
    seqlengths(bins) <- seqlengths(data)
    if (any(width(bins)!=binsize)) {
        stop("tileGenome failed")
    }
    stopTimedMessage(ptm)
    
    ptm <- startTimedMessage("Counting total overlaps ...")
    bins$total <- countOverlaps(bins, data)
    stopTimedMessage(ptm)
    
    ptm <- startTimedMessage("Counting context-specific overlaps ...")
    contexts <- levels(factor(data$context))
    bins$context <- matrix(0, ncol=length(contexts), nrow=length(bins), dimnames=list(NULL, context=contexts))
    for (context in contexts) {
        mask <- data$context == context
        bins$context[,context] <- countOverlaps(bins, data[mask])
    }
    stopTimedMessage(ptm)
    
    ptm <- startTimedMessage("Adding distance ...")
    bins$distance <- c(-1, start(bins)[-1] - end(bins)[-length(bins)] - 1)
    bins$distance[bins$distance < 0] <- Inf 
    stopTimedMessage(ptm)
    
    return(bins)
}


#' @describeIn binning Get number of cytosines and aggregated counts for the specified contexts.
#' 
#' @param contexts A character vector with contexts for which the binning will be done.
#' @param columns.average A character vector with names of columns in \code{data} that should be averaged in bins.
#' 
#' @importFrom stats aggregate
#' @export
binMethylome <- function(data, binsize, contexts='total', columns.average=NULL) {

    ptm <- startTimedMessage("Making fixed-width bins with ", binsize, "bp ...")
    chrom.lengths.floor <- floor(seqlengths(data) / binsize) * binsize
    tiles <- GenomicRanges::tileGenome(chrom.lengths.floor, tilewidth=binsize)
    bins.template <- unlist(GenomicRanges::tileGenome(chrom.lengths.floor, tilewidth=binsize), use.names=FALSE)
    seqlengths(bins.template) <- seqlengths(data)
    if (any(width(bins.template)!=binsize)) {
        stop("tileGenome failed")
    }
    stopTimedMessage(ptm)
    
    bins.list <- list()
    for (context in contexts) {
        messageU("Working on context '", context, "'", underline=NULL, overline='-')
        bins <- bins.template
        if (context == 'total') {
            data.context <- data
        } else {
            mask <- data$context == context
            data.context <- data[mask]
        }

        ptm <- startTimedMessage("Counting cytosine overlaps ...")
        bins$cytosines <- countOverlaps(bins, data.context)
        stopTimedMessage(ptm)
        
        bins$context <- factor(context)
        
        ptm <- startTimedMessage("Aggregating counts ...")
        ind <- findOverlaps(data, bins, select='first')
        df <- stats::aggregate(as.data.frame(data$counts), by=list(ind), FUN=sum)
        bins$counts <- matrix(0, ncol=2, nrow=length(bins))
        colnames(bins$counts) <- colnames(data$counts)
        bins$counts[df$Group.1,'methylated'] <- df$methylated
        bins$counts[df$Group.1,'total'] <- df$total
        stopTimedMessage(ptm)
        for (col in columns.average) {
            ptm <- startTimedMessage("Averaging column ", col, " ...")
            if (!is.null(mcols(data)[col])) {
                df <- stats::aggregate(mcols(data)[[col]], by=list(ind), FUN=mean)
                mcols(bins)[col] <- NA
                mcols(bins)[df$Group.1, col] <- df[,2]
            }
            stopTimedMessage(ptm)
        }
        
        bins.list[[context]] <- bins
    }
    
    return(bins.list)
}

#' Get distinct colors
#' 
#' Get a set of distinct colors selected from \code{\link{colors}}.
#' 
#' The function computes the euclidian distance between all \code{\link{colors}} and iteratively selects those that have the furthest closes distance to the set of already selected colors.
#' 
#' @param n Number of colors to select. If \code{n} is a character vector, \code{length(n)} will be taken as the number of colors and the colors will be named by \code{n}.
#' @param start.color Color to start the selection process from.
#' @param exclude.colors Character vector with colors that should not be used.
#' @param exclude.rgb.above Exclude colors where all RGB values are above. This is useful to exclude whitish colors.
#' @param exclude.brightness.above Exclude colors where the 'brightness' value in HSV space is above. This is useful to obtain a matt palette.
#' @return A character vector with colors.
#' @author Aaron Taudt
#' @importFrom grDevices col2rgb rgb2hsv
#' @importFrom stats dist
#' @export
#' @examples
#'cols <- getDistinctColors(5)
#'pie(rep(1,5), labels=cols, col=cols)
#'
getDistinctColors <- function(n, start.color='blue4', exclude.colors=c('white','black','gray','grey','\\<yellow\\>', 'yellow1', 'lemonchiffon'), exclude.brightness.above=1, exclude.rgb.above=210) {
    
    n.names <- NULL
    if (is.character(n)) {
        n.names <- n
        n <- length(n)
    } else if (is.factor(n)) {
        n.names <- as.character(n)
        n <- length(n)
    }
    cols <- grDevices::colors()
    
    # Exclude unwanted colors
    cols <- grep(paste(exclude.colors, collapse='|'), cols, invert=TRUE, value=TRUE)
    # Exclude too bright colors
    colsrgb <- grDevices::col2rgb(cols)
    colshsv <- t(mapply(grDevices::rgb2hsv, r=colsrgb[1,], g=colsrgb[2,], b=colsrgb[3,]))
    rownames(colshsv) <- cols
    colshsv <- colshsv[colshsv[,3] <= exclude.brightness.above,]
    cols <- rownames(colshsv)
    # Get RGB values
    rgbs <- t(grDevices::col2rgb(cols))
    rownames(rgbs) <- cols
    # Exclude whitish colors
    rgbs <- rgbs[apply(rgbs, 1, function(x) { !all(x>exclude.rgb.above) }), ]
    # Calculate distance
    coldist <- as.matrix(stats::dist(rgbs, method='euclidean'))
    
    if (n == 1) {
        return(start.color)
    }
    # Iteratively select colors
    selected.cols <- character()
    selected.cols[1] <- start.color
    for (i1 in 2:n) {
        m <- as.matrix(coldist[,selected.cols])
        closest.dist <- apply(m, 1, min)
        furthest.dist <- which.max(closest.dist)
        selected.cols[i1] <- names(furthest.dist)
        if (selected.cols[i1] == selected.cols[i1-1]) {
            selected.cols <- selected.cols[-length(selected.cols)]
            break
        }
    }
    colors <- rep(selected.cols, ceiling(n / length(selected.cols)))[1:n]
    if (length(colors) > length(selected.cols)) {
        warning("Recycling colors because we only have ", length(selected.cols), " distinct colors to choose from.")
    }
    names(colors) <- n.names
    return(colors)
    
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

fitBinomialTestHMMcontextTransition <- function(counts_total, counts_meth, context, transitionContext, distances, params, algorithm, update_procedure) {
    .Call('_methimpute_fitBinomialTestHMMcontextTransition', PACKAGE = 'methimpute', counts_total, counts_meth, context, transitionContext, distances, params, algorithm, update_procedure)
}

cleanup <- function() {
    invisible(.Call('_methimpute_cleanup', PACKAGE = 'methimpute'))
}

fitHMM <- function(counts, distances, params, algorithm) {
    .Call('_methimpute_fitHMM', PACKAGE = 'methimpute', counts, distances, params, algorithm)
}

#' @useDynLib methimpute
#' @import Rcpp
#' @import IRanges
#' @import GenomeInfoDb
#' @import GenomicRanges
#' @importFrom methods as is
NULL

# =======================================================
# Some global variables that can be used in all functions
# =======================================================

#' Distance correlation
#' 
#' Compute the distance correlation from a \code{\link{methimputeData}} object.
#' 
#' @param data A \code{\link{methimputeData}} object.
#' @param distances An integer vector specifying the distances for which the correlation will be calculated.
#' @param separate.contexts A logical indicating whether contexts are treated separately. If set to \code{TRUE}, correlations will only be calculated between cytosines of the same context.
#' @return A list() with an array containing the correlation values and the corresponding \code{\link[ggplot2]{ggplot}}.
#' 
#' @export
#' @examples 
#'## Get some toy data
#'file <- system.file("data","arabidopsis_toydata.RData",
#'                     package="methimpute")
#'data <- get(load(file))
#'distcor <- distanceCorrelation(data)
#'print(distcor$plot)
#'
distanceCorrelation <- function(data, distances=0:50, separate.contexts=FALSE) {
    
    ## Contexts
    contexts <- intersect(levels(data$context), unique(data$context))
    
    ## Add meth.lvl column
    data$meth.lvl <- data$counts[,'methylated'] / data$counts[,'total']
    
    ### Add distance and transition context to bins ###
    data$distance <- addDistance(data, separate.contexts=separate.contexts)
    data$transitionContext <- addTransitionContext(data, separate.contexts=separate.contexts)

    ## Loop through distances
    ptm <- startTimedMessage("Calculating correlations\n")
    # cors <- list()
    cor.array <- array(NA, dim=c(length(contexts), length(contexts), length(distances), 2), dimnames=list(context=contexts, context=contexts, distance=distances, what=c('correlation','weight')))
    for (i1 in distances) {
        if (i1 == distances[1]) {
            message("  for distance ", i1, appendLF=FALSE)
        } else {
            message(", ", i1, appendLF=FALSE)
        }
        ind <- which(data$distance == i1)
        meth.lvl <- data$meth.lvl[ind]
        meth.lvl.shift <- data$meth.lvl[ind-1]
        # ## All contexts
        # cor <- tryCatch(cor(meth.lvl, meth.lvl.shift, use='complete.obs'), error = function(e) NA, warning = function(w) NA)
        # cors[[as.character(i1)]] <- data.frame(correlation = cor, weight = length(ind))
        ## Context specific
        context.transition <- data$transitionContext[ind]
        cor.matrix <- matrix(NA, ncol=length(contexts), nrow=length(contexts), dimnames=list(context=contexts, context=contexts))
        weight.matrix <- matrix(NA, ncol=length(contexts), nrow=length(contexts), dimnames=list(context=contexts, context=contexts))
        for (c1 in 1:length(contexts)) {
            for (c2 in c1:length(contexts)) {
                context <- paste0(contexts[c1], '-', contexts[c2])
                context.rev <- paste0(contexts[c2], '-', contexts[c1])
                mask <- context.transition == context | context.transition == context.rev
                ratio1 <- meth.lvl[mask]
                ratio2 <- meth.lvl.shift[mask]
                cor.matrix[contexts[c1], contexts[c2]] <- tryCatch(cor(ratio1, ratio2, use='complete.obs'), error = function(e) NA, warning = function(w) NA)
                cor.matrix[contexts[c2], contexts[c1]] <- cor.matrix[contexts[c1], contexts[c2]]
                weight.matrix[contexts[c1], contexts[c2]] <- length(ratio1)
                weight.matrix[contexts[c2], contexts[c1]] <- weight.matrix[contexts[c1], contexts[c2]]
            }
        }
        cor.array[,,as.character(i1),'correlation'] <- cor.matrix
        cor.array[,,as.character(i1),'weight'] <- weight.matrix
    }
    message("\nFinished calculating correlations in", appendLF = FALSE)
    stopTimedMessage(ptm)
    
    ## Context correlation plots
    maxweights <- numeric()
    dfs <- list()
    for (c1 in 1:length(contexts)) {
        for (c2 in 1:length(contexts)) {
            context.transition <- paste0(contexts[c1], '-', contexts[c2])
            if (c1 <= c2) {
                df <- data.frame(distance = distances, correlation = cor.array[c1,c2,,'correlation'], weight = cor.array[c1,c2,,'weight'], from = contexts[c1], to = contexts[c2])
                df$logweight <- log(df$weight+1)
                maxweights[context.transition] <- max(df$logweight, na.rm = TRUE)
                dfs[[context.transition]] <- df
            }
        }
    }
    maxweight <- max(maxweights, na.rm = TRUE)
    miny <- min(cor.array, na.rm = TRUE)
    
    ## Plot correlation
    df <- do.call(rbind, dfs)
    ggplt <- ggplot(df) + theme_bw() + geom_line(aes_string(x='distance', y='correlation', alpha='logweight'))
    ggplt <- ggplt + xlab('distance in [bp]')
    ggplt <- ggplt + facet_grid(from ~ to)
    if (miny < 0) {
        ggplt <- ggplt + geom_hline(aes_string('yintercept'=0), linetype=2, alpha=0.5)
    }
    
    r <- list(data=cor.array, plot=ggplt, separate.contexts=separate.contexts)
    return(r)
}
    
#' \code{transDist} parameter
#' 
#' Obtain an estimate for the \code{transDist} parameter (used in function \code{\link{callMethylation}}) by fitting an exponential function to the supplied correlations (from \code{\link{distanceCorrelation}}).
#' 
#' @param distcor The output produced by \code{\link{distanceCorrelation}}.
#' @param skip Skip the first n cytosines for the fitting. This can be necessary to avoid periodicity artifacts due to the context definition.
#' @param plot.parameters Whether to plot fitted parameters on to the plot or not.
#' @return A list() with fitted \code{transDist} parameters and the corresponding \code{\link[ggplot2]{ggplot}}.
#' 
#' @importFrom stats na.omit coefficients
#' @importFrom minpack.lm nlsLM
#' 
#' @export
#' @examples 
#'## Get some toy data
#'file <- system.file("data","arabidopsis_toydata.RData",
#'                     package="methimpute")
#'data <- get(load(file))
#'distcor <- distanceCorrelation(data)
#'fit <- estimateTransDist(distcor)
#'print(fit)
estimateTransDist <- function(distcor, skip=2, plot.parameters=TRUE) {

    ## Context correlation fits and plots
    contexts <- dimnames(distcor$data)[[1]]
    cor.array <- distcor$data
    maxweights <- numeric()
    params.list <- list()
    miny <- min(cor.array, na.rm = TRUE)
    dfs <- list()
    for (c1 in 1:length(contexts)) {
        for (c2 in 1:length(contexts)) {
            context.transition <- paste0(contexts[c1], '-', contexts[c2])
            if (distcor$separate.contexts) {
                if (c1 != c2) {
                    next
                }
            }
            if (c1 <= c2) {
                df <- data.frame(distance = as.numeric(dimnames(cor.array)[[3]]), correlation = cor.array[c1,c2,,'correlation'], weight = cor.array[c1,c2,,'weight'], from = contexts[c1], to = contexts[c2])
                
                ## Fit
                y <- df$correlation[(skip+1):nrow(df)]
                x <- df$distance[(skip+1):nrow(df)]
                weight <- df$weight[(skip+1):nrow(df)]
                startvalues <- list(a0 = stats::na.omit(y)[1], D = 50)
                p <- tryCatch({
                    fit <- minpack.lm::nlsLM(y ~ a0 * exp(-x/D), start=startvalues, weights=weight)
                    s <- summary(fit)
                    c <- stats::coefficients(s)
                    params <- c[1:length(startvalues)]
                    names(params) <- names(startvalues)
                    as.list(params)
                }, error = function(e) {
                    NULL
                })
                if (is.null(p)) {
                    startvalues <- list(a0 = stats::na.omit(y)[1])
                    p <- tryCatch({
                        fit <- minpack.lm::nlsLM(y ~ a0 * exp(-x/Inf), start=startvalues, weights=weight)
                        s <- summary(fit)
                        c <- stats::coefficients(s)
                        params <- c[1:length(startvalues)]
                        names(params) <- names(startvalues)
                        params <- as.list(params)
                        params$D <- Inf
                        params
                    }, error = function(e) {
                        startvalues$D <- Inf
                        startvalues
                    })
                }
                
                ## Check if we have negative D
                if (p$D <= 0) {
                    p$D <- Inf
                }
                params.list[[context.transition]] <- p
                
                ## Plot
                df$correlation.fit <- p$a0 * exp(-df$distance/p$D)
                df$logweight <- log(df$weight+1)
                dfs[[context.transition]] <- df
                maxweights[context.transition] <- max(df$logweight, na.rm = TRUE)
            }
        }
    }
    maxweight <- max(maxweights, na.rm = TRUE)
            
    ## Plot correlation
    df <- do.call(rbind, dfs)
    df$a0 <- round(sapply(params.list[paste0(df$from, '-', df$to)], '[[', 'a0'), 2)
    df$D <- round(sapply(params.list[paste0(df$from, '-', df$to)], '[[', 'D'), 0)
    df$params <- paste0("a0 = ", df$a0, ", D = ", df$D)
    ggplt <- ggplot(df) + theme_bw() + geom_line(aes_string(x='distance', y='correlation', alpha='logweight'))
    ggplt <- ggplt + geom_line(aes_string(x='distance', y='correlation.fit'), col='blue')
    if (plot.parameters) {
        ggplt <- ggplt + geom_text(aes_string(label='params'), x=max(df$distance, na.rm = TRUE), y=max(df$correlation, na.rm = TRUE), vjust=1, hjust=1)
    }
    ggplt <- ggplt + xlab('distance in [bp]')
    ggplt <- ggplt + facet_grid(from ~ to)
    if (miny < 0) {
        ggplt <- ggplt + geom_hline(aes_string('yintercept'=0), linetype=2, alpha=0.5)
    }
    
    transDist <- sapply(params.list, '[[', 'D')
    return(list(transDist=transDist, plot=ggplt))
}

#' Collapse consecutive bins
#'
#' The function will collapse consecutive bins which have, for example, the same combinatorial state.
#'
#' The following tables illustrate the principle of the collapsing:
#'
#' Input data:
#' \tabular{rrrrrr}{
#' seqnames \tab start \tab end \tab column2collapseBy \tab moreColumns \tab columns2sumUp \cr
#' chr1     \tab     0 \tab 199 \tab                 2 \tab        1 10 \tab           1 3 \cr
#' chr1     \tab   200 \tab 399 \tab                 2 \tab        2 11 \tab           0 3 \cr
#' chr1     \tab   400 \tab 599 \tab                 2 \tab        3 12 \tab           1 3 \cr
#' chr1     \tab   600 \tab 799 \tab                 1 \tab        4 13 \tab           0 3 \cr
#' chr1     \tab   800 \tab 999 \tab                 1 \tab        5 14 \tab           1 3 \cr
#' }
#' Output data:
#' \tabular{rrrrrr}{
#' seqnames \tab start \tab end \tab column2collapseBy \tab moreColumns \tab columns2sumUp \cr
#' chr1     \tab     0 \tab 599 \tab                 2 \tab        1 10 \tab           2 9 \cr
#' chr1     \tab   600 \tab 999 \tab                 1 \tab        4 13 \tab           1 6 \cr
#' }
#' 
#' @param data A data.frame containing the genomic coordinates in the first three columns.
#' @param column2collapseBy The number of the column which will be used to collapse all other inputs. If a set of consecutive bins has the same value in this column, they will be aggregated into one bin with adjusted genomic coordinates.
#' @param columns2sumUp Column numbers that will be summed during the aggregation process.
#' @param columns2average Column numbers that will be averaged during the aggregation process.
#' @param columns2getMax Column numbers where the maximum will be chosen during the aggregation process.
#' @param columns2drop Column numbers that will be dropped after the aggregation process.
#' @return A data.frame.
#' @author Aaron Taudt
#' @export
#' @examples
#'## Load example data
#'## Get an example multiHMM
#'data(arabidopsis_toydata)
#'df <- as.data.frame(arabidopsis_toydata)
#'shortdf <- collapseBins(df, column2collapseBy='context', columns2sumUp='width', columns2average=7:8)
#'
collapseBins = function(data, column2collapseBy=NULL, columns2sumUp=NULL, columns2average=NULL, columns2getMax=NULL, columns2drop=NULL) {

    ## Name to index
    if (is.character(column2collapseBy)) {
        column2collapseBy <- which(column2collapseBy == names(data))
    }
    if (is.character(columns2sumUp)) {
        columns2sumUp <- unlist(lapply(columns2sumUp, function(x) { which(x == names(data)) }))
    }
    if (is.character(columns2average)) {
        columns2average <- unlist(lapply(columns2average, function(x) { which(x == names(data)) }))
    }
    if (is.character(columns2getMax)) {
        columns2getMax <- unlist(lapply(columns2getMax, function(x) { which(x == names(data)) }))
    }
    if (is.character(columns2drop)) {
        columns2drop <- unlist(lapply(columns2drop, function(x) { which(x == names(data)) }))
    }
    ## Indices
    ind_coords <- 1:3
    ind_morecols <- setdiff(1:ncol(data), c(ind_coords, columns2sumUp, columns2average, columns2getMax, columns2drop))
    ind_sumcols <- columns2sumUp
    ind_meancols <- columns2average
    ind_maxcols <- columns2getMax

    ## Make the comparison vector
    ptm <- startTimedMessage('Making comparison vector ...')
    if (is.null(column2collapseBy)) {
        c <- data$start
        cShift1 <- rep(NA,length(c))
        cShift1[2:length(cShift1)] <- data$end[-length(c)] + 1
    } else {
        if (is(data[,column2collapseBy], "factor")) {
            c <- as.integer(data[,column2collapseBy])
        } else {
            c <- data[,column2collapseBy]
        }
        cShift1 <- rep(NA,length(c))
        cShift1[-1] <- c[-length(c)]
    }
    compare_custom <- c != cShift1
    ## Make the comparison vector to separate chromosomes
    c <- as.integer(data[,1])
    cShift1 <- rep(NA,length(c))
    cShift1[-1] <- c[-length(c)]
    compare_chrom <- c != cShift1
    ## Combine the vectors
    compare <- compare_custom | compare_chrom
    compare[1] <- TRUE
    numcollapsedbins <- length(which(compare==TRUE))
    numbins <- nrow(data)
    stopTimedMessage(ptm)
    if (any(is.na(compare))) {
        stop("NAs in vector 'compare'")
    }

    ## Select the collapsed rows
    ptm <- startTimedMessage('Selecting rows ...')
    collapsed.bins <- list()
    collapsed.bins[[names(data)[1]]] <- data[which(compare),1] #which to remove NAs which shouldn't be there in the first place
    collapsed.bins[[names(data)[2]]] <- data[which(compare),2]
    collapsed.bins[[names(data)[3]]] <- data[c((which(compare)-1)[-1],numbins), 3]
    if (length(ind_morecols)==1) {
        collapsed.bins[[names(data)[ind_morecols]]] <- data[which(compare), ind_morecols]
    } else if (length(ind_morecols)>1) {
        lcb <- length(collapsed.bins)
        lmc <- length(ind_morecols)
        collapsed.bins[(lcb+1):(lcb+lmc)] <- data[which(compare), ind_morecols]
        names(collapsed.bins)[(lcb+1):(lcb+lmc)] <- names(data)[ind_morecols]
    }
    stopTimedMessage(ptm)

    ## Sum up columns
    xfuns <- list(sum, mean, max)
    xstrings <- list('sum', 'mean', 'max')
    columns2xs <- list(columns2sumUp, columns2average, columns2getMax)
    inds_xcols <- list(ind_sumcols, ind_meancols, ind_maxcols)
    for (ix in 1:length(xfuns)) {
        xfun <- xfuns[[ix]]
        xstring <- xstrings[[ix]]
        columns2x <- columns2xs[[ix]]
        ind_xcols <- inds_xcols[[ix]]
        if (!is.null(columns2x)) {
            ptm <- startTimedMessage('Calculating ',xstring,' ...')
            xcols <- as.matrix(data[,columns2x])
            collapsed.xcols <- matrix(NA, nrow=numcollapsedbins, ncol=length(columns2x))
            icount <- 1
            i1_lasttrue <- 1
            for (i1 in 1:length(compare)) {
                if (compare[i1]==TRUE) {
                    if (length(columns2x)==1) {
                        collapsed.xcols[icount-1] <- xfun(xcols[i1_lasttrue:(i1-1),])
                    } else if (length(columns2x) > 1) {
                        if (i1_lasttrue==i1-1 | i1==1) {
                            collapsed.xcols[icount-1,] <- as.numeric(xcols[i1_lasttrue,])
                        } else {
                            collapsed.xcols[icount-1,] <- apply(xcols[i1_lasttrue:(i1-1),],2,xfun)
                        }
                    }
                    icount <- icount+1
                    i1_lasttrue <- i1
                }
            }
            i1 = i1+1
            if (length(columns2x)==1) {
                collapsed.xcols[icount-1] <- xfun(xcols[i1_lasttrue:(i1-1),])
            } else if (length(columns2x) > 1) {
                if (i1_lasttrue==i1-1 | i1==1) {
                    collapsed.xcols[icount-1,] <- as.numeric(xcols[i1_lasttrue,])
                } else {
                    collapsed.xcols[icount-1,] <- apply(xcols[i1_lasttrue:(i1-1),],2,xfun)
                }
            }
            if (length(ind_xcols) > 0) {
                lcb <- length(collapsed.bins)
                lsc <- length(ind_xcols)
                collapsed.bins[(lcb+1):(lcb+lsc)] <- as.data.frame(collapsed.xcols)
                names(collapsed.bins)[(lcb+1):(lcb+lsc)] <- paste(xstring, names(data)[ind_xcols], sep='.')
            }
            stopTimedMessage(ptm)
        }
    }

    return(as.data.frame(collapsed.bins))

}

#' Get original posteriors
#' 
#' Transform the 'posteriorMeth', 'posteriorMax', and 'status' columns into original posteriors from the HMM.
#' 
#' @param data The \code{$data} entry from a \code{\link{methimputeBinomialHMM}} object.
#' @return A matrix with posteriors.
#' 
getPosteriors <- function(data) {
    
    p <- array(NA, dim = c(length(data), length(levels(data$status))), dimnames = list(NULL, status = levels(data$status)))
    
    mask <- data$status == 'Methylated'
    p[mask,'Methylated'] <- data$posteriorMax[mask]
    p[mask,'Intermediate'] <- (data$posteriorMeth[mask] - p[mask,'Methylated']) * 2
    p[mask,'Unmethylated'] <- 1 - p[mask,'Methylated'] - p[mask,'Intermediate']
    
    mask <- data$status == 'Intermediate'
    p[mask,'Intermediate'] <- data$posteriorMax[mask]
    p[mask,'Methylated'] <- (data$posteriorMeth[mask] - 0.5*p[mask,'Intermediate'])
    p[mask,'Unmethylated'] <- 1 - p[mask,'Intermediate'] - p[mask,'Methylated']
    
    mask <- data$status == 'Unmethylated'
    p[mask,'Unmethylated'] <- data$posteriorMax[mask]
    p[mask,'Intermediate'] <- (1 - p[mask,'Unmethylated'] - data$posteriorMeth[mask]) * 2
    p[mask,'Methylated'] <- 1 - p[mask,'Unmethylated'] - p[mask,'Intermediate']
    
    return(p)
}

#' Chromosome lengths for Arabidopsis
#'
#' A data.frame with chromosome lengths for Arabidopsis.
#'
#' @docType data
#' @name arabidopsis_chromosomes
#' @format A data.frame.
#' @examples
#'data(arabidopsis_chromosomes)
#'print(arabidopsis_chromosomes)
NULL

#' Toy data for Arabidopsis (200.000bp of chr1)
#'
#' A \code{\link{methimputeData}} object for demonstration purposes in examples of package \pkg{\link{methimpute}}. The object contains the first 200.000 cytosines of chr1 from Arabidopsis.
#'
#' @docType data
#' @name arabidopsis_toydata
#' @format A \code{\link{methimputeData}} object.
#' @examples
#'data(arabidopsis_toydata)
#'print(arabidopsis_toydata)
NULL

#' Gene coordinates for Arabidopsis (chr1)
#'
#' A \code{\link[GenomicRanges]{GRanges-class}} object for demonstration purposes in examples of package \pkg{\link{methimpute}}. The object contains gene coordinates of chr1 from Arabidopsis.
#'
#' @docType data
#' @name arabidopsis_genes
#' @format A \code{\link[GenomicRanges]{GRanges-class}} object.
#' @examples
#'data(arabidopsis_genes)
#'print(arabidopsis_genes)
NULL

#' Transposable element coordinates for Arabidopsis (chr1)
#'
#' A \code{\link[GenomicRanges]{GRanges-class}} object for demonstration purposes in examples of package \pkg{\link{methimpute}}. The object contains transposable element coordinates of chr1 from Arabidopsis.
#'
#' @docType data
#' @name arabidopsis_TEs
#' @format A \code{\link[GenomicRanges]{GRanges-class}} object.
#' @examples
#'data(arabidopsis_TEs)
#'print(arabidopsis_TEs)
NULL


#' methimpute objects
#'
#' @description
#' \pkg{\link{methimpute}} defines several objects.
#' \itemize{
#' \item \code{\link{methimputeData}}: Returned by \code{\link{importBSSeeker}}, \code{\link{importBismark}} and \code{\link{inflateMethylome}}.
#' \item \code{\link{methimputeBinomialHMM}}: Returned by \code{\link{callMethylation}}.
#' }
#'
#' @name methimpute-objects
NULL


#' methimputeData
#'
#' A \code{\link[GenomicRanges]{GRanges-class}} object containing cytosine coordinates with meta-data columns 'context' and 'counts'.
#' @name methimputeData
#' @seealso \code{\link{methimpute-objects}}
NULL


#' methimputeBinomialHMM
#' 
#' The \code{methimputeBinomialHMM} is a list() which contains various entries (see Value section). The main entry of this object is \code{$data}, which contains the methylation status calls and posterior values. See Details for a description of all columns.
#' 
#' The \code{$data} entry in this object contains the following columns:
#' \itemize{
#' \item{context }{The sequence context of the cytosine.}
#' \item{counts }{Counts for methylated and total number of reads at each position.}
#' \item{distance }{The distance in base-pairs from the previous to the current cytosine.}
#' \item{transitionContext }{Transition context in the form "previous-current".}
#' \item{posteriorMax }{Maximum posterior value of the methylation status call, can be interpreted as the confidence in the call.}
#' \item{posteriorMeth }{Posterior value of the "methylated" component.}
#' \item{posteriorUnmeth }{Posterior value of the "unmethylated" component.}
#' \item{status }{Methylation status.}
#' \item{rc.meth.lvl }{Recalibrated methylation level, calculated as \code{r$data$rc.meth.lvl = r$data$params$emissionParams$Unmethylated[data$context,] * r$data$posteriorUnmeth + r$params$emissionParams$Methylated[data$context,] * r$data$posteriorMeth}, where \code{r} is the \code{methimputeBinomialHMM} object.}
# #' \item{rc.counts }{Recalibrated counts for methylated and total number of reads at each position, calculated as \code{r$data$rc.counts[,2] <- sort(r$data$counts[,2])[rank(r$data$posteriorMax, ties.method = 'first')]; r$data$rc.counts[,1] <- round(r$data$rc.counts[,2] * r$data$rc.meth.lvl)}, where \code{r} is the \code{methimputeBinomialHMM} object.}
#' }
#' 
#' @return
#' A list() with the following entries:
#' \item{convergenceInfo}{A list() with information about the convergence of the model fitting procedure.}
#' \item{params}{A list() with fitted and non-fitted model parameters.}
#' \item{params.initial}{A list() with initial values for the model parameters.}
#' \item{data}{A \code{\link[GenomicRanges]{GRanges-class}} with cytosine positions and methylation status calls.}
#' \item{segments}{The \code{data} entry where coordinates of consecutive cytosines with the same methylation status have been merged.}
#' 
#' @name methimputeBinomialHMM
#' @seealso \code{\link{methimpute-objects}}
NULL

%\VignetteIndexEntry{Methylation status calling with METHimpute}
%\VignetteEngine{knitr::knitr}
\documentclass{article}

<<style-knitr, eval=TRUE, echo=FALSE, results="asis">>=
BiocStyle::latex()
@

\author{Aaron Taudt\thanks{\email{aaron.taudt@gmail.com}}}
\title{Methylation status calling with METHimpute}

\begin{document}
\maketitle
\tableofcontents
\clearpage
<<options, results='hide', message=FALSE, eval=TRUE, echo=FALSE>>=
library(methimpute)
options(width=80)
@
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Introduction}

\Rpackage{Methimpute} implements a powerful HMM-based binomial test for methylation status calling. Besides improved accuracy over the classical binomial test, the HMM allows imputation of the methylation status of \textbf{all cytosines} in the genome. It achieves this by borrowing information from neighboring covered cytosines. The confidence in the methylation status call is reported as well. The HMM can also be used to impute the methylation status for binned data instead of individual cytosines. Furthermore, \Rpackage{methimpute} outputs context-specific conversion rates, which might be used to optimize the experimental procedure.

For the exact workings of \Rpackage{methimpute} we refer the interested reader to our publication at \url{https://doi.org/10.1186/s12864-018-4641-x}.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Methylation status calling on individual cytosines}

The following examples explain the necessary steps for methylation status calling (and imputation). To keep the calculation time short, it uses only the first 200.000 bp of the Arabidopsis genome. The example consists of three steps: 1) Data import, 2) estimating the distance correlation and 3) methylation status calling.
At the end of this example you will see that positions without counts are assigned a methylation status, but the confidence (column "posteriorMax") is generally quite low for those cytosines, whereas it is high for well-covered cytosines ($>=$0.99).

\subsection{\label{separatecontextmodel}Separate-context model}

The separate-context model runs a separate HMM for each context. This assumes that only within-context correlations are important, and between-context correlations do not need to be considered.\\

\begin{small}
<<methylation_status_calling_separate, results='markup', message=FALSE, eval=TRUE, fig.width=10, fig.height=7, out.width='\\textwidth', fig.align='center'>>=
library(methimpute)

# ===== Step 1: Importing the data ===== #

# We load an example file in BSSeeker format that comes with the package
file <- system.file("extdata","arabidopsis_bsseeker.txt.gz", package="methimpute")
bsseeker.data <- importBSSeeker(file)
print(bsseeker.data)

# Because most methylation extractor programs report only covered cytosines,
# we need to inflate the data to inlcude all cytosines (including non-covered sites)
fasta.file <- system.file("extdata","arabidopsis_sequence.fa.gz", package="methimpute")
cytosine.positions <- extractCytosinesFromFASTA(fasta.file,
                                                contexts = c('CG','CHG','CHH'))
methylome <- inflateMethylome(bsseeker.data, cytosine.positions)
print(methylome)


# ===== Step 2: Obtain correlation parameters ===== #

# The correlation of methylation levels between neighboring cytosines is an important
# parameter for the methylation status calling, so we need to get it first. Keep in mind
# that we only use the first 200.000 bp here, that's why the plot looks a bit meagre.
distcor <- distanceCorrelation(methylome, separate.contexts = TRUE)
fit <- estimateTransDist(distcor)
print(fit)


# ===== Step 3: Methylation status calling (and imputation) ===== #

model <- callMethylationSeparate(data = methylome, transDist = fit$transDist,
                                 verbosity = 0)
# The confidence in the methylation status call is given in the column "posteriorMax".
# For further analysis one could split the results into high-confidence
# (posteriorMax >= 0.98) and low-confidence calls (posteriorMax < 0.98) for instance.
print(model)

# Bisulfite conversion rates can be obtained with
1 - model$params$emissionParams$Unmethylated
@
\end{small}

You can also check several properties of the fitted Hidden Markov Model, such as convergence or transition probabilities, and check how well the fitted distributions describe the data.

\begin{small}
<<methylation_status_calling_separate_checkresults, results='markup', message=FALSE, eval=TRUE, fig.width=10, fig.height=2.5, out.width='\\textwidth', fig.align='center'>>=
plotConvergence(model)
plotTransitionProbs(model)
plotHistogram(model, total.counts = 10)
@
\end{small}

\subsection{Interacting-context model}

The interacting-context model runs a single HMM for all contexts. This takes into account the within-context and between-context correlations and should be more accurate than the separate-context model if sufficient data is available. However, we have observed that in low coverage settings too much information from well covered contexts is diffusing into the low covered contexts (e.g. CHH and CHG will look like CG with very low coverage). In this case, please use the separate-context model in section~\ref{separatecontextmodel}.\\

\begin{small}
<<methylation_status_calling, results='markup', message=FALSE, eval=TRUE, fig.width=10, fig.height=7, out.width='\\textwidth', fig.align='center'>>=
library(methimpute)

# ===== Step 1: Importing the data ===== #

# We load an example file in BSSeeker format that comes with the package
file <- system.file("extdata","arabidopsis_bsseeker.txt.gz", package="methimpute")
bsseeker.data <- importBSSeeker(file)
print(bsseeker.data)

# Because most methylation extractor programs report only covered cytosines,
# we need to inflate the data to inlcude all cytosines (including non-covered sites)
fasta.file <- system.file("extdata","arabidopsis_sequence.fa.gz", package="methimpute")
cytosine.positions <- extractCytosinesFromFASTA(fasta.file,
                                                contexts = c('CG','CHG','CHH'))
methylome <- inflateMethylome(bsseeker.data, cytosine.positions)
print(methylome)


# ===== Step 2: Obtain correlation parameters ===== #

# The correlation of methylation levels between neighboring cytosines is an important
# parameter for the methylation status calling, so we need to get it first. Keep in mind
# that we only use the first 200.000 bp here, that's why the plot looks a bit meagre.
distcor <- distanceCorrelation(methylome)
fit <- estimateTransDist(distcor)
print(fit)


# ===== Step 3: Methylation status calling (and imputation) ===== #

model <- callMethylation(data = methylome, transDist = fit$transDist)
# The confidence in the methylation status call is given in the column "posteriorMax".
# For further analysis one could split the results into high-confidence
# (posteriorMax >= 0.98) and low-confidence calls (posteriorMax < 0.98) for instance.
print(model)

# Bisulfite conversion rates can be obtained with
1 - model$params$emissionParams$Unmethylated
@
\end{small}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Methylation status calling on binned data}

The following examples explain the necessary steps for methylation status calling (and imputation) on binned data, such as commonly used 100bp bins. To keep the calculation time short, it uses only the first 200.000 bp of the Arabidopsis genome. The example consists of four steps: 1) Data import, 2) binning and 3) methylation status calling.\\

\begin{small}
<<methylation_status_calling_binned, results='markup', message=FALSE, eval=TRUE, fig.width=10, fig.height=7, out.width='\\textwidth', fig.align='center'>>=
library(methimpute)

# ===== Step 1: Importing the data ===== #

# We load an example file in BSSeeker format that comes with the package
file <- system.file("extdata","arabidopsis_bsseeker.txt.gz", package="methimpute")
bsseeker.data <- importBSSeeker(file)
print(bsseeker.data)

# Because most methylation extractor programs report only covered cytosines,
# we need to inflate the data to inlcude all cytosines (including non-covered sites)
fasta.file <- system.file("extdata","arabidopsis_sequence.fa.gz", package="methimpute")
cytosine.positions <- extractCytosinesFromFASTA(fasta.file,
                                                contexts = c('CG','CHG','CHH'))
methylome <- inflateMethylome(bsseeker.data, cytosine.positions)
print(methylome)


# ===== Step 2: Binning into 100bp bins ===== #
binnedMethylome <- binMethylome(methylome, binsize = 100, contexts = c('total','CG'))
print(binnedMethylome$CG)


# ===== Step 3: Methylation status calling (and imputation) ===== #

binnedModel <- callMethylation(data = binnedMethylome$CG)
print(binnedModel)
@
\end{small}

You can also check several properties of the fitted Hidden Markov Model, such as convergence or transition probabilities, and check how well the fitted distributions describe the data. This last point is important because the binomial distributions that the HMM uses were originally meant to describe individual cytosines and not bins. However, we have observed that they still capture the bimodal distributions of methylation levels in binned data quite well. Note that the histogram for our example looks quite sparse due to the very low number of bins that were used.

\begin{small}
<<methylation_status_calling_binned_checkresults, results='markup', message=FALSE, eval=TRUE, fig.width=5, fig.height=2.5, out.width='0.5\\textwidth', fig.align='center'>>=
plotConvergence(binnedModel)
plotTransitionProbs(binnedModel)
plotHistogram(binnedModel, total.counts = 150)
@
\end{small}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{\label{sec:descriptionofcolumns}Description of columns in the output}
\begin{itemize}
\item \textbf{context} The sequence context of the cytosine.
\item \textbf{counts} Counts for methylated and total number of reads at each position.
\item \textbf{distance} The distance in base-pairs from the previous to the current cytosine.
\item \textbf{transitionContext} Transition context in the form "previous-current".
\item \textbf{posteriorMax} Maximum posterior value of the methylation status call, can be interpreted as the confidence in the call.
\item \textbf{posteriorMeth} Posterior value of the "methylated" component.
\item \textbf{posteriorUnmeth} Posterior value of the "unmethylated" component.
\item \textbf{status} Methylation status.
\item \textbf{rc.meth.lvl} Recalibrated methylation level, calculated from the posteriors and the fitted parameters (see ?methimputeBinomialHMM for details).
% \item \textbf{rc.counts} Recalibrated counts for methylated and total number of reads at each position (see ?methimputeBinomialHMM for details).
\end{itemize}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Plots and enrichment analysis}

This package also offers plotting functions for a simple enrichment analysis. Let's say we are interested in the methylation level around genes and transposable elements. We would also like to see how the imputation works on cytosines with missing data compared to covered cytosines.

\begin{small}
<<enrichment_analysis, results='markup', message=FALSE, eval=TRUE, fig.width=10, fig.height=4, out.width='0.8\\textwidth', fig.align='center'>>=
# Define categories to distinguish missing from covered cytosines
model$data$category <- factor('covered', levels=c('missing', 'covered'))
model$data$category[model$data$counts[,'total']>=1] <- 'covered'
model$data$category[model$data$counts[,'total']==0] <- 'missing'

# Note that the plots look a bit ugly because our toy data has only 200.000 datapoints.
data(arabidopsis_genes)
seqlengths(model$data) <- seqlengths(arabidopsis_genes)[seqlevels(model$data)] # this
                          # line should only be necessary for our toy example
plotEnrichment(model$data, annotation=arabidopsis_genes, range = 2000,
               category.column = 'category')
data(arabidopsis_TEs)
plotEnrichment(model$data, annotation=arabidopsis_TEs, range = 2000,
               category.column = 'category')
@
\end{small}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Export results}

You can export the results as TSV file with the following columns:
\begin{itemize}
\item chromosome, position, strand, context, counts.methylated, counts.total, posteriorMax, posteriorMeth, posteriorUnmeth, status, rc.meth.lvl
\end{itemize}

\begin{small}
<<export_results, results='markup', message=FALSE, eval=TRUE>>=
exportMethylome(model, filename = tempfile())
@
\end{small}

Please see section \ref{sec:descriptionofcolumns} for a description of the columns.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Session Info}
\begin{scriptsize}
<<sessionInfo, eval=TRUE, results="asis">>=
toLatex(sessionInfo())
@
\end{scriptsize}

\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/callMethylation.R
\name{callMethylation}
\alias{callMethylation}
\title{Call methylation status}
\usage{
callMethylation(data, fit.on.chrom = NULL, transDist = Inf, eps = 1,
  max.time = Inf, max.iter = Inf, count.cutoff = 500,
  verbosity = 1, num.threads = 2 + include.intermediate,
  initial.params = NULL, include.intermediate = FALSE,
  update = "independent", min.reads = 0)
}
\arguments{
\item{data}{A \code{\link{methimputeData}} object.}

\item{fit.on.chrom}{A character vector specifying the chromosomes on which the HMM will be fitted.}

\item{transDist}{The decaying constant for the distance-dependent transition matrix. Either a single numeric or a named numeric vector, where the vector names correspond to the transition contexts. Such a vector can be obtained from \code{\link{estimateTransDist}}.}

\item{eps}{Convergence threshold for the Baum-Welch algorithm.}

\item{max.time}{Maximum running time in seconds for the Baum-Welch algorithm.}

\item{max.iter}{Maximum number of iterations for the Baum-Welch algorithm.}

\item{count.cutoff}{A cutoff for the counts to remove artificially high counts from mapping artifacts. Set to \code{Inf} to disable this filtering (not recommended).}

\item{verbosity}{An integer from 1 to 5 specifying the verbosity of the fitting procedure. Values > 1 are only for debugging.}

\item{num.threads}{Number of CPU to use for the computation. Parallelization is implemented on the number of states, which is 2 or 3, so setting \code{num.threads > 3} will not give additional performance increase.}

\item{initial.params}{A \code{\link{methimputeBinomialHMM}} object. This parameter is useful to continue the fitting procedure for a \code{\link{methimputeBinomialHMM}} object.}

\item{include.intermediate}{A logical specifying wheter or not the intermediate component should be included in the HMM.}

\item{update}{One of \code{c("independent", "constrained")}. If \code{update="independent"} probability parameters for the binomial test will be updated independently. If \code{update="constrained"} the probability parameter of the intermediate component will be constrained to the mean of the unmethylated and the methylated component.}

\item{min.reads}{The minimum number of reads that a position must have to contribute in the Baum-Welch fitting procedure.}
}
\value{
A \code{\link{methimputeBinomialHMM}} object.
}
\description{
Call methylation status of cytosines (or bins) with a Hidden Markov Model.
}
\details{
The Hidden Markov model uses a binomial test for the emission densities. Transition probabilities are modeled with a distance dependent decay, specified by the parameter \code{transDist}.
}
\examples{
## Get some toy data
file <- system.file("data","arabidopsis_toydata.RData", package="methimpute")
data <- get(load(file))
print(data)
model <- callMethylation(data)
print(model)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collapseBins.R
\name{collapseBins}
\alias{collapseBins}
\title{Collapse consecutive bins}
\usage{
collapseBins(data, column2collapseBy = NULL, columns2sumUp = NULL,
  columns2average = NULL, columns2getMax = NULL, columns2drop = NULL)
}
\arguments{
\item{data}{A data.frame containing the genomic coordinates in the first three columns.}

\item{column2collapseBy}{The number of the column which will be used to collapse all other inputs. If a set of consecutive bins has the same value in this column, they will be aggregated into one bin with adjusted genomic coordinates.}

\item{columns2sumUp}{Column numbers that will be summed during the aggregation process.}

\item{columns2average}{Column numbers that will be averaged during the aggregation process.}

\item{columns2getMax}{Column numbers where the maximum will be chosen during the aggregation process.}

\item{columns2drop}{Column numbers that will be dropped after the aggregation process.}
}
\value{
A data.frame.
}
\description{
The function will collapse consecutive bins which have, for example, the same combinatorial state.
}
\details{
The following tables illustrate the principle of the collapsing:

Input data:
\tabular{rrrrrr}{
seqnames \tab start \tab end \tab column2collapseBy \tab moreColumns \tab columns2sumUp \cr
chr1     \tab     0 \tab 199 \tab                 2 \tab        1 10 \tab           1 3 \cr
chr1     \tab   200 \tab 399 \tab                 2 \tab        2 11 \tab           0 3 \cr
chr1     \tab   400 \tab 599 \tab                 2 \tab        3 12 \tab           1 3 \cr
chr1     \tab   600 \tab 799 \tab                 1 \tab        4 13 \tab           0 3 \cr
chr1     \tab   800 \tab 999 \tab                 1 \tab        5 14 \tab           1 3 \cr
}
Output data:
\tabular{rrrrrr}{
seqnames \tab start \tab end \tab column2collapseBy \tab moreColumns \tab columns2sumUp \cr
chr1     \tab     0 \tab 599 \tab                 2 \tab        1 10 \tab           2 9 \cr
chr1     \tab   600 \tab 999 \tab                 1 \tab        4 13 \tab           1 6 \cr
}
}
\examples{
## Load example data
## Get an example multiHMM
data(arabidopsis_toydata)
df <- as.data.frame(arabidopsis_toydata)
shortdf <- collapseBins(df, column2collapseBy='context', columns2sumUp='width', columns2average=7:8)

}
\author{
Aaron Taudt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inflateMethylome.R
\name{inflateMethylome}
\alias{inflateMethylome}
\title{Inflate an imported methylation extractor file}
\usage{
inflateMethylome(methylome, methylome.full)
}
\arguments{
\item{methylome}{A \code{\link[GenomicRanges]{GRanges-class}} with methylation counts.}

\item{methylome.full}{A \code{\link[GenomicRanges]{GRanges-class}} with positions for all cytosines or a file with such an object.}
}
\value{
The \code{methylome.full} object with added metadata column 'counts'.
}
\description{
Inflate an imported methylation extractor file to contain all cytosine positions. This is useful to obtain a full methylome, including non-covered cytosines, because most methylation extractor programs only report covered cytosines.
}
\examples{
## Get an example file in BSSeeker format
file <- system.file("extdata","arabidopsis_bsseeker.txt.gz", package="methimpute")
bsseeker.data <- importBSSeeker(file)
bsseeker.data

## Inflate to full methylome (including non-covered sites)
data(arabidopsis_toydata)
full.methylome <- inflateMethylome(bsseeker.data, arabidopsis_toydata)
full.methylome

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documented_objects.R
\docType{data}
\name{arabidopsis_chromosomes}
\alias{arabidopsis_chromosomes}
\title{Chromosome lengths for Arabidopsis}
\format{A data.frame.}
\description{
A data.frame with chromosome lengths for Arabidopsis.
}
\examples{
data(arabidopsis_chromosomes)
print(arabidopsis_chromosomes)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{getStateColors}
\alias{getStateColors}
\title{Get state colors}
\usage{
getStateColors(states = NULL)
}
\arguments{
\item{states}{A character vector.}
}
\value{
A character vector with colors.
}
\description{
Get the colors that are used for plotting.
}
\examples{
cols <- getStateColors()
pie(1:length(cols), col=cols, labels=names(cols))
}
\seealso{
\code{\link{plotting}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documented_objects.R
\docType{data}
\name{arabidopsis_toydata}
\alias{arabidopsis_toydata}
\title{Toy data for Arabidopsis (200.000bp of chr1)}
\format{A \code{\link{methimputeData}} object.}
\description{
A \code{\link{methimputeData}} object for demonstration purposes in examples of package \pkg{\link{methimpute}}. The object contains the first 200.000 cytosines of chr1 from Arabidopsis.
}
\examples{
data(arabidopsis_toydata)
print(arabidopsis_toydata)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/correlation.R
\name{distanceCorrelation}
\alias{distanceCorrelation}
\title{Distance correlation}
\usage{
distanceCorrelation(data, distances = 0:50, separate.contexts = FALSE)
}
\arguments{
\item{data}{A \code{\link{methimputeData}} object.}

\item{distances}{An integer vector specifying the distances for which the correlation will be calculated.}

\item{separate.contexts}{A logical indicating whether contexts are treated separately. If set to \code{TRUE}, correlations will only be calculated between cytosines of the same context.}
}
\value{
A list() with an array containing the correlation values and the corresponding \code{\link[ggplot2]{ggplot}}.
}
\description{
Compute the distance correlation from a \code{\link{methimputeData}} object.
}
\examples{
## Get some toy data
file <- system.file("data","arabidopsis_toydata.RData",
                    package="methimpute")
data <- get(load(file))
distcor <- distanceCorrelation(data)
print(distcor$plot)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documented_objects.R
\name{methimpute-objects}
\alias{methimpute-objects}
\title{methimpute objects}
\description{
\pkg{\link{methimpute}} defines several objects.
\itemize{
\item \code{\link{methimputeData}}: Returned by \code{\link{importBSSeeker}}, \code{\link{importBismark}} and \code{\link{inflateMethylome}}.
\item \code{\link{methimputeBinomialHMM}}: Returned by \code{\link{callMethylation}}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binMethylome.R
\name{binning}
\alias{binning}
\alias{binCounts}
\alias{binPositions}
\alias{binMethylome}
\title{Methimpute binning functions}
\usage{
binCounts(data, binsize)

binPositions(data, binsize)

binMethylome(data, binsize, contexts = "total", columns.average = NULL)
}
\arguments{
\item{data}{A \code{\link[GenomicRanges]{GRanges-class}} object with metadata columns 'context' and 'counts' (which is a matrix with columns 'methylated' and 'total').}

\item{binsize}{The window size used for binning.}

\item{contexts}{A character vector with contexts for which the binning will be done.}

\item{columns.average}{A character vector with names of columns in \code{data} that should be averaged in bins.}
}
\value{
A \code{\link[GenomicRanges]{GRanges-class}} object for \code{binCounts} and \code{binPostions}. A \code{list()} of \code{\link[GenomicRanges]{GRanges-class}} objects for \code{binMethylome}.
}
\description{
This page provides an overview of all \pkg{\link{methimpute}} binning functions.
}
\section{Functions}{
\itemize{
\item \code{binCounts}: Get the aggregated number of counts in each bin (no context).

\item \code{binPositions}: Get the number of cytosines in each bin (total and per context).

\item \code{binMethylome}: Get number of cytosines and aggregated counts for the specified contexts.
}}

\examples{
## Get some toy data
file <- system.file("data","arabidopsis_toydata.RData",
                    package="methimpute")
data <- get(load(file))
print(data)
## Bin the data in various ways
binCounts(data, binsize=1000)
binPositions(data, binsize=1000)
binMethylome(data, binsize=1000, contexts=c("total", "CG"),
            columns.average=NULL)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameterScan.R
\name{parameterScan}
\alias{parameterScan}
\title{Perform a parameter scan}
\usage{
parameterScan(f, param, values, ...)
}
\arguments{
\item{f}{A function for which to perform the scan.}

\item{param}{A character with the parameter for which to perform the scan.}

\item{values}{A vector with parameter values for which to perform the scan.}

\item{...}{Other parameters passed through to \code{f}.}
}
\value{
A data.frame with loglikelihood values.
}
\description{
Perform a parameter scan for an arbitrary parameter.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/printMethods.R
\name{print.methimputeBinomialHMM}
\alias{print.methimputeBinomialHMM}
\title{Print model object}
\usage{
\method{print}{methimputeBinomialHMM}(x, ...)
}
\arguments{
\item{x}{A \code{\link{methimputeBinomialHMM}} object.}

\item{...}{Ignored.}
}
\value{
An invisible \code{NULL}.
}
\description{
Print model object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export.R
\name{exportMethylome}
\alias{exportMethylome}
\title{Export a methylome}
\usage{
exportMethylome(model, filename)
}
\arguments{
\item{model}{A \code{\link{methimputeBinomialHMM}} object.}

\item{filename}{The name of the file to be exported.}
}
\value{
\code{NULL}
}
\description{
Export a methylome as a TSV file.
}
\examples{
\dontrun{
## Get some toy data
file <- system.file("data","arabidopsis_toydata.RData", package="methimpute")
data <- get(load(file))
print(data)
model <- callMethylation(data, max.iter=10)
exportMethylome(model, filename = tempfile())
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDistinctColors.R
\name{getDistinctColors}
\alias{getDistinctColors}
\title{Get distinct colors}
\usage{
getDistinctColors(n, start.color = "blue4", exclude.colors = c("white",
  "black", "gray", "grey", "\\\\<yellow\\\\>", "yellow1", "lemonchiffon"),
  exclude.brightness.above = 1, exclude.rgb.above = 210)
}
\arguments{
\item{n}{Number of colors to select. If \code{n} is a character vector, \code{length(n)} will be taken as the number of colors and the colors will be named by \code{n}.}

\item{start.color}{Color to start the selection process from.}

\item{exclude.colors}{Character vector with colors that should not be used.}

\item{exclude.brightness.above}{Exclude colors where the 'brightness' value in HSV space is above. This is useful to obtain a matt palette.}

\item{exclude.rgb.above}{Exclude colors where all RGB values are above. This is useful to exclude whitish colors.}
}
\value{
A character vector with colors.
}
\description{
Get a set of distinct colors selected from \code{\link{colors}}.
}
\details{
The function computes the euclidian distance between all \code{\link{colors}} and iteratively selects those that have the furthest closes distance to the set of already selected colors.
}
\examples{
cols <- getDistinctColors(5)
pie(rep(1,5), labels=cols, col=cols)

}
\author{
Aaron Taudt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/callMethylation.R
\name{binomialTestMethylation}
\alias{binomialTestMethylation}
\title{Call methylation status}
\usage{
binomialTestMethylation(data, conversion.rate, min.coverage = 3,
  p.threshold = 0.05)
}
\arguments{
\item{data}{A \code{\link{methimputeData}} object.}

\item{conversion.rate}{A conversion rate between 0 and 1.}

\item{min.coverage}{Minimum coverage to consider for the binomial test.}

\item{p.threshold}{Significance threshold between 0 and 1.}
}
\value{
A vector with methylation statuses.
}
\description{
Call methylation status of cytosines (or bins) with a binomial test.
}
\details{
The function uses a binomial test with the specified \code{conversion.rate}. P-values are then multiple testing corrected with the Benjamini & Yekutieli procedure. Methylated positions are selected with the \code{p.threshold}.
}
\examples{
## Get some toy data
file <- system.file("data","arabidopsis_toydata.RData", package="methimpute")
data <- get(load(file))
data$binomial <- binomialTestMethylation(data, conversion.rate=0.998)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/importData.R
\name{import}
\alias{import}
\alias{importBSMAP}
\alias{importMethylpy}
\alias{importBSSeeker}
\alias{importBismark}
\title{Methimpute data import}
\usage{
importBSMAP(file, chrom.lengths = NULL, skip = 1, contexts = c(CG =
  "NNCGN", CHG = "NNCHG", CHH = "NNCHH"))

importMethylpy(file, chrom.lengths = NULL, skip = 1, contexts = c(CG
  = "CGN", CHG = "CHG", CHH = "CHH"))

importBSSeeker(file, chrom.lengths = NULL, skip = 0)

importBismark(file, chrom.lengths = NULL, skip = 0)
}
\arguments{
\item{file}{The file to import.}

\item{chrom.lengths}{A data.frame with chromosome names in the first, and chromosome lengths in the second column. Only chromosomes named in here will be returned. Alternatively a tab-separated file with such a data.frame (with headers).}

\item{skip}{The number of lines to skip. Usually 1 if the file contains a header and 0 otherwise.}

\item{contexts}{A character vector of the contexts that are to be assigned. Since some programs report 5-letter contexts, this parameter can be used to obtain a reduced number of contexts. Will yield contexts CG, CHG, CHH by default. Set \code{contexts=NULL} to obtain all available contexts.}
}
\value{
A \code{\link{methimputeData}} object.
}
\description{
This page provides an overview of all \pkg{\link{methimpute}} data import functions.
}
\section{Functions}{
\itemize{
\item \code{importBSMAP}: Import a BSMAP methylation extractor file.

\item \code{importMethylpy}: Import a Methylpy methylation extractor file.

\item \code{importBSSeeker}: Import a BSSeeker methylation extractor file.

\item \code{importBismark}: Import a Bismark methylation extractor file.
}}

\examples{
## Get an example file in BSSeeker format
file <- system.file("extdata","arabidopsis_bsseeker.txt.gz", package="methimpute")
data(arabidopsis_chromosomes)
bsseeker.data <- importBSSeeker(file, chrom.lengths=arabidopsis_chromosomes)

## Get an example file in Bismark format
file <- system.file("extdata","arabidopsis_bismark.txt", package="methimpute")
data(arabidopsis_chromosomes)
arabidopsis_chromosomes$chromosome <- sub('chr', '', arabidopsis_chromosomes$chromosome)
bismark.data <- importBismark(file, chrom.lengths=arabidopsis_chromosomes)

## Get an example file in BSMAP format
file <- system.file("extdata","arabidopsis_BSMAP.txt", package="methimpute")
data(arabidopsis_chromosomes)
bsmap.data <- importBSMAP(file, chrom.lengths=arabidopsis_chromosomes)

## Get an example file in Methylpy format
file <- system.file("extdata","arabidopsis_methylpy.txt", package="methimpute")
data(arabidopsis_chromosomes)
arabidopsis_chromosomes$chromosome <- sub('chr', '', arabidopsis_chromosomes$chromosome)
methylpy.data <- importMethylpy(file, chrom.lengths=arabidopsis_chromosomes)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/correlation.R
\name{estimateTransDist}
\alias{estimateTransDist}
\title{\code{transDist} parameter}
\usage{
estimateTransDist(distcor, skip = 2, plot.parameters = TRUE)
}
\arguments{
\item{distcor}{The output produced by \code{\link{distanceCorrelation}}.}

\item{skip}{Skip the first n cytosines for the fitting. This can be necessary to avoid periodicity artifacts due to the context definition.}

\item{plot.parameters}{Whether to plot fitted parameters on to the plot or not.}
}
\value{
A list() with fitted \code{transDist} parameters and the corresponding \code{\link[ggplot2]{ggplot}}.
}
\description{
Obtain an estimate for the \code{transDist} parameter (used in function \code{\link{callMethylation}}) by fitting an exponential function to the supplied correlations (from \code{\link{distanceCorrelation}}).
}
\examples{
## Get some toy data
file <- system.file("data","arabidopsis_toydata.RData",
                    package="methimpute")
data <- get(load(file))
distcor <- distanceCorrelation(data)
fit <- estimateTransDist(distcor)
print(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loadFromFiles.R
\name{loadFromFiles}
\alias{loadFromFiles}
\title{Load \pkg{methimpute} objects from file}
\usage{
loadFromFiles(files, check.class = c("GRanges", "methimputeBinomialHMM"))
}
\arguments{
\item{files}{A list of \code{\link{GRanges-class}} or \code{\link{methimputeBinomialHMM}} objects or a character vector with files that contain such objects.}

\item{check.class}{Any combination of \code{c('GRanges', 'methimputeBinomialHMM')}. If any of the loaded objects does not belong to the specified class, an error is thrown.}
}
\value{
A list of \code{\link{GRanges-class}} or \code{\link{methimputeBinomialHMM}} objects.
}
\description{
Wrapper to load \pkg{\link{methimpute}} objects from file and check the class of the loaded objects.
}
\examples{
## Get some files that you want to load
file <- system.file("data","arabidopsis_toydata.RData",
                    package="methimpute")
## Load and print
data <- loadFromFiles(file)
print(data)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getPosteriors.R
\name{getPosteriors}
\alias{getPosteriors}
\title{Get original posteriors}
\usage{
getPosteriors(data)
}
\arguments{
\item{data}{The \code{$data} entry from a \code{\link{methimputeBinomialHMM}} object.}
}
\value{
A matrix with posteriors.
}
\description{
Transform the 'posteriorMeth', 'posteriorMax', and 'status' columns into original posteriors from the HMM.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documented_objects.R
\name{methimputeBinomialHMM}
\alias{methimputeBinomialHMM}
\title{methimputeBinomialHMM}
\value{
A list() with the following entries:
\item{convergenceInfo}{A list() with information about the convergence of the model fitting procedure.}
\item{params}{A list() with fitted and non-fitted model parameters.}
\item{params.initial}{A list() with initial values for the model parameters.}
\item{data}{A \code{\link[GenomicRanges]{GRanges-class}} with cytosine positions and methylation status calls.}
\item{segments}{The \code{data} entry where coordinates of consecutive cytosines with the same methylation status have been merged.}
}
\description{
The \code{methimputeBinomialHMM} is a list() which contains various entries (see Value section). The main entry of this object is \code{$data}, which contains the methylation status calls and posterior values. See Details for a description of all columns.
}
\details{
The \code{$data} entry in this object contains the following columns:
\itemize{
\item{context }{The sequence context of the cytosine.}
\item{counts }{Counts for methylated and total number of reads at each position.}
\item{distance }{The distance in base-pairs from the previous to the current cytosine.}
\item{transitionContext }{Transition context in the form "previous-current".}
\item{posteriorMax }{Maximum posterior value of the methylation status call, can be interpreted as the confidence in the call.}
\item{posteriorMeth }{Posterior value of the "methylated" component.}
\item{posteriorUnmeth }{Posterior value of the "unmethylated" component.}
\item{status }{Methylation status.}
\item{rc.meth.lvl }{Recalibrated methylation level, calculated as \code{r$data$rc.meth.lvl = r$data$params$emissionParams$Unmethylated[data$context,] * r$data$posteriorUnmeth + r$params$emissionParams$Methylated[data$context,] * r$data$posteriorMeth}, where \code{r} is the \code{methimputeBinomialHMM} object.}
}
}
\seealso{
\code{\link{methimpute-objects}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extractCytosinesFromFASTA.R
\name{extractCytosinesFromFASTA}
\alias{extractCytosinesFromFASTA}
\title{Extract cytosine coordinates}
\usage{
extractCytosinesFromFASTA(file, contexts = c("CG", "CHG", "CHH"),
  anchor.C = NULL)
}
\arguments{
\item{file}{A character with the file name.}

\item{contexts}{The contexts that should be extracted. If the contexts are named, the returned object will use those names for the contexts.}

\item{anchor.C}{A named vector with positions of the anchoring C in the \code{contexts}. This is necessary to distinguish contexts such as C*C*CG (anchor.C = 2) and *C*CCG (anchor.C = 1). Names must match the contexts. If unspecified, the first C within each context will be taken as anchor.}
}
\value{
A \code{\link{GRanges-class}} object with coordinates of extracted cytosines and meta-data column 'context'.
}
\description{
Extract cytosine coordinates and context information from a FASTA file. Cytosines in ambiguous reference contexts are not reported.
}
\examples{
## Read a non-compressed FASTA files:
filepath <- system.file("extdata", "arabidopsis_sequence.fa.gz", package="methimpute")

## Only CG context
cytosines <- extractCytosinesFromFASTA(filepath, contexts = 'CG')
table(cytosines$context)

## Split CG context into subcontexts
cytosines <- extractCytosinesFromFASTA(filepath,
               contexts = c('DCG', 'CCG'),
               anchor.C = c(DCG=2, CCG=2))
table(cytosines$context)
               
## With contexts that differ only by anchor
cytosines <- extractCytosinesFromFASTA(filepath,
               contexts = c('DCG', 'CCG', 'CCG', 'CWG', 'CHH'),
               anchor.C = c(DCG=2, CCG=2, CCG=1, CWG=1, CHH=1))
table(cytosines$context)
               
## With named contexts
contexts <- c(CG='DCG', CG='CCG', CHG='CCG', CHG='CWG', CHH='CHH')
cytosines <- extractCytosinesFromFASTA(filepath,
               contexts = contexts,
               anchor.C = c(DCG=2, CCG=2, CCG=1, CWG=1, CHH=1))
table(cytosines$context)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methimpute-package.R
\docType{package}
\name{methimpute-package}
\alias{methimpute-package}
\alias{methimpute}
\title{methIMPUTE: Imputation-guided methylation status calling for WGBS-seq}
\description{
\pkg{methimpute} is an R-package for methylation status calling in Whole-Genome Bisulfite-sequencing (WGBS-seq) data. Its powerful Hidden Markov model implementation enables imputation of methylation status calls for cytosines without any coverage.
}
\details{
Please read the vignette for a tutorial on how to use this package. You can do this by typing \code{browseVignettes("methimpute")}. Here is an overview of all \code{\link{plotting}} functions.
}
\author{
Aaron Taudt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documented_objects.R
\docType{data}
\name{arabidopsis_genes}
\alias{arabidopsis_genes}
\title{Gene coordinates for Arabidopsis (chr1)}
\format{A \code{\link[GenomicRanges]{GRanges-class}} object.}
\description{
A \code{\link[GenomicRanges]{GRanges-class}} object for demonstration purposes in examples of package \pkg{\link{methimpute}}. The object contains gene coordinates of chr1 from Arabidopsis.
}
\examples{
data(arabidopsis_genes)
print(arabidopsis_genes)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documented_objects.R
\name{methimputeData}
\alias{methimputeData}
\title{methimputeData}
\description{
A \code{\link[GenomicRanges]{GRanges-class}} object containing cytosine coordinates with meta-data columns 'context' and 'counts'.
}
\seealso{
\code{\link{methimpute-objects}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plotting}
\alias{plotting}
\alias{plotHistogram}
\alias{plotScatter}
\alias{plotTransitionProbs}
\alias{plotConvergence}
\alias{plotEnrichment}
\alias{plotPosteriorDistance}
\title{Methimpute plotting functions}
\usage{
plotHistogram(model, total.counts, binwidth = 1)

plotScatter(model, datapoints = 1000)

plotTransitionProbs(model)

plotConvergence(model)

plotEnrichment(model, annotation, windowsize = 100, insidewindows = 20,
  range = 1000, category.column = NULL, plot = TRUE,
  df.list = NULL)

plotPosteriorDistance(model, datapoints = 1e+06, binwidth = 5,
  max.coverage.y = 0, min.coverage.x = 3, xmax = 200,
  xbreaks.interval = xmax/10, cutoffs = NULL)
}
\arguments{
\item{model}{A \code{\link{methimputeBinomialHMM}} object.}

\item{total.counts}{The number of total counts for which the histogram is to be plotted.}

\item{binwidth}{The bin width for the histogram/boxplot.}

\item{datapoints}{The number of randomly selected datapoints for the plot.}

\item{annotation}{A \code{\link[GenomicRanges]{GRanges-class}} object with coordinates for the annotation.}

\item{windowsize}{Resolution in base-pairs for the curve upstream and downstream of the annotation.}

\item{insidewindows}{Number of data points for the curve inside the annotation.}

\item{range}{Distance upstream and downstream for which the enrichment profile is calculated.}

\item{category.column}{The name of a column in \code{data} that will be used for facetting of the plot.}

\item{plot}{Logical indicating whether a plot or the underlying data.frame is to be returned.}

\item{df.list}{A list() of data.frames, output from \code{plotEnrichment(..., plot=FALSE)}. If specified, option \code{data} will be ignored.}

\item{max.coverage.y}{Maximum coverage for positions on the y-axis.}

\item{min.coverage.x}{Minimum coverage for positions on the x-axis.}

\item{xmax}{Upper limit for the x-axis.}

\item{xbreaks.interval}{Interval for breaks on the x-axis.}

\item{cutoffs}{A vector with values that are plotted as horizontal lines. The names of the vector must match the context levels in \code{data$context}.}
}
\value{
A \code{\link[ggplot2]{ggplot}} object.
}
\description{
This page provides an overview of all \pkg{\link{methimpute}} plotting functions.
}
\section{Functions}{
\itemize{
\item \code{plotHistogram}: Plot a histogram of count values and fitted distributions.

\item \code{plotScatter}: Plot a scatter plot of read counts colored by methylation status.

\item \code{plotTransitionProbs}: Plot a heatmap of transition probabilities.

\item \code{plotConvergence}: Plot the convergence of the probability parameters.

\item \code{plotEnrichment}: Plot an enrichment profile around an annotation.

\item \code{plotPosteriorDistance}: Maximum posterior vs. distance to nearest covered cytosine.
}}

\examples{
## Get some toy data
file <- system.file("data","arabidopsis_toydata.RData",
                    package="methimpute")
data <- get(load(file))
print(data)
model <- callMethylation(data)
## Make nice plots
plotHistogram(model, total.counts=5)
plotScatter(model)
plotTransitionProbs(model)
plotConvergence(model)
plotPosteriorDistance(model$data)

## Get annotation data and make an enrichment profile
# Note that this looks a bit ugly because our toy data
# has only 200000 datapoints.
data(arabidopsis_genes)
plotEnrichment(model, annotation=arabidopsis_genes)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documented_objects.R
\docType{data}
\name{arabidopsis_TEs}
\alias{arabidopsis_TEs}
\title{Transposable element coordinates for Arabidopsis (chr1)}
\format{A \code{\link[GenomicRanges]{GRanges-class}} object.}
\description{
A \code{\link[GenomicRanges]{GRanges-class}} object for demonstration purposes in examples of package \pkg{\link{methimpute}}. The object contains transposable element coordinates of chr1 from Arabidopsis.
}
\examples{
data(arabidopsis_TEs)
print(arabidopsis_TEs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/importData.R
\name{importRene}
\alias{importRene}
\title{Import a Rene methylation extractor file}
\usage{
importRene(file, chrom.lengths = NULL, skip = 1)
}
\arguments{
\item{file}{The file to import.}

\item{chrom.lengths}{A data.frame with chromosome names in the first, and chromosome lengths in the second column. Only chromosomes named in here will be returned. Alternatively a tab-separated file with such a data.frame (with headers).}

\item{skip}{The number of lines to skip. Usually 1 if the file contains a header and 0 otherwise.}
}
\value{
A \code{\link{methimputeData}} object.
}
\description{
Import a Rene methylation extractor file into a \code{\link[GenomicRanges]{GRanges-class}} object.
}
\examples{
## Get an example file in Rene format
file <- system.file("extdata","arabidopsis_rene.txt", package="methimpute")
data(arabidopsis_chromosomes)
rene.data <- methimpute:::importRene(file, chrom.lengths=arabidopsis_chromosomes)

}
